//DibFuncs.cpp
#include "stdafx.h"
#include "ImageFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WIDTHBYTES(bits)    (((bits) + 31) / 32 * 4)

//GetDib()////////////////////////////////////////////////////////////
//	This function creates a new DIB based on the info in the class
//////////////////////////////////////////////////////////////////////
HDIB ImageFile::GetImageDib()
{
	HDIB 				hDIB;
	BITMAPINFOHEADER    bi;             // bitmap header
	LPBITMAPINFOHEADER  lpBMIH;			// pointer to BITMAPINFOHEADER
	DWORD               dwLen;          // size of memory block
	DWORD               dwBytesPerLine; // Number of bytes per scanline
	LPSTR     			lpBits;			//pointer to the bits of DIB

	//fill in the fields of the bitmap info header
	bi.biSize = sizeof(BITMAPINFOHEADER);
	bi.biWidth = image_width;
	bi.biHeight = image_height;
    bi.biPlanes = 1;
    bi.biBitCount = 24;			//true color bit depth
    bi.biCompression = BI_RGB;	//no compression
    bi.biXPelsPerMeter = 0;
    bi.biYPelsPerMeter = 0;
    bi.biClrUsed = 0;
    bi.biClrImportant = 0;

    // calculate size of memory block required to store the DIB.  This 
    // block should be big enough to hold the BITMAPINFOHEADER, the color 
    // table, and the bits 
    dwBytesPerLine = WIDTHBYTES(bi.biBitCount * image_width);
    dwLen = bi.biSize + (dwBytesPerLine * image_height); 

	//fill in the total image size in bytes field 
 	bi.biSizeImage = dwBytesPerLine * image_height;

    // alloc memory block to store our bitmap 
    hDIB = ::GlobalAlloc(GHND, dwLen); 

    // if we couldn't get memory block 
    if (!hDIB) 
        return NULL;

    // lock memory and get pointer to it 
    lpBMIH = (LPBITMAPINFOHEADER) ::GlobalLock(hDIB); 

    // use our bitmap info structure to fill in first part of 
    // our DIB with the BITMAPINFOHEADER 
    *lpBMIH = bi; 

 	//Align = Number of byte to add at the end of each line of the DIB
    int Align = 4 - (dwBytesPerLine % 4);
    if (Align == 4)	
    	Align = 0;

 	//get the pointer to the bits of the dib
	lpBits = (LPSTR)lpBMIH + (WORD)lpBMIH->biSize;
	lpBits += ((image_width * RGBSample) + Align) * (image_height - 1);
	//now lpBits point on the bottom line

 	//convert the image to bottom-top and RGB -> BGR as per DIB
 	for (int row = 0; row < image_height; row++) {
 		
 		//if our image is in RGB color
 		if (SamplePerPixel == RGBSample) {
 			for (int col = 0; col < image_width * RGBSample; col += 3) {
    
           		lpBits[col+0] = pImage[row][col+2];	  //copy the color components
           		lpBits[col+1] = pImage[row][col+1];
           		lpBits[col+2] = pImage[row][col+0];
       		}
		}
		//otherwise it is grayscale
		else {
			for (int col = 0, colg = 0; col < image_width * RGBSample; col += 3, colg++ ) {
    
         		lpBits[col+0] = pImage[row][colg];	  //copy the color components
           		lpBits[col+1] = pImage[row][colg];
           		lpBits[col+2] = pImage[row][colg];
       		}
		}				
		//decrement the pointer to point to the next DWORD aligned 
		// row of the DIB 
		lpBits -= image_width * RGBSample + Align;
	}

    // Since we don't know what bits should contain, 
    // just leave that blank. Unlock the DIB and return the HDIB. 
    ::GlobalUnlock(hDIB); 

    //return handle to the DIB 
    return hDIB; 
}

//static functions for the 16 bit version of the DIB
static int GetLeftShiftCount(DWORD dwVal);
static int GetRightShiftCount(DWORD dwVal);


////////////////////////////////////////////////////////////////
//This function fills a pImage with the RGB values from a specified bitmap
//
//It has been designed to handle all legal bitmap formats.
//It handles RLE compressed files
//Note that rows in a bitmap are processed from bottom to top, they
//are inverted for our image. Also you'll need twice as much memory as 
//the image for this routine to work since it loads a whole bitmap first
////////////////////////////////////////////////////////////////
BOOL ImageFile::LoadDib()
{
    DWORD rmask, gmask, bmask;
    int rright, gright, bright;    
    int rleft, gleft, bleft;    
    BYTE* pbBits;
    WORD* pwBits;    
    DWORD* pdwBits;    
    WORD wColor;
    DWORD dwColor;    
    HDC hdc;    
    BYTE* pBuffer;    
    BITMAPINFO bmi;
    DIBSECTION ds;    
	HDIB hImage;
	unsigned char *rowPtr;
	int row, col;


	// try... catch block
	try {
	
	hImage = NULL;     //initialize handle to bitmap
	
    // Use LoadImage() to get the image loaded into a DIBSection
    hImage = (HDIB) ::LoadImage(NULL, filename, IMAGE_BITMAP, 0, 0,
                 LR_CREATEDIBSECTION | LR_LOADFROMFILE);

	//if the handle is NULL     
    if(hImage == NULL)
    	throw new CException;
    	
    //get the dibsection from the handle to bitmap
    ::GetObject(hImage, sizeof(ds), &ds);
    
    int nPad = ds.dsBm.bmWidthBytes - (((ds.dsBmih.biWidth *
        ds.dsBmih.biBitCount) + 7) / 8);

	//assign our ImageFile class members
    image_width = ds.dsBmih.biWidth;
    image_height = ds.dsBmih.biHeight;
	SamplePerPixel = RGBSample;		//it will allways end up as 24 bit RGB
    	
    if (!AllocateImage())		//allocate our image
		throw new CException;
    	
	            
    switch (ds.dsBmih.biBitCount) {
    
    case 1: // 1-bit DIB    
    case 4: // 4-bit DIB    
    case 8: // 8-bit DIB
		//
        // The strategy here is to use ::GetDIBits to convert the
        // image into a 24-bit DIB one scan line at a time. A pleasant
        // side effect of using ::GetDIBits in this manner is that RLE-
        // encoded 4-bit and 8-bit DIBs will be uncompressed.
        //
		pBuffer = new BYTE[ds.dsBmih.biWidth * RGBSample];
		
		//did we get the memory?
        if (pBuffer == NULL)
			throw new CException;
		
		hdc = ::GetDC(NULL);      //get the DC that we will use to get the rows  
		
		::ZeroMemory(&bmi, sizeof(bmi));
        bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
        bmi.bmiHeader.biWidth = ds.dsBmih.biWidth;
        bmi.bmiHeader.biHeight = ds.dsBmih.biHeight;
        bmi.bmiHeader.biPlanes = 1;
        bmi.bmiHeader.biBitCount = 24;
        bmi.bmiHeader.biCompression = BI_RGB;
        
		for (row = 0; row < ds.dsBmih.biHeight; row++) {
           
			rowPtr = pImage[(image_height-1) - row];		//invert rows for our image
			::GetDIBits(hdc, (HBITMAP) hImage, row, 1, pBuffer, &bmi, DIB_RGB_COLORS);
            
			for (col = 0; col < ds.dsBmih.biWidth * RGBSample; col+=3) {
                rowPtr[col+2] = pBuffer[col+0];
				rowPtr[col+1] = pBuffer[col+1];
				rowPtr[col+0] = pBuffer[col+2];
			}
        }
        delete [] pBuffer;
        ::ReleaseDC(NULL, hdc);
        break;
    
    case 16: // 16-bit DIB

    	if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];            
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];        
        }
        else {
            rmask = 0x7C00;            
            gmask = 0x03E0;
            bmask = 0x001F;        
        } 
        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);
        rleft = GetLeftShiftCount (rmask);
        gleft = GetLeftShiftCount (gmask);
        bleft = GetLeftShiftCount (bmask);
        pwBits = (WORD*) ds.dsBm.bmBits;
      
        for (row = 0; row < ds.dsBmih.biHeight; row++) {
        
			rowPtr = pImage[(image_height-1) - row];
 
            for (col = 0; col < ds.dsBmih.biWidth * RGBSample; col+=3) {
                wColor = *pwBits++;
                rowPtr[col+2] = (BYTE) (((wColor & (WORD) bmask) >> bright) << bleft);
                rowPtr[col+1] = (BYTE) (((wColor & (WORD) gmask) >> gright) << gleft);
                rowPtr[col+0] = (BYTE) (((wColor & (WORD) rmask) >> rright) << rleft);
            }
            pwBits = (WORD*) (((BYTE*) pwBits) + nPad); 
        }
        break;    
    
    case 24: // 24-bit DIB        
		
    	pbBits = (BYTE*) ds.dsBm.bmBits;
        for (row = 0; row < ds.dsBmih.biHeight; row++) {
        	
        	rowPtr = pImage[(image_height-1) - row];
        	
            for (col = 0; col < ds.dsBmih.biWidth * RGBSample; col+=3) {                
            	rowPtr[col+2] = *pbBits++;
                rowPtr[col+1] = *pbBits++;                
                rowPtr[col+0] = *pbBits++;
            }            
            pbBits += nPad;        
        }
        break;
    
    case 32: // 32-bit DIB        
    	
    	if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];            
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];        
        }
        else {
            rmask = 0x00FF0000;            
            gmask = 0x0000FF00;
            bmask = 0x000000FF;        
        }
        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);
        pdwBits = (DWORD*) ds.dsBm.bmBits;
        for (row = 0; row < ds.dsBmih.biHeight; row++) {
            
            rowPtr = pImage[(image_height-1) - row];
            
            for (col = 0; col < ds.dsBmih.biWidth * RGBSample; col+=3) {
                dwColor = *pdwBits++;
                rowPtr[col+2] = (BYTE) ((dwColor & bmask) >> bright);
                rowPtr[col+1] = (BYTE) ((dwColor & gmask) >> gright);
                rowPtr[col+0] = (BYTE) ((dwColor & rmask) >> rright);
            }
            pdwBits = (DWORD*) (((BYTE*) pdwBits) + nPad);        
        }
        break;    
    default: // Unrecognized color format
    	throw new CException;
   		break;
   		
 	} // end switch
 	} // end try
	catch(CException* pe) {
		CString pcsMsg = "Error reading ";
		pcsMsg += filename;
		AfxMessageBox(pcsMsg);
		::DeleteObject(hImage);
		pe->Delete();
		return FALSE;
	}
	
	//no errors occured
	::DeleteObject(hImage);
	return TRUE;
}	
	

static int GetLeftShiftCount(DWORD dwVal)
{  
	int nCount = 0;
    for (int i = 0; i < sizeof(DWORD) * 8; i++) {
        if (dwVal & 1)
            nCount++;
        dwVal >>= 1;
    }
    return (8 - nCount);
}

static int GetRightShiftCount(DWORD dwVal)
{
    for (int i = 0; i < sizeof(DWORD) * 8; i++) {
        if (dwVal & 1)
            return i;
        dwVal >>= 1;
    
	}
    return -1;
}


//////
//SaveDib()/////////////////////////////////////////////////////
//	This function stores the contents of pImage in a DIB file
////////////////////////////////////////////////////////////////
BOOL ImageFile::SaveDib()
{
	BITMAPFILEHEADER 	bmfh;
	LPSTR 				lpRowBits;		// pointer to one row
	BITMAPINFOHEADER    bi;             // bitmap header
	DWORD               dwBytesPerLine; // Number of bytes per scanline


	try {	

	//open the file
	CFile file(filename, CFile::modeReadWrite | CFile::modeCreate);

	//fill in the fields of the bitmap info header
	bi.biSize = sizeof(BITMAPINFOHEADER);
	bi.biWidth = image_width;
	bi.biHeight = image_height;
    bi.biPlanes = 1;
    bi.biBitCount = 24;			//true color bit depth
    bi.biCompression = BI_RGB;	//no compression
    bi.biXPelsPerMeter = 0;
    bi.biYPelsPerMeter = 0;
    bi.biClrUsed = 0;
    bi.biClrImportant = 0;

    // calculate size of memory block required to store the DIB.  This 
    // block should be big enough to hold the BITMAPINFOHEADER, the color 
    // table, and the bits 
    dwBytesPerLine = WIDTHBYTES(bi.biBitCount * image_width);

	//fill in the total image size in bytes field 
 	bi.biSizeImage = dwBytesPerLine * image_height;
	
	//allocate space for one row
	lpRowBits = (char *) ::GlobalAlloc(GMEM_FIXED, dwBytesPerLine);

    // if we couldn't get memory block 
    if (!lpRowBits) 
        throw new CFileException; 

	//fill in the fields of the bitmap file header
	bmfh.bfType = 0x4d42;
	bmfh.bfSize = 0;
	bmfh.bfReserved1 = bmfh.bfReserved1 = 0;
	//no palette factor since we will only save 24bit DIBs
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
		
	//write the header to the file
	file.Write((LPVOID) &bmfh, sizeof(BITMAPFILEHEADER));
	file.Write((LPVOID) &bi, sizeof(BITMAPINFOHEADER));
	
 	//index the rows to bottom-top and RGB -> BGR as per DIB
 	for (int row = image_height-1; row >= 0; row--) {
 		
 		//if our image is in RGB color
 		if (SamplePerPixel == RGBSample) {
 			for (int col = 0; col < image_width * RGBSample; col += 3) {
    
           		lpRowBits[col+0] = pImage[row][col+2];	  //copy the color components
           		lpRowBits[col+1] = pImage[row][col+1];
           		lpRowBits[col+2] = pImage[row][col+0];
       		}
		}
		//otherwise it is grayscale
		else {
			for (int col = 0, colg = 0; col < image_width * RGBSample; col += 3, colg++ ) {
    
         		lpRowBits[col+0] = pImage[row][colg];	  //copy the color components
           		lpRowBits[col+1] = pImage[row][colg];
           		lpRowBits[col+2] = pImage[row][colg];
       		}
		}				

		// write the row to file	
		file.Write((LPVOID) lpRowBits, dwBytesPerLine);

	} // end for
	} // end try
	catch (CFileException * pe) {
		pe->Delete();
		::GlobalFree((HGLOBAL) lpRowBits);	//free the space occupied by the row
		CString pcsMsg = "Error: Can't save ";
		pcsMsg += filename;
		AfxMessageBox(pcsMsg);
		return FALSE;
	}

	//everything was ok
	::GlobalFree((HGLOBAL) lpRowBits);	//free the space occupied by the row
	return TRUE;
}

