//ImageFile.cpp
#include "stdafx.h"
#include "ImageFile.h"
#include "Dialogs.h"
#include <string.h>
#include <memory.h>
#include <Dlgs.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//the hook function for the save file dialog
UINT APIENTRY OFNHookProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

static const char szFilters[] = 
"BMP (*.BMP, *.DIB)|*.BMP;*.DIB|JPEG (*.JPG, *.JPE)|*.JPG;*.JPE|Targa (*.TGA, *.VDA, *.ICB, *.VST, *.WIN)|*.TGA;*.VDA;*.ICB;*.VST;*.WIN|TIFF (*.TIF)|*.TIF|"; //All Files (*.*)|*.*||";


//constructor/////////////////////////////////////
ImageFile::ImageFile(BOOL openFile)
{
	filename = NULL;
	pImage = NULL;
	image_height = 0;
	image_width = 0;
	SamplePerPixel = 0;
	currnt_row = 0;
	imageLoaded = FALSE;

	//open file right away?
	if (openFile)
		imageLoaded = OpenFile();	//did we load the image?
}

//destructor////////////////////////////////////////
ImageFile::~ImageFile()
{
	if (filename)
		delete [] filename;
	DeallocateImage();
}

//Reset()///////////////////////////////////////////
//if you want to delete all of the members in the class
//this is called from the beginning of every load function
// nSamples can be either GRAYSample or RGBSample
void ImageFile::Reset(int new_height /* =0*/, int new_width /* =0*/, int nSamples /* =0*/)
{
	//get rid of the occupied image space
	DeallocateImage();

	//reset the dimensions and attributes
	if (new_height < 0)
		new_height = -(new_height);
	
	image_height = new_height;

	//get absolute value of new_width
	if (new_width < 0)
		new_width = -(new_width);

	image_width = new_width; //assign the width of our row

	//if number of samples is not zero .. the user wants to create
	//his own image
	if (nSamples != 0)
		SamplePerPixel = (nSamples == GRAYSample) ? GRAYSample : RGBSample;
	else
		SamplePerPixel = 0;

	//allocate the image if the user wants to..
	if ((image_height != 0 && image_width != 0 && SamplePerPixel != 0) && AllocateImage()) {
		
		for (int i = 0; i < image_height; i++)
			::ZeroMemory(pImage[i], image_width * SamplePerPixel); //zero out rows
	
		imageLoaded = TRUE;
	}
	else {
		image_height = 0;		//otherwise reset everything
		image_width = 0;
		SamplePerPixel = 0;
		currnt_row = 0;
		imageLoaded = FALSE;
	}
}

//AllocateImage()///////////////////////////////////
//	This function allocates the space for the image
//	image_wisth and image_height must already be assigned
////////////////////////////////////////////////////
BOOL ImageFile::AllocateImage()
{	
	//sanity check, all must be non-zero
	if (SamplePerPixel == 0 ||
		image_width == 0 ||
		image_height == 0)
		return FALSE;

	//allocate pointers to rows
    if ((pImage = new unsigned char*[image_height]) == NULL)
		return FALSE;

    //allocate rows of the image
    for (int i = 0; i < image_height; i++) {
        if ((pImage[i] = new unsigned char[image_width * SamplePerPixel]) == NULL) {
            //new.. deallocate all of the allocated rows.. safest way
			int j = i;
			while (--j <= 0)
				delete pImage[j];

			delete [] pImage; //delete the pointer to rows
			pImage = NULL;	  //reset pImage
            return FALSE;
        }
	}
    return TRUE;
}

//DeallocateImage()/////////////////////////////////
// 	This function deletes the image from the heap
////////////////////////////////////////////////////
void ImageFile::DeallocateImage()
{
	if (pImage != NULL){
    	for(int i = 0; i < image_height; i++)
      		delete pImage[i];
    	
    	delete [] pImage;
    	pImage = NULL;	//reset pImage
  	}
}

/*
RGBColor ImageFile::GetRGBColorPixel(int x, int y)
{
	RGBColor rgb(0);
	unsigned char* ptr;
  	
  	//make sure input is valid and tile the image if necessary
  	if (x < 0)  
  		x = image_width - (abs(x) % image_width);
  	else 
  		x = x % image_width;
  	if (y < 0)  
  		y = image_height - (abs(y) % image_height);
  	else 
  		y = y % image_height;
  	
  	ptr = pImage[y];	//point ptr to the row in question
  	ptr += x * SamplePerPixel;	//offset the position to the color in the row
   	
   	//if the image is RGB
   	if (SamplePerPixel == 3) {
   		rgb[0] = ptr[0] / 255.0; // convert the byte values to float
  		rgb[1] = ptr[1] / 255.0;
  		rgb[2] = ptr[2] / 255.0;
	}
	//otherwise it is grayscale
	else
		rgb = RGBColor(*ptr / 255.0)
	
	return rgb;
}
*/


//PutScanline()///////////////////////////////////////////////
//Used for adding scanlines to the image from the outside
//pBuffer must be as long as image_width*SamplePerPixel
BOOL ImageFile::PutScanline(unsigned char* pBuffer)
{
	//sanity check
	if (SamplePerPixel == 0 || image_width == 0 ||
		image_height == 0 || imageLoaded == FALSE ||
		currnt_row >= image_height)
		return FALSE;

	//move the memory from the buffer to our image scanline
	::MoveMemory(pImage[currnt_row], pBuffer, image_width * SamplePerPixel);
	currnt_row++;

	return TRUE;
}

/*
BOOL ImageFile::PutRGBColorRow(RGBColor* pBuffer)
{
	int col_index;

	//sanity check
	if (SamplePerPixel == 0 || image_width == 0 ||
		image_height == 0 || imageLoaded == FALSE ||
		currnt_row >= image_height)
		return FALSE;

	unsigned char* rowPtr = pImage[currnt_row];

	//copy the color components from the RGBColor array to our row
	for (int i = 0; i < image_width; i++) {
		if (SamplePerPixel == RGBSample) {
			col_index = RGBSample * i;
			rowPtr[col_index+0] = pBuffer[i][0] * 255.0;
			rowPtr[col_index+1] = pBuffer[i][1] * 255.0;
			rowPtr[col_index+2] = pBuffer[i][2] * 255.0;
		}
		else
			rowPtr[i] = pBuffer[i].GetGray() * 255.0;
	}
	currnt_row++;

	return TRUE;
}
*/


//OpenFile()////////////////////////////////////////////////
//Opens a file dialog, and opens the file based 
//on the file type.
BOOL ImageFile::OpenFile()
{
	CString filters = szFilters;
	filters += "All Files (*.*)|*.*||";
	//create an Open file dialog
	CFileDialog openFileDlg(TRUE, NULL, NULL, 
		OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
		OFN_FILEMUSTEXIST, filters);

	//initial filter index is "All Files"
	openFileDlg.m_ofn.nFilterIndex = 5;

	//show the dialog
	if (openFileDlg.DoModal() != IDOK)
		return FALSE;

	//get the extention and file name
	CString fileExt = openFileDlg.GetFileExt();
	CString file_name = openFileDlg.GetPathName();

	//if there is already a filename .. delete it
	char *pStr = filename;
	if (pStr) {
 		delete [] pStr;
		filename = NULL;
	}	
	
	int nameLen = file_name.GetLength();
	filename = new char[nameLen+1];
	strcpy(filename, file_name);
	bool error = false;

	//make all characters upper case so we can compare them
	//to other extentions
	fileExt.MakeUpper();  

	//compare the file extention to known types and load 
	//the image.
	if (fileExt == "JPG" || fileExt == "JPE") {
		Reset();
		if (!LoadJpeg())
			error = true;
	}
	else if (fileExt == "TIF") {
		Reset();		//reset the image since we're about to open a file
		if (!LoadTiff())
			error = true;
	}
	else if (fileExt == "BMP" || fileExt == "DIB") {
		Reset();
		if (!LoadDib())
			error = true;
	}
	else if (fileExt == "TGA" || fileExt == "VDA"||
			 fileExt == "ICB" || fileExt == "VST"||
			 fileExt == "WIN") {
		Reset();
		if (!LoadTga())
			error = true;
	}
	else {
		CString pcsMsg = "Sorry, No provision for ";
		pcsMsg += fileExt + " file type.";
		AfxMessageBox(pcsMsg);
		return FALSE;
	}

	if (error) {
		Reset();	//couldn't load.. reset everything
		return FALSE;
	}
	
	//if user tries to add a row to our image..he can't
	currnt_row = image_height;
	imageLoaded = TRUE;
	return TRUE;
}

//SaveFile()///////////////////////////////////////////////
//Opens a file dialog and saves the specified file
BOOL ImageFile::SaveFile()
{
	//does the our image exist 
	if (!imageLoaded)
		return FALSE;		

	//create an Open file dialog
	CFileDialog saveFileDlg(FALSE, NULL, NULL, 
		OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
		OFN_ENABLEHOOK | OFN_EXPLORER, szFilters);

	//enable our hook procedure
	saveFileDlg.m_ofn.lpfnHook = (LPOFNHOOKPROC) OFNHookProc;

	//show the dialog
	if (saveFileDlg.DoModal() != IDOK)
		return FALSE;

	//get the extention and file name
	CString fileExt = saveFileDlg.GetFileExt();
	CString file_name = saveFileDlg.GetPathName();

	//if there is already a filename .. delete it
	char *pStr = filename;
	if (pStr) {
 		delete [] pStr;
		filename = NULL;
	}	
	
	int nameLen = file_name.GetLength();
	filename = new char[nameLen+1];
	strcpy(filename, file_name);
	bool error = false;

	//make all characters upper case so we can compare them
	//to other extentions
	fileExt.MakeUpper();  

	//compare the file extention to known types and load 
	//the image.
	if (fileExt == "JPG" || fileExt == "JPE") {
		CJpegDlg jpegDlg;
		jpegDlg.m_Mode = (SamplePerPixel == 3) ? 1 : 0;
		if (jpegDlg.DoModal() == IDOK) {
			if (!SaveJpeg(jpegDlg.m_Quality * 10, jpegDlg.m_Mode))		//quality = 0..100
				error = true;
		}
	}
	else if (fileExt == "TIF") {
		CTiffDlg tiffDlg;
		tiffDlg.m_Mode = (SamplePerPixel == 3) ? 1 : 0;
		if (tiffDlg.DoModal() == IDOK) {
			if (!SaveTiff(tiffDlg.m_Mode))
				error = true;
		}
	}
	else if (fileExt == "BMP" || fileExt == "DIB") {
		if (!SaveDib())
			error = true;
	}
	else if (fileExt == "TGA" || fileExt == "VDA"||
			 fileExt == "ICB" || fileExt == "VST"||
			 fileExt == "WIN") {
		CTgaDlg tgaDlg;
		tgaDlg.m_Mode = (SamplePerPixel == 3) ? 1 : 0;
		if (tgaDlg.DoModal() == IDOK) {
			if (!SaveTga(tgaDlg.m_Mode))
				error = true;
		}
	}
	else {
		CString pcsMsg = "Sorry, No provision for ";
		pcsMsg += fileExt + " file type.";
		AfxMessageBox(pcsMsg);
		error = true;
	}

	if (error)
		return FALSE;
		
	return TRUE;
}


//OFNHookProc()//////////////////////////////////////////////////////////
// A hook procedure for the Save As dialog. There is mainly a lot 
// of system oriented error handling here to reject bad filenames 
UINT APIENTRY OFNHookProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_NOTIFY) {	//only accept WM_NOTIFY messages

	HWND hpDlg = GetParent(hDlg);	//get a handle to the parent dialog

	//only respond when the user changes a file type from the combo box
	//or when the OK button is pressed
	if (((LPOFNOTIFY) lParam)->hdr.code == CDN_TYPECHANGE ||
		((LPOFNOTIFY) lParam)->hdr.code == CDN_FILEOK) {

		CString fileName;
		bool nameChange = false;	//if the file name was changed by our procedure
		int filterIndex = ((LPOFNOTIFY) lParam)->lpOFN->nFilterIndex;	//current type
		CString ext;		//filename extention
		int extLen = 0;		//length of the extention
		int pos;			//position of '.' in the string

		//get the test from the edit control in the dialog
		GetDlgItemText(hpDlg, edt1, fileName.GetBuffer(MAX_PATH), MAX_PATH);
		fileName.ReleaseBuffer();

		//are there any leading white spaces
		CString spaceTestStr = fileName;
		spaceTestStr.TrimLeft();
		spaceTestStr.TrimRight();

		//was the string changed in any way?
		//if it was then only accept this change.. right now
		if (spaceTestStr != fileName) {
			fileName = spaceTestStr;
			nameChange = true;
		}
		//otherwise procede with extention testing
		else {
			//if it is possible for an extention to exist
			if (fileName.GetLength() > 0) {
			
				pos = fileName.ReverseFind('.'); //find the start of the extention
		
				if (pos >= 0) {
					extLen = fileName.GetLength() - pos - 1;
					ext = fileName.Right(extLen);	//get the extention
					ext.MakeUpper();	//make all chars upper so we can later compare
				}
				//if extention is less than 3 chars long, then erase it from the file name
				if (extLen <= 3 && extLen > 0)
					fileName = fileName.Left(pos);
			}
		
			//check for individual file type filters and append one if necessary
			if (filterIndex == 1 && (ext != _T("BMP") && ext != _T("DIB"))) {
				fileName += _T(".BMP");
				nameChange = true;
			}	
			else if (filterIndex == 2 && (ext != _T("JPG") && ext != _T("JPE"))) {
				fileName += _T(".JPG");
				nameChange = true;
			}
			else if (filterIndex == 3 && (ext != _T("TGA") && ext != _T("VDA") &&
				ext != _T("VST") && ext != _T("ICB") &&	ext != _T("WIN"))) {
				fileName += _T(".TGA");
				nameChange = true;
			}	
			else if (filterIndex == 4 && ext != _T("TIF")) {
				fileName += _T(".TIF");
				nameChange = true;
			}				  
		} //end else			
		
		if (nameChange)	{	//if the name was changed, update the edit control
			SetDlgItemText(hpDlg, edt1, fileName);

			//if the user pressed the OK button with a bad filename
			if (((LPOFNOTIFY) lParam)->hdr.code == CDN_FILEOK) {
				SetFocus(GetDlgItem(hpDlg, edt1));
				SetWindowLong(hDlg, DWL_MSGRESULT, 1L);		//keep the dialog open

				//for an assert bug
				_AFX_THREAD_STATE* pThreadState = AfxGetThreadState();
				pThreadState->m_pAlternateWndInit = NULL;
				return 1;	//non zero return, we took care of the message
			}
		}
		//if the file wasn't changed and the user pressed OK see 
		// if the file already exists
		else if (((LPOFNOTIFY) lParam)->hdr.code == CDN_FILEOK) {				
			
			//find the file
			CFileFind search;
			if (search.FindFile(((LPOFNOTIFY) lParam)->lpOFN->lpstrFile)) {
				//if the file exists then show the overwrite prompt
				CString pcsMsg = ((LPOFNOTIFY) lParam)->lpOFN->lpstrFile;
				pcsMsg += " already exists.\nDo you want to replace it?";
				if (MessageBox(hDlg, pcsMsg, "Save As", MB_YESNO | MB_ICONEXCLAMATION) == IDNO) {
				  //if user chose NO then the dialog stays open
				  SetFocus(GetDlgItem(hpDlg, edt1));
				  SetWindowLong(hDlg, DWL_MSGRESULT, 1L);
				
				  //this avoids the assert
				  _AFX_THREAD_STATE* pThreadState = AfxGetThreadState();
				  pThreadState->m_pAlternateWndInit = NULL;
				  return 1;		//the message was processed
				}
			}
			//otherwise let the default procedure take care of the message
			_AFX_THREAD_STATE* pThreadState = AfxGetThreadState();
	        pThreadState->m_pAlternateWndInit = NULL;
            return 0;
		}
		
	}	//end if CDN_TYPECHANGE or CDN_FILEOK
	}	//end if WM_NOTIFY
	return 0;	//any other messages are not processed
}
