//JpegFuncs.cpp
#include "stdafx.h"
#include "ImageFile.h"
#include "jpeglib.h"
#include <stdio.h>	//for file i/o 
//<setjmp.h> is used for the optional error recovery mechanism shown in
//the second part of the example.
#include <setjmp.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////
///////////////////////////////////////////////////////////////////////////
// Error manager for the Jpeg file
///////////////////////////////////////////////////////////////////////////
struct ImageFileJPEG_error_mgr {
  struct jpeg_error_mgr pub;	// "public" fields

  jmp_buf setjmp_buffer;	// for return to caller
};

typedef struct ImageFileJPEG_error_mgr * ImageFileJPEG_error_ptr;


//ImageFileJPEG_error_exit()///////////////////////////////////////////////
// Here's the routine that will replace the standard Jpeg error_exit method.
///////////////////////////////////////////////////////////////////////////
METHODDEF(void)
ImageFileJPEG_error_exit(j_common_ptr cinfo)
{
  // cinfo->err really points to a ImageFileJPEG_error_mgr struct, so coerce pointer
  ImageFileJPEG_error_ptr myerr = (ImageFileJPEG_error_ptr) cinfo->err;

  // display the message.
  (*cinfo->err->output_message) (cinfo);

  // Return control to the setjmp point
  longjmp(myerr->setjmp_buffer, 1);
}


//OpenJpeg()//////////////////////////////////////////////////////////////
// This function opens a Jpeg file
//////////////////////////////////////////////////////////////////////////
BOOL ImageFile::LoadJpeg()
{
  //This struct contains the JPEG decompression parameters and pointers to
  //working space.
  struct jpeg_decompress_struct cinfo;
  
  // We use our private extension JPEG error handler.
  struct ImageFileJPEG_error_mgr jerr;
  
  FILE * infile;		// source file
  JSAMPARRAY buffer;	// Output row buffer
  int row_stride;		// physical row width in output buffer

  
  if ((infile = fopen(filename, "rb")) == NULL) {
    CString pcsMsg = "Error opening ";
    pcsMsg += filename;
    AfxMessageBox(pcsMsg);
    return FALSE;
  }

  //allocate and initialize JPEG decompression object

  // We set up the normal JPEG error routines, then override error_exit.
  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = ImageFileJPEG_error_exit;
  
  // Establish the setjmp return context for ImageFile_error_exit to use.
  if (setjmp(jerr.setjmp_buffer)) {
    // If we get here, the JPEG code has signaled an error.
    // We need to clean up the JPEG object, close the input file, and return.
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
    return FALSE;
  }
  
  // Now we can initialize the JPEG decompression object.
  jpeg_create_decompress(&cinfo);

  //specify data source (eg, a file)
  jpeg_stdio_src(&cinfo, infile);

  //read file parameters with jpeg_read_header()
  (void) jpeg_read_header(&cinfo, TRUE);

  //Start decompressor
  (void) jpeg_start_decompress(&cinfo);
	
  //After jpeg_start_decompress() we have the correct scaled
  // output image dimensions available, as well as the output colormap
  // if we asked for color quantization. 
  image_width = cinfo.output_width;
  image_height = cinfo.output_height;
  
  //check the color space
  if (cinfo.out_color_space == JCS_RGB)
  	SamplePerPixel = RGBSample;
  else { 
	   if (cinfo.out_color_space == JCS_GRAYSCALE)
  		   SamplePerPixel = GRAYSample;  			
	   else {
		   AfxMessageBox("Error: Can only open 24-bit and grayscale JPEG files");  
  		   jpeg_destroy_decompress(&cinfo);
		   fclose(infile);
  		   return FALSE;
	   }
  }

  //allocate space for our image
  if (!AllocateImage()) {
	  jpeg_destroy_decompress(&cinfo);
	  fclose(infile);
	  return FALSE;
  }
  
  // JSAMPLEs per row in output buffer
  row_stride = cinfo.output_width * cinfo.output_components;
  // Make a one-row-high sample array that will go away when done with image
  buffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

  int curr_row = 0;
  // Here we use the library's state variable cinfo.output_scanline as the
  // loop counter, so that we don't have to keep track ourselves.
  while (cinfo.output_scanline < cinfo.output_height) {
  
    (void) jpeg_read_scanlines(&cinfo, buffer, 1);
	
	int i = 0;
	while (i < image_width * SamplePerPixel) {
		//if 3 samples per pixel (color)
		if (SamplePerPixel == RGBSample) {			
			pImage[curr_row][i+0] = buffer[0][i+0];
			pImage[curr_row][i+1] = buffer[0][i+1];
			pImage[curr_row][i+2] = buffer[0][i+2];
			i += 3;
		}
		//otherwise grayscale
		else {
			pImage[curr_row][i] = buffer[0][i];
			i++;
		}			
  	}
  	curr_row++;
  }	

  (void) jpeg_finish_decompress(&cinfo);

  // Release JPEG decompression object
  jpeg_destroy_decompress(&cinfo);

  // After finish_decompress, we can close the input file.
  fclose(infile);

  //At this point you may want to check to see whether any corrupt-data
  // warnings occurred (test whether jerr.pub.num_warnings is nonzero).
  if (jerr.pub.num_warnings) {
  	CString pcsMsg = "Error file ";
  	pcsMsg += filename;
  	pcsMsg += " is corrupt.";
  	AfxMessageBox(pcsMsg);
  	return FALSE;
  }
   
  return TRUE;		
}	


///////
//SaveJpeg()//////////////////////////////////////////////////////////////
// This function opens the jpeg file specified by filename
//////////////////////////////////////////////////////////////////////////
BOOL ImageFile::SaveJpeg(int quality, int mode24)
{
  // This struct contains the JPEG compression parameters and pointers to
  // working space (which is allocated as needed by the JPEG library).
  struct jpeg_compress_struct cinfo;
  
  struct ImageFileJPEG_error_mgr jerr;
  // More stuff
  FILE * outfile;			// target file
  JSAMPROW row_pointer[1];	// pointer to JSAMPLE row[s]
  BOOL row_alloc = FALSE;	//do we need to alocate a row.. not yet 

  // We set up the normal JPEG error routines, then override error_exit.
  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = ImageFileJPEG_error_exit;
  
  //initialize the JPEG compression object.
  jpeg_create_compress(&cinfo);

  //open the output file
  if ((outfile = fopen(filename, "wb")) == NULL) {
    jpeg_destroy_compress(&cinfo);
    CString pcsMsg = "Can't open ";
    pcsMsg += filename;
    AfxMessageBox(pcsMsg);
    return FALSE;
  }
  
  //specify data destination
  jpeg_stdio_dest(&cinfo, outfile);

  //set parameters for compression
  // supply a description of the input image.
  cinfo.image_width = image_width; 		//image width and height, in pixels
  cinfo.image_height = image_height;

  if (mode24) {
	cinfo.input_components = RGBSample; //SamplePerPixel; //# of color components per pixel
	cinfo.in_color_space = JCS_RGB; 	// colorspace of input image
	if (SamplePerPixel == GRAYSample) {
		row_pointer[0] = new JSAMPLE[image_width * RGBSample]; 
		row_alloc = TRUE;	//we just allocated a row
	}
  }
  else {
	cinfo.input_components = GRAYSample;
	cinfo.in_color_space = JCS_GRAYSCALE;
	if (SamplePerPixel == RGBSample) {
		row_pointer[0] = new JSAMPLE[image_width];
		row_alloc = TRUE;
	}
  }

  //set default compression parameters.
  jpeg_set_defaults(&cinfo);
  
  // quality is (quantization table) scaling:
  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  //Start compressor
  // TRUE ensures that we will write a complete interchange-JPEG file.
  jpeg_start_compress(&cinfo, TRUE);
  
  //while there are still scanlines left
  // Here we use the library's state variable cinfo.next_scanline as the
  // loop counter, so that we don't have to keep track ourselves.
  while (cinfo.next_scanline < cinfo.image_height) {
    
    //get the scanline from our image
	  if (SamplePerPixel == RGBSample) {
		if (mode24)
			row_pointer[0] = pImage[cinfo.next_scanline];
		else {
			int alt_col = 0;
			for (int col = 0; col < image_width; col++) {
				row_pointer[0][col] = (unsigned char) 
						 (pImage[cinfo.next_scanline][alt_col+0] * .299 + 
						  pImage[cinfo.next_scanline][alt_col+1] * .587 + 
						  pImage[cinfo.next_scanline][alt_col+2] * .114);
				alt_col+=3;
			}
		}
	  }
	  else {
		  if (mode24) {
			  int alt_col = 0;
			  for (int col = 0; col < image_width * RGBSample; col+=3) {
				row_pointer[0][col+0] = (unsigned char) pImage[cinfo.next_scanline][alt_col]; 
				row_pointer[0][col+1] = (unsigned char) pImage[cinfo.next_scanline][alt_col];
				row_pointer[0][col+2] = (unsigned char) pImage[cinfo.next_scanline][alt_col];
				
				alt_col++;
			  }
		  }
		  else
			row_pointer[0] = pImage[cinfo.next_scanline];
	 }

    //write the scanline to file
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  if (row_alloc)
     delete [] row_pointer[0];

  // Finish compression
  jpeg_finish_compress(&cinfo);
  
  // close the output file.
  fclose(outfile);

  // release memory.
  jpeg_destroy_compress(&cinfo);

  return TRUE;
}

