//TiffFuncs.cpp
#include "stdafx.h"
#include "ImageFile.h"
#include "tiffio.h" 

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////
static int checkcmap(int n, uint16* r, uint16* g, uint16* b);

//LoadTiff()//////////////////////////////////////////////////////////
// This function loads a tif file into our image array
//////////////////////////////////////////////////////////////////////
BOOL ImageFile::LoadTiff()    
{
    TIFF          *tif;
    unsigned long imageLength; 
    unsigned long imageWidth; 
    unsigned int  BitsPerSample;
    unsigned long LineSize;
    unsigned long RowsPerStrip;  
    uint16		  Photometric;
    uint16  	  tifSamplePerPixel;
    long          nrow;
	unsigned long row;
    unsigned char *buf;          
    HGLOBAL       hStrip = NULL;
    uint16*		  red;
    uint16*		  green;
    uint16*		  blue;
	int			  i;


	//open the file for input
    tif = TIFFOpen(filename, "r");
    
    if (!tif) {
    	CString pcsMsg = "Error opening ";
    	pcsMsg += filename;
    	AfxMessageBox(pcsMsg);
    	return FALSE;
    }	
        
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imageWidth);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imageLength);  
    TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &BitsPerSample);
    TIFFGetField(tif, TIFFTAG_ROWSPERSTRIP, &RowsPerStrip);   
    TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &Photometric);
    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &tifSamplePerPixel);
    
    LineSize = TIFFScanlineSize(tif); //Number of bytes in the line

	image_width = (int) imageWidth;
	image_height = (int) imageLength;
	
    //tifSamplePerPixel = (int) (LineSize/imageWidth);

	//if the image is 24 bit RGB
	if (tifSamplePerPixel == 3)
		SamplePerPixel = RGBSample;
	//if the tif is color mapped or grayscale
	else {
	   //if only one sample per pixel, bits per sample could be 1, 2, 4, 8 -bit
	   if (tifSamplePerPixel == 1) {
		  if (Photometric == PHOTOMETRIC_PALETTE)
			  SamplePerPixel = RGBSample;	//color mapped, later converted to RGB
		  else
	          SamplePerPixel = GRAYSample;	//grayscale stays grayscale
	   }
	   //otherwise it is something like CMYK, which we don't handle
	   else {
	   	  CString pcsMsg;
	   	  pcsMsg.Format("Can't read a %d channel image", tifSamplePerPixel); 
	   	  AfxMessageBox(pcsMsg);
	   	  TIFFClose(tif);
		  return FALSE;  	
	   }
	}	  	
		

	try { 
		
		//allocate room for our image since we got all the 
		// dimensions we need	
		if (!AllocateImage())
			throw new CException;
		
        hStrip = GlobalAlloc(GHND, TIFFStripSize(tif));
        buf = (unsigned char *) GlobalLock(hStrip);           
        
        if (!buf)
        	throw new CException;
        
        //PhotometricInterpretation = 2 image is RGB
        //PhotometricInterpretation = 3 image has a color palette              
        if (Photometric == PHOTOMETRIC_PALETTE)
        {         	
		  //get the palette from the tiff file	           
          if (!TIFFGetField(tif, TIFFTAG_COLORMAP, &red, &green, &blue))
			  throw new CException;

		  int num_colors = (1 << BitsPerSample);
		  
		  if (checkcmap(num_colors, red, green, blue) == 16) {
		  //Convert colormap to 8-bits values.

#define	CVT(x)		(((x) * 255) / ((1L<<16)-1))
			for (i = 0; i < num_colors; i++) {
				red[i] = CVT(red[i]);
				green[i] = CVT(green[i]);
				blue[i] = CVT(blue[i]);
			}
#undef CVT
		  }
        }  //end if PhotoInterp. == palette               
         
        int my_row = 0;
        //read the tiff lines and save them in pImage
		for (row = 0; row < imageLength; row += RowsPerStrip) {     
            nrow = (row + RowsPerStrip > imageLength ? imageLength - row : RowsPerStrip);
            if (TIFFReadEncodedStrip(tif, TIFFComputeStrip(tif, row, 0),
                buf, nrow*LineSize)==-1) 
                  throw new CException;
            else {  
                for (int l = 0; l < nrow; l++) {
                    //if the image is in full color
                    if (Photometric == PHOTOMETRIC_RGB) {
                        for (i = 0; i < (int) imageWidth * SamplePerPixel; i += 3) {
                        	pImage[my_row][i+0] = buf[l*LineSize+i+0];                            
                            pImage[my_row][i+1] = buf[l*LineSize+i+1];
                            pImage[my_row][i+2] = buf[l*LineSize+i+2];
                        }
                    }
                    //if the image is palletized color    
                    else {
                    if (Photometric == PHOTOMETRIC_PALETTE) {
                    	int j = 0;
                    	int k = 0;
                    	while (j < (int) imageWidth * SamplePerPixel) {
					   	   pImage[my_row][j+0] = (char) red[buf[l*LineSize+k]];        
                      	   pImage[my_row][j+1] = (char) green[buf[l*LineSize+k]];
                       	   pImage[my_row][j+2] = (char) blue[buf[l*LineSize+k]];	

                           j += 3;
                  		   k++;
                        }	
            		}	
            		// otherwise if PhotoInterp. == 0 or 1 then the image is grayscale
            		else {
						for (i = 0; i < (int) imageWidth * SamplePerPixel; i++) {
                        	if (Photometric == PHOTOMETRIC_MINISBLACK)  //0 == white, 2^BitsPerSample == black
                        		pImage[my_row][i] = buf[l*LineSize+i];
                        	else 						//0 == black, 2^BitsPerSample == white
                        		pImage[my_row][i] = ((1<<BitsPerSample)-1) - buf[l*LineSize+i];	
                    	}
                    	
                    }
                    }
                    my_row++;
                }
            }
        }
        GlobalUnlock(hStrip);
        GlobalFree(hStrip); 
        TIFFClose(tif);
		return TRUE;
	} 
	catch (CException * pe) {
		pe->Delete();
		CString pcsMsg = "Error reading ";
        pcsMsg += filename;
        AfxMessageBox(pcsMsg);
		GlobalUnlock(hStrip); 
        GlobalFree(hStrip);
        TIFFClose(tif);
        return FALSE;
	}         
}


static int checkcmap(int n, uint16* r, uint16* g, uint16* b)
{
    while (n-- > 0)
        if (*r++ >= 256 || *g++ >= 256 || *b++ >= 256)
        return (16);
    
    return (8);
}


///////
//SaveTiff()///////////////////////////////////////////////////////////////
// This function stores our image in a tiff file specified by filename
///////////////////////////////////////////////////////////////////////////
BOOL ImageFile::SaveTiff(int mode24)
{
	TIFF * tif;     //the output file
	uint16 bitsPerSample = 8; 	//8-bits per sample in both RGB and grayscale cases
	uint16 PhotometricInterpretation;
	unsigned char* buffer;
	BOOL buf_alloc = FALSE;
	int tifSamplePerPixel;

	//open the file for output	
	if ((tif = TIFFOpen(filename, "w")) == NULL) {
        CString pcsMsg = "Error: Can't open ";
        pcsMsg += filename;
        AfxMessageBox(pcsMsg);
        return FALSE;
    }
	
	if (mode24) { //force rgb
		PhotometricInterpretation = PHOTOMETRIC_RGB;
		tifSamplePerPixel = RGBSample;
		if (SamplePerPixel == GRAYSample) {
			buffer = new unsigned char[image_width * RGBSample];
			buf_alloc = TRUE;
		}
	}	
	else {
		PhotometricInterpretation = PHOTOMETRIC_MINISBLACK;
		tifSamplePerPixel = GRAYSample;
		if (SamplePerPixel == RGBSample) {
			buffer = new unsigned char[image_width];
			buf_alloc = TRUE;
		}
	}

	//set the header fields of the tiff file
	TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, (uint32) image_width);
	TIFFSetField(tif, TIFFTAG_IMAGELENGTH, (uint32) image_height);
	TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, bitsPerSample);
    TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
    TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PhotometricInterpretation);
    TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, (uint16) tifSamplePerPixel);
    TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, 1);
 	TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);   //single image plane
	TIFFSetField(tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_NONE);  //no meaningful units of resolution

	try {
	//try writing all of the scanlines to the file
	for (int row = 0; row < image_height; row++) {
		if (mode24) {
			if (SamplePerPixel == RGBSample)
				buffer = pImage[row];
			else {
				int aux_col = 0;
				for (int col = 0; col < image_width * RGBSample; col += 3) {
					buffer[col+0] = pImage[row][aux_col];
					buffer[col+1] = pImage[row][aux_col];
					buffer[col+2] = pImage[row][aux_col];
					aux_col++;
				}
			}
		}		
		else {
			if (SamplePerPixel == GRAYSample)
				buffer = pImage[row];
			else {
				int aux_col = 0;
				for (int col = 0; col < image_width; col++) {
					buffer[col] = (unsigned char) 
								   (pImage[row][aux_col+0] * .299 +
								    pImage[row][aux_col+1] * .587 +
									pImage[row][aux_col+2] * .114);
					aux_col += 3;
				}
			}
		}

		if (!TIFFWriteScanline(tif, (unsigned char *) buffer, row, 0))
        	throw new CException;
    }
	}
	catch (CException *pe) {
		pe->Delete();
		if (buf_alloc)
			delete [] buffer;
		TIFFClose(tif);			//close the file
		CString pcsMsg = "Error saving file ";
		pcsMsg += filename;
		AfxMessageBox(pcsMsg);	//display the error message
		return FALSE;			//abnormal return
	}
	if (buf_alloc)
		delete [] buffer;
	TIFFClose(tif);				//everything went ok
	return TRUE;

}

