// MaterialDialog.cpp : implementation file
//

#include "stdafx.h"
#include "RayTrace3D.h"
#include "MaterialDialog.h"
#include "NumEdit.h"
#include "PreviewPic.h"
//#include "RayTracer/include/Oort.h"//ImageFile.h"

#include "RayTracer/include/Oort.h"
#include "RayTracer/include/World.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMaterialDialog dialog


CMaterialDialog::CMaterialDialog(CWnd* pParent /*=NULL*/)
: CDialog(CMaterialDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMaterialDialog)
	//}}AFX_DATA_INIT
	
	m_pMaterialList = NULL;
	m_pCurrentSurf = NULL;
	m_TextureAmount = 0;
	m_BumpAmount = 0;
	m_Shininess = 0;
	m_Opacity = 0;
	m_Refract = 1.0;
	m_Reflect = 0;
	m_pBumpImage = NULL;
	m_pTexImage = NULL;

	//default usage to false
	m_TexApply = FALSE;
	m_BumpApply = FALSE;

	m_Modified = FALSE;
}

CMaterialDialog::~CMaterialDialog()
{
	if (m_pCurrentSurf != NULL) {

		//if the texture is not attached to the current surface
		ImageMap* pSurfTex = (ImageMap *) m_pCurrentSurf->GetDiffuseTexture();
		if (pSurfTex != NULL)
			if (pSurfTex->GetImageFile() != m_pTexImage && m_pTexImage != NULL)
				delete m_pTexImage;

		//if the bump texture is not applied to the current surface
		ImageBumpMap* pSurfBumpTex = (ImageBumpMap *) m_pCurrentSurf->GetBumpMapTexture();
		if (pSurfBumpTex != NULL)
			if (pSurfBumpTex->GetImageFile() != m_pBumpImage && m_pBumpImage != NULL)
				delete m_pBumpImage;
	}
}


void CMaterialDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMaterialDialog)
	DDX_Control(pDX, IDC_PREVIEW_PIC, m_RenderPreviewPic);
	DDX_Control(pDX, IDC_TEXTURE_APPLY, m_TexApplyButton);
	DDX_Control(pDX, IDC_BUMP_APPLY, m_BumpApplyButton);
	DDX_Control(pDX, IDC_MATERIAL_SELECT, m_MaterialCombo);
	DDX_Control(pDX, IDC_TEXTURE_MAP, m_TextureMapCtl);
	DDX_Control(pDX, IDC_SPECULAR_COLOR, m_SpecularCtl);
	DDX_Control(pDX, IDC_DIFFUSE_COLOR, m_DiffuseCtl);
	DDX_Control(pDX, IDC_BUMP_MAP, m_BumpMapCtl);
	DDX_Control(pDX, IDC_AMBIENT_COLOR, m_AmbientCtl);
	DDX_Control(pDX, IDC_REFRACT_PRESETS, m_RefractCombo);
	DDX_Control(pDX, IDC_BUMP_AMOUNT, m_BumpEdit);
	DDX_Control(pDX, IDC_REFRACT, m_RefractEdit);
	DDX_Control(pDX, IDC_REFLECT, m_ReflectEdit);
	DDX_Control(pDX, IDC_OPACITY, m_OpacityEdit);
	DDX_Control(pDX, IDC_TEXTURE_AMOUNT, m_TextureEdit);
	DDX_Control(pDX, IDC_SHININESS, m_ShininessEdit);
	//}}AFX_DATA_MAP

}


BEGIN_MESSAGE_MAP(CMaterialDialog, CDialog)
	//{{AFX_MSG_MAP(CMaterialDialog)
	ON_BN_CLICKED(ID_APPLY, OnMaterialApply)
	ON_BN_CLICKED(IDC_LOAD_AMBIENT, OnLoadAmbient)
	ON_BN_CLICKED(IDC_LOAD_BUMP, OnLoadBump)
	ON_BN_CLICKED(IDC_LOAD_DIFFUSE, OnLoadDiffuse)
	ON_BN_CLICKED(IDC_LOAD_SPECULAR, OnLoadSpecular)
	ON_BN_CLICKED(IDC_LOAD_TEXTURE, OnLoadTexture)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SHININESS_SPIN, OnShininessSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_TEXTURE_SPIN, OnTextureSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_REFRACT_SPIN, OnRefractSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_REFLECT_SPIN, OnReflectSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_OPACITY_SPIN, OnOpacitySpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_BUMP_SPIN, OnBumpSpin)
	ON_CBN_SELENDOK(IDC_REFRACT_PRESETS, OnSelEndOkRefractCombo)
	ON_CBN_SELCHANGE(IDC_REFRACT_PRESETS, OnSelChangeRefractCombo)
	ON_CBN_SELCHANGE(IDC_MATERIAL_SELECT, OnSelchangeMaterialSelect)
	ON_CBN_SELENDOK(IDC_MATERIAL_SELECT, OnSelendokMaterialSelect)
	ON_BN_CLICKED(IDC_RENDER_PREVIEW, OnRenderPreview)
	//}}AFX_MSG_MAP
	ON_MESSAGE (WM_USER_PREVIEW_FINISHED, OnEndPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMaterialDialog message handlers

void CMaterialDialog::OnLoadAmbient() 
{
	CColorDialog dlg(m_AmbientCol, CC_FULLOPEN, NULL);

	if (dlg.DoModal() == IDOK)
	{
		m_AmbientCol = dlg.GetColor();
		m_AmbientCtl.SetColor(m_AmbientCol);
		
		m_Modified = TRUE;  //set modified flag
	}
}

void CMaterialDialog::OnLoadDiffuse() 
{
	CColorDialog dlg(m_DiffuseCol, CC_FULLOPEN, NULL);
	
	if (dlg.DoModal() == IDOK)
	{
		m_DiffuseCol = dlg.GetColor();
		m_DiffuseCtl.SetColor(m_DiffuseCol);

		m_Modified = TRUE;  //set modified flag
	}
}

void CMaterialDialog::OnLoadSpecular() 
{
	CColorDialog dlg(m_SpecularCol, CC_FULLOPEN, NULL);
	
	if (dlg.DoModal() == IDOK)
	{
		m_SpecularCol = dlg.GetColor();	
		m_SpecularCtl.SetColor(m_SpecularCol);

		m_Modified = TRUE;  //set modified flag
	}
}

void CMaterialDialog::OnLoadTexture() 
{
	if (m_pTexImage == NULL)
		m_pTexImage = new ImageFile(FALSE);

	if (m_pTexImage->OpenFile()) {
		HDIB hDib = m_pTexImage->GetImageDib();
		m_TextureMapCtl.SetImage(hDib);

		//enable all of fields related to the 
		// textures
		m_TexApplyButton.EnableWindow(TRUE);
		m_TextureEdit.EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TEX)->EnableWindow(TRUE);
		GetDlgItem(IDC_TEXTURE_SPIN)->EnableWindow(TRUE);
	
		m_Modified = TRUE;  //set modified flag
	}
}

//open the texture file and load it into the preview control
void CMaterialDialog::OnLoadBump() 
{	
	if (m_pBumpImage == NULL)
		m_pBumpImage = new ImageFile(FALSE);

	if (m_pBumpImage->OpenFile()) {
		HDIB hDib = m_pBumpImage->GetImageDib();
		m_BumpMapCtl.SetImage(hDib);

		//enable all of fields related to the 
		// bump textures
		m_BumpApplyButton.EnableWindow(TRUE);
		m_BumpEdit.EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_BUMP)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUMP_SPIN)->EnableWindow(TRUE);

		m_Modified = TRUE;  //set modified flag
	}
}

//used to verify the validity of all the values in the controls
BOOL CMaterialDialog::VerifyAll()
{
	int texAmt, bumpAmt, shininess, opacity, reflect;
	float refract;

	//check the corrrectness of the edit controls
	if (m_TextureEdit.GetEditValue(texAmt) &&
		m_BumpEdit.GetEditValue(bumpAmt) &&
		m_ShininessEdit.GetEditValue(shininess) &&
		m_OpacityEdit.GetEditValue(opacity) &&
		m_ReflectEdit.GetEditValue(reflect) &&
		m_RefractEdit.GetEditFloatValue(refract)) {
		
		//if any of the controls have updated values
		//then set the modified flag
		if (m_TextureAmount != texAmt) {
			m_TextureAmount = texAmt;
			m_Modified = TRUE;
		}
		if (m_BumpAmount != bumpAmt) {
			m_BumpAmount = bumpAmt;
			m_Modified = TRUE;
		}
		if (m_Shininess != shininess) {
			m_Shininess = shininess;
			m_Modified = TRUE;
		}
		if (m_Opacity != opacity) {
			m_Opacity = opacity;
			m_Modified = TRUE;
		}	
		if (m_Reflect != reflect) {
			m_Reflect = reflect;
			m_Modified = TRUE;
		}
		if (m_Refract != refract) {
			m_Refract = refract;
			m_Modified = TRUE;
		}

		//if the boolean flags were changed
		int buttonState = m_BumpApplyButton.GetCheck();
		if (m_BumpApply != buttonState) {
			m_BumpApply = buttonState;
			m_Modified = TRUE;
		}

		buttonState = m_TexApplyButton.GetCheck();
		if (m_TexApply != buttonState) {
			m_TexApply = buttonState;
			m_Modified = TRUE;
		}
		
		return TRUE; //the values are all correct
	}
	else
		return FALSE; //the values are faulty
}


void CMaterialDialog::OnMaterialApply() 
{
	//do the data exchange
	UpdateData(TRUE);
	
	//if data is legit and is modified, then apply data to surface
	if (VerifyAll()) {
		
		//if the material was not modified then there's nothing to apply
		if (!m_Modified)
			return;

		//apply the rest here
		ApplyToSurface(m_pCurrentSurf);
		//post the message to save the current material
		m_pView->PostMessage(WM_USER_APPLY, 0, 0);
		m_Modified = FALSE;	//reset m_Modified flag
	}
	else
		AfxMessageBox("Please enter a valid value");
}

void CMaterialDialog::OnOK() 
{
	//do the data exchange
	UpdateData(TRUE);

	//verify validity of edit controls
	if (VerifyAll())
	{
		//if the material was not modified then there's nothing to apply
		// so just shut the window down and return
		if (!m_Modified) {
			DestroyWindow();
			return;
		}

		ApplyToSurface(m_pCurrentSurf);
		//post the message to save the material
		m_pView->PostMessage(WM_USER_APPLY, 0, 0);

		DestroyWindow();
	}
	else
		AfxMessageBox("Please enter a valid value");
}



//ApplyToSurface()///////////////////////3/20/2000
// Apply all the changed fields to the current surface 
// called from OnOk and OnApply
void CMaterialDialog::ApplyToSurface(HallSurface* surf)
{
	surf->SetTransmitted(RGBColor(float(m_Opacity) / 100), m_Refract);
	surf->SetReflected(RGBColor(float(m_Reflect) / 100));
	surf->SetAmbient(RGBColor(m_AmbientCol));
	surf->SetDiffuseColor(RGBColor(m_DiffuseCol));

	//shininess here is specnum //we don't set the shin_strength
	surf->SetSpecular(RGBColor(m_SpecularCol), float(m_Shininess));


	//see if we can apply the texture map
	if (m_pTexImage != NULL) {
		ImageMap* ptMap = surf->GetDiffuseTexture();
		//if the map doesn't exist, create it
		if (ptMap == NULL) {
			ptMap = new ImageMap(IdentityMatrix(), FALSE, FALSE, 
						 m_pTexImage->Dup(), 0.0, 1.0, 1.0,
						 GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT, GL_DECAL, 1.0);
			surf->SetDiffuseTexture(ptMap);
		}
		else
			//if the surface's image file is different from that of
			// the dialog's
			if (ptMap->GetImageFile() != m_pTexImage)
				ptMap->SetImageFile(m_pTexImage->Dup());
	}
	

	//now try to apply the bump map
	if (m_pBumpImage != NULL) {
		ImageBumpMap* pbMap = surf->GetBumpMapTexture();
		//if the bump map doesn't exist, create it
		if (pbMap == NULL) {
			pbMap = new ImageBumpMap(IdentityMatrix(), FALSE, m_pBumpImage->Dup(),
					0, 1.0, 1.0, 1.0); 
			surf->SetBumpMap(pbMap);
		}
		else
			//if the dialog's bump image is different from that
			// of the surface's
			if (pbMap->GetImageFile() != m_pBumpImage)
				pbMap->SetImageFile(m_pBumpImage->Dup());
	}

	//set the texture amounts
	surf->SetTexAmount(float(m_TextureAmount) / 100);
	surf->SetBumpAmount(float(m_BumpAmount) / 100);

	//set the texture usage flags
	surf->SetTextureFlag(m_TexApply);
	surf->SetBumpFlag(m_BumpApply);
}



void CMaterialDialog::OnCancel() 
{
	// TODO: Add extra cleanup here
	
	//CDialog::OnCancel();
	DestroyWindow();
}

//This function makes sure that the Materials Dialog object is deleted
//when the dialog box is destroyed
void CMaterialDialog::PostNcDestroy() 
{
	// TODO: Add your specialized code here and/or call the base class
	
	CDialog::PostNcDestroy();

	//used for modeless dialog boxes to notify the view that the object
	// no longer exists

	//AfxGetMainWnd()->PostMessage(WM_USER_MAT_DIALOG_DESTROYED, 0, 0);
	m_pView->PostMessage(WM_USER_MAT_DIALOG_DESTROYED, 0, 0);
	
	//delete the object
	delete this;
}

void CMaterialDialog::OnShininessSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_ShininessEdit.UpdateEditValue(-(pNMUpDown->iDelta)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}

void CMaterialDialog::OnTextureSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_TextureEdit.UpdateEditValue(-(pNMUpDown->iDelta)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}

void CMaterialDialog::OnRefractSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_RefractEdit.UpdateEditFloatValue(float(-(pNMUpDown->iDelta)*.005)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}

void CMaterialDialog::OnReflectSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_ReflectEdit.UpdateEditValue(-(pNMUpDown->iDelta)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}

void CMaterialDialog::OnOpacitySpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_OpacityEdit.UpdateEditValue(-(pNMUpDown->iDelta)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}

void CMaterialDialog::OnBumpSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	if (!m_BumpEdit.UpdateEditValue(-(pNMUpDown->iDelta)))
		AfxMessageBox("Please enter a valid value");

	*pResult = 0;
}


BOOL CMaterialDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// TODO: Add extra initialization here
	m_TextureEdit.SetLimitText(3);
	m_TextureEdit.SetRange(0, 100);
	m_BumpEdit.SetLimitText(3);
	m_BumpEdit.SetRange(0, 300);
	m_RefractEdit.SetLimitText(6);
	m_RefractEdit.SetFloatRange(1.000, 3.0);	//refract
	m_ReflectEdit.SetLimitText(3);
	m_ReflectEdit.SetRange(0, 100);
	m_OpacityEdit.SetLimitText(3);
	m_OpacityEdit.SetRange(0, 100);
	m_ShininessEdit.SetLimitText(3);
	m_ShininessEdit.SetRange(0, 300); //exponent from 0 to 300
	m_TextureEdit.SetEditValue(m_TextureAmount);
	m_BumpEdit.SetEditValue(m_BumpAmount);
	m_RefractEdit.SetEditFloatValue(m_Refract);		//refract double
	m_ReflectEdit.SetEditValue(m_Reflect);
	m_OpacityEdit.SetEditValue(m_Opacity);
	m_ShininessEdit.SetEditValue(m_Shininess);
	m_RefractCombo.AddString("Custom");
	m_RefractCombo.AddString("Air");
	m_RefractCombo.AddString("Glass");
	m_RefractCombo.AddString("Ice");
	m_RefractCombo.AddString("Water");
	m_RefractCombo.AddString("Diamond");
	m_RefractCombo.AddString("Emerald");
	m_RefractCombo.AddString("Ruby");

	InitRefract();

	UpdateMaterialCombo(); //updates everything

	//initalize the sphere with a surface for m_PreviewWorld
	InitPreview();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


//these are initial Preview World parameters 
void CMaterialDialog::InitPreview()
{
	HallSurface* newSurf = new HallSurface;

	newSurf->SetAmbient(RGBColor(0.5f));
	newSurf->SetDiffuseColor(RGBColor(0.5));
    newSurf->SetSpecular(RGBColor(1.0), 128.0);
	newSurf->SetShinStrength(.5);
	newSurf->SetTransmitted(RGBColor(0), 1.0);

	//OpenGL only
	newSurf->SetPolygonMode(GL_FILL);	     // GL_LINE for wire frame, GL_FILL for all other types
	newSurf->SetShadeModel(GL_SMOOTH);       // GL_FLAT for wire frame and flat shading type, GL_SMOOTH for all other types
	newSurf->SetTwoSided(FALSE);

	//name the material				
	newSurf->SetName("SphereSurf");
	m_PreviewWorld.GetSurfaces()->AppendToList(newSurf);  //put the material in the list

	//create a new sphere with spherical mapping
	Sphere* sphere = new Sphere(Vector3D(0,0,0), 10.0, 
		newSurf, new SphericalMapping(Vector3D(0,0,0)));

	//add the object to the Objects database
	m_PreviewWorld.AddObject(sphere);

	//add the camera to the local World
	Camera* camera = new Camera;
	
	camera->m_Name = "Default";
	camera->m_Pos = Vector3D(0, 0, 30);
	camera->m_Target = Vector3D(0, 0, 0); //origin is the target.. sphere location
	camera->m_Up = Vector3D(0, 1, 0);

	camera->m_Focal_length = 50; //was 70
	camera->m_Range_far = 1000;  //Depth Of View
	camera->m_Range_near = 0;
	camera->m_Roll = 0.0;
	camera->m_Fov = 45.0;  //in degrees
		
	m_PreviewWorld.AddCamera(camera);

	//Add lights to the Preview World
	Light* theLight;

//	Vector3D Light1(-15.0f, 15.0f, 30.0f);
//	Vector3D Light2(30.0f, -30.0f, 5.0f);
	Vector3D Light1(-20, 20, 30);
	Vector3D Light2(25, -25, -10);
	RGBColor color(0.7f, 0.7f, 0.7f);

//	theLight = new PointLight(Light1, color); //0.25, 0.25, 0.5, TRUE);
	theLight = new DirectLight(Light1, color, TRUE);
	theLight->SetRange(10000, 12000, FALSE); //no attenuation

	theLight->SetName("Default 1");
	m_PreviewWorld.AddLight(theLight);

//	theLight = new PointLight(Light2, color);
	theLight = new DirectLight(Light2, color, TRUE);
	theLight->SetRange(10000, 12000, FALSE); //no attenuation

	theLight->SetName("Default 2");
	m_PreviewWorld.AddLight(theLight);

	//now initialize the imageDib that will contain the 
	//rendering
	CRect rect;
	GetDlgItem(IDC_PREVIEW_PIC)->GetClientRect(&rect);
	ImageFile image(FALSE);
	//rect.Width() and rect.Height() as arguments to reset 
	//don't seem to work with StretchDIBits()
	image.Reset(120, 110, RGBSample);
	HDIB hDib = image.GetImageDib();
	m_RenderPreviewPic.SetImage(hDib);
}



/////REFRACT Settings///////////////////////////////////////////////////

void CMaterialDialog::InitRefract()
{
	if (m_Refract == airRef)
		m_RefractCombo.SelectString(-1,"Air");
	
	else if (m_Refract == glassRef)
		m_RefractCombo.SelectString(-1,"Glass");
	
	else if (m_Refract == iceRef)
		m_RefractCombo.SelectString(-1,"Ice");

	else if (m_Refract == waterRef)
		m_RefractCombo.SelectString(-1,"Water");

	else if (m_Refract == diamondRef)
		m_RefractCombo.SelectString(-1,"Diamond");

	else if (m_Refract == emeraldRef)
		m_RefractCombo.SelectString(-1,"Emerald");

	else if (m_Refract == rubyRef)
		m_RefractCombo.SelectString(-1,"Ruby");

	else
		m_RefractCombo.SelectString(-1,"Custom");
	
	UpdateRefractEdit();
}	

void CMaterialDialog::UpdateRefractEdit()
{

	CString value, preset;
	m_RefractCombo.GetWindowText(preset);

	if (preset == "Air")
		m_RefractEdit.SetEditFloatValue(airRef);
	
	if (preset == "Glass")
		m_RefractEdit.SetEditFloatValue(glassRef);
	
	if (preset == "Ice")
		m_RefractEdit.SetEditFloatValue(iceRef);
	
	if (preset == "Water")
		m_RefractEdit.SetEditFloatValue(waterRef);
	
	if (preset == "Diamond")
		m_RefractEdit.SetEditFloatValue(diamondRef);
	
	if (preset == "Emerald")
		m_RefractEdit.SetEditFloatValue(emeraldRef);
	
	if (preset == "Ruby")
		m_RefractEdit.SetEditFloatValue(rubyRef);

}

void CMaterialDialog::OnSelEndOkRefractCombo() 
{
	UpdateRefractEdit();
}

void CMaterialDialog::OnSelChangeRefractCombo() 
{
	UpdateRefractEdit();
}

//THE CREATE FUNCTION CALLED FOR THE MODELESS DIALOG
BOOL CMaterialDialog::Create(CView *pView) 
{
	m_pView = pView;

	return CDialog::Create( CMaterialDialog::IDD );
}

void CMaterialDialog::OnSelchangeMaterialSelect() 
{
	UpdateData(TRUE);

	if (VerifyAll())
		UpdateAllControls();	
	else
		AfxMessageBox("Please enter a valid value");
}

void CMaterialDialog::OnSelendokMaterialSelect() 
{
//	UpdateAllControls();
//	UpdateData(TRUE);

//	if (VerifyAll())
//		UpdateAllControls();	
//	else
//		AfxMessageBox("Please enter a valid value");
}

void CMaterialDialog::UpdateAllControls()
{
	CString mat_name;
	HDIB hDib;
	BOOL texExists;
	m_MaterialCombo.GetWindowText(mat_name);

	//if the material was modified then ask save
	if (m_Modified == TRUE) {
		CString pcsMsg;
		pcsMsg.Format("Material %s was modified. Apply this material to the scene?", 
						m_pCurrentSurf->GetName());

		int reply = AfxMessageBox(pcsMsg, MB_YESNO);
			
		//if the user wants to save the material then use OnMaterialApply handler
		if (reply == IDYES) {
			//OnMaterialApply();//SendMessage(ID_APPLY);
			//apply the data to the current surface like in OnMaterialApply
			ApplyToSurface(m_pCurrentSurf);
			//post the message to redraw the main scene the current material
			m_pView->PostMessage(WM_USER_APPLY, 0, 0);
		}
	}

	//Save this surface to use later in resetting textures
	HallSurface* prevSurf = m_pCurrentSurf;

	//find the surface selected in the combo
	m_pCurrentSurf = m_pMaterialList->FindSurfaceByName(mat_name);

	//if somehow the user deleted the materials database - 
	//do not alter current material
	if (m_pCurrentSurf == NULL) {
		CString pcsMsg = "Cannot find surface ";
		pcsMsg += mat_name;
		AfxMessageBox(pcsMsg);
		return;
	}

	//reset the class variables
	m_Shininess = (int) m_pCurrentSurf->GetShininess();
	m_Opacity = int(GetGray(m_pCurrentSurf->GetTransmitted()) * 100);
	m_Reflect = int(GetGray(m_pCurrentSurf->GetReflected()) * 100);
	m_Refract = m_pCurrentSurf->GetIndex();
	m_BumpAmount = int(m_pCurrentSurf->GetBumpAmount() * 100);
	m_TextureAmount = int(m_pCurrentSurf->GetTexAmount() * 100);

	m_AmbientCol = GetCOLORREF(m_pCurrentSurf->GetAmbient());
	m_AmbientCtl.SetColor(m_AmbientCol);
	m_DiffuseCol = GetCOLORREF(m_pCurrentSurf->GetDiffuse());
	m_DiffuseCtl.SetColor(m_DiffuseCol);
	m_SpecularCol = GetCOLORREF(m_pCurrentSurf->GetSpecular());
	m_SpecularCtl.SetColor(m_SpecularCol);


	//This is a temporary variable used to store the 
	// previous texture so it can be deleted if needed
	ImageFile* tempImage = m_pTexImage;

	//Assign the texture map to the pic control
	ImageMap* pSurfTex = (ImageMap *) m_pCurrentSurf->GetDiffuseTexture();
	if (pSurfTex != NULL) {
		//paint the texture in the window
		hDib = pSurfTex->GetImageFile()->GetImageDib();
		m_TextureMapCtl.SetImage(hDib);
		
		//assign the image map to the member pointer
		m_pTexImage = pSurfTex->GetImageFile();

		texExists = TRUE;
	}
	else {
		m_TextureMapCtl.SetEmptyColor();
		texExists = FALSE;
		//reset the bump image file
		m_pTexImage = NULL;
	}
	//See if the texture is used by the previous material and
	// discard it if it is not
	if (prevSurf != NULL) {
		ImageMap* pPrevTex = (ImageMap *) prevSurf->GetDiffuseTexture();
		if (pPrevTex != NULL) {
			if (pPrevTex->GetImageFile() != tempImage &&
				tempImage != NULL)
				delete tempImage;
		}
	}
	//enable all of fields related to the textures
	m_TexApplyButton.EnableWindow(texExists);
	m_TextureEdit.EnableWindow(texExists);
	GetDlgItem(IDC_STATIC_TEX)->EnableWindow(texExists);
	GetDlgItem(IDC_TEXTURE_SPIN)->EnableWindow(texExists);

	
	//Assign the previous bump map image to the
	//temp so it can later be referenced and maybe deleted
	tempImage = m_pBumpImage;

	//Now the bump map
	ImageBumpMap* pSurfBumpTex = (ImageBumpMap *) m_pCurrentSurf->GetBumpMapTexture();
	if (pSurfBumpTex != NULL) {
		//paint the texture in the window
		hDib = pSurfBumpTex->GetImageFile()->GetImageDib();
		m_BumpMapCtl.SetImage(hDib);
		
		//now set the member bump image to the new surface's bump image
		m_pBumpImage = pSurfBumpTex->GetImageFile();

		texExists = TRUE;
	}
	else {
		m_BumpMapCtl.SetEmptyColor();
		texExists = FALSE;
		//no map so reset to null
		m_pBumpImage = NULL;
	}
	//See if the texture is used by the previous material and
	// discard it if it is not
	if (prevSurf != NULL) {
		ImageBumpMap* pPrevBumpTex = (ImageBumpMap *) prevSurf->GetBumpMapTexture();
		if (pPrevBumpTex != NULL) {
			if (pPrevBumpTex->GetImageFile() != tempImage &&
				tempImage != NULL)
				delete tempImage;
		}
	}
	//enable all of fields related to the bump textures
	m_BumpApplyButton.EnableWindow(texExists);
	m_BumpEdit.EnableWindow(texExists);
	GetDlgItem(IDC_STATIC_BUMP)->EnableWindow(texExists);
	GetDlgItem(IDC_BUMP_SPIN)->EnableWindow(texExists);

	//update flag variables
	m_TexApply = m_pCurrentSurf->TextureUsed();
	m_BumpApply = m_pCurrentSurf->BumpUsed();

	m_TexApplyButton.SetCheck(m_TexApply);
	m_BumpApplyButton.SetCheck(m_BumpApply);

	//update the all of the controls
	m_TextureEdit.SetEditValue(m_TextureAmount);
	m_BumpEdit.SetEditValue(m_BumpAmount);
	m_RefractEdit.SetEditFloatValue(m_Refract);		//refract double
	m_ReflectEdit.SetEditValue(m_Reflect);
	m_OpacityEdit.SetEditValue(m_Opacity);
	m_ShininessEdit.SetEditValue(m_Shininess);
	
	InitRefract();
	m_Modified = FALSE;  //reset modified flag

	//also Update the preview Pic
//IMPORTANT CALL ON PREVIEW
//	OnRenderPreview();

	//update all of the controls
	UpdateData(TRUE);
}

void CMaterialDialog::SetMaterialList(SurfaceList* theMatList)
{
	m_pMaterialList = theMatList;
//	UpdateMaterialCombo(); //updates everything
}

//this function is called from the View to
// reset the materials in the Dialog when a 
// new database is loaded
void CMaterialDialog::UpdateMaterials()
{
	m_Modified = FALSE;

	//take out all of the currently present material
	// names in the Material Combo Box
	int currItem = m_MaterialCombo.GetCount() - 1;
	while (currItem >= 0)
		m_MaterialCombo.DeleteString(currItem--);

	//now load all of the new information
	UpdateMaterialCombo();
}

void CMaterialDialog::UpdateMaterialCombo()
{
	bool firstMat = true;

	for (SurfaceList::Iterator sc(*m_pMaterialList); sc.Valid(); sc.GotoNext()) {
		
		CString matName = sc.Contents()->GetName();
		m_MaterialCombo.AddString(matName);
		
		if (firstMat) {
			m_MaterialCombo.SelectString(-1, matName);
			firstMat = false;
		}
	}
	
	UpdateAllControls();

}

//when the preview button is pressed or the material is changed
// in the combo box
void CMaterialDialog::OnRenderPreview() 
{
	SurfaceList* sList = m_PreviewWorld.GetSurfaces();
	
	HallSurface* previewSurface = (*sList)[0];//sList->FindSurfaceByName("SphereSurf");

	//do the data exchange
	UpdateData(TRUE);

	//verify validity of edit controls
	if (VerifyAll())
		//apply the material settings to the sphere's surface
		ApplyToSurface(previewSurface);
	else {
		AfxMessageBox("Please enter a valid value");
		return;
	}
	
	//copy the texture map and bump map for Multithreading to work
//	ImageMap* ptMap = (ImageMap*) previewSurface->GetDiffuseTexture();
//	if (ptMap != NULL) {
//		ImageFile* pFile = ptMap->GetImageFile();
//		if (pFile != NULL)
//			ptMap->SetImageFile(pFile->Dup());
//		ptMap = (ImageMap*) ptMap->Dup();
//	}

//	ImageBumpMap* ptBumpMap = (ImageBumpMap*) previewSurface->GetBumpMapTexture();
//	if (ptBumpMap != NULL) {
//		ImageFile* pFile = ptMap->GetImageFile();
//		if (pFile != NULL)
//			ptBumpMap->SetImageFile(pFile->Dup());
//			ptBumpMap = (ImageBumpMap*) ptBumpMap->Dup();
//	}

	//Disable the render button for the time of the render
	GetDlgItem(IDC_RENDER_PREVIEW)->EnableWindow(FALSE);

	//initialize the thread parameters
	THREADPARAMSDLG* ptrParams = new THREADPARAMSDLG;
	ptrParams->previewPic = &m_RenderPreviewPic;
	ptrParams->pWorld = &m_PreviewWorld;

	//Create the RayTracing thread here
	AfxBeginThread(CMaterialDialog::PreviewThreadFunc, (LPVOID) ptrParams);

}

//The thread function that handles main Ray Tracing
//Ray trace the scene, pass in the window the output will go to
UINT CMaterialDialog::PreviewThreadFunc(LPVOID pParam)
{
	THREADPARAMSDLG* ptrParms = (THREADPARAMSDLG*) pParam;
	CPreviewPic* previewPic = ptrParms->previewPic;
	World* pWorld = ptrParms->pWorld;
	delete ptrParms;

	//RayTrace the preview
	pWorld->RayTracePreview(previewPic);
	
	return 0;
}

//called when the rendering of the preview has finished
LRESULT CMaterialDialog::OnEndPreview(WPARAM wParam, LPARAM lParam)
{
	//enable the rendering button back
	GetDlgItem(IDC_RENDER_PREVIEW)->EnableWindow(TRUE);

	return 0;
}