#include "NumEdit.h"
#include "PreviewPic.h"
//#include "Material.h"
#include "RayTracer/include/Oort.h"
#include "RayTracer/include/World.h"

#if !defined(AFX_MATERIALDIALOG_H__904CC7C0_AE3F_11D3_9E40_C78CD3DB6E30__INCLUDED_)
#define AFX_MATERIALDIALOG_H__904CC7C0_AE3F_11D3_9E40_C78CD3DB6E30__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// MaterialDialog.h : header file
//

//constants for refraction defined
const float		airRef = 1.0003f;
const float		glassRef = 1.6000f;
const float		iceRef = 1.3050f;
const float		waterRef = 1.3330f;
const float	    diamondRef = 2.4180f;
const float		emeraldRef = 1.5760f;
const float		rubyRef = 1.7110f;

typedef struct {
	CPreviewPic* previewPic;
	World*		 pWorld;
} THREADPARAMSDLG;

/////////////////////////////////////////////////////////////////////////////
// CMaterialDialog dialog

class CMaterialDialog : public CDialog
{
// Construction
public:
	CMaterialDialog(CWnd* pParent = NULL);   // standard constructor
	~CMaterialDialog();

// Dialog Data
	//{{AFX_DATA(CMaterialDialog)
	enum { IDD = IDD_MAT_DIALOG };
	CPreviewPic	m_RenderPreviewPic;
	CButton	m_TexApplyButton;
	CButton	m_BumpApplyButton;
	CComboBox	m_MaterialCombo;
	CPreviewPic	m_TextureMapCtl;
	CPreviewPic	m_SpecularCtl;
	CPreviewPic	m_DiffuseCtl;
	CPreviewPic	m_BumpMapCtl;
	CPreviewPic	m_AmbientCtl;
	CComboBox	m_RefractCombo;
	CNumEdit	m_BumpEdit;
	CNumEdit	m_RefractEdit;
	CNumEdit	m_ReflectEdit;
	CNumEdit	m_OpacityEdit;
	CNumEdit	m_TextureEdit;
	CNumEdit	m_ShininessEdit;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMaterialDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

//other public functions
public:
	BOOL Create( CView* );
	void SetMaterialList(SurfaceList* theMatList);
	void UpdateMaterials();
	static UINT	PreviewThreadFunc(LPVOID pParam);
// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CMaterialDialog)
	afx_msg void OnMaterialApply();
	afx_msg void OnLoadAmbient();
	afx_msg void OnLoadBump();
	afx_msg void OnLoadDiffuse();
	afx_msg void OnLoadSpecular();
	afx_msg void OnLoadTexture();
	virtual void OnOK();
	virtual void OnCancel();
	afx_msg void OnShininessSpin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnTextureSpin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnRefractSpin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnReflectSpin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnOpacitySpin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnBumpSpin(NMHDR* pNMHDR, LRESULT* pResult);
	virtual BOOL OnInitDialog();
	afx_msg void OnSelEndOkRefractCombo();
	afx_msg void OnSelChangeRefractCombo();
	afx_msg void OnSelchangeMaterialSelect();
	afx_msg void OnSelendokMaterialSelect();
	afx_msg void OnRenderPreview();
	//}}AFX_MSG
	afx_msg LRESULT OnEndPreview(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()

private:
	//used to verify the validity of all the values in the controls
	BOOL VerifyAll(void);
	void UpdateRefractEdit();
	void InitRefract();
	void UpdateAllControls();
	void UpdateMaterialCombo();
	void ApplyToSurface(HallSurface* surf);
	void InitPreview();

public:
	World			m_PreviewWorld;

	//all of the sufaces from the main scene
	SurfaceList*	m_pMaterialList;

	HallSurface*	m_pCurrentSurf;
//	ImageMap*		m_pTexImage;
//	ImageBumpMap*	m_pBumpImage;
	ImageFile*		m_pTexImage;  //now these are only image files
	ImageFile*		m_pBumpImage;


	int				m_Shininess;
	int				m_Opacity;
	int				m_Reflect;
	float			m_Refract;
	int				m_BumpAmount;
	int				m_TextureAmount;
	CString			m_szTextureName;
	CString			m_szBumpName;
	COLORREF		m_AmbientCol;
	COLORREF		m_DiffuseCol;
	COLORREF		m_SpecularCol;
	CView*			m_pView;

	BOOL			m_TexApply;
	BOOL			m_BumpApply;
	BOOL			m_Modified;

};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MATERIALDIALOG_H__904CC7C0_AE3F_11D3_9E40_C78CD3DB6E30__INCLUDED_)
