// PreviewPic.cpp : implementation file
//

#include "stdafx.h"
#include "RayTrace3D.h"
#include "PreviewPic.h"
#include "RayTracer/include/Oort.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WIDTHBYTES(bits)    (((bits) + 31) / 32 * 4)

/////////////////////////////////////////////////////////////////////////////
// CPreviewPic

CPreviewPic::CPreviewPic() : 
	m_DibAttached(FALSE)
{
	m_Color = ::GetSysColor(COLOR_3DFACE);	//initial color
	m_hDib = NULL;		//initialize the pointers
	m_lpBits = NULL;
	m_lpBMIH = NULL;
	m_iWidth = 0;
	m_iHeight = 0;
}

CPreviewPic::~CPreviewPic()
{
	DestroyDib();
}


BEGIN_MESSAGE_MAP(CPreviewPic, CButton)
	//{{AFX_MSG_MAP(CPreviewPic)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPreviewPic message handlers

void CPreviewPic::DrawItem(LPDRAWITEMSTRUCT lpDis) 
{
	CDC* pDC = CDC::FromHandle(lpDis->hDC); // Get CDC to draw
	int	iWidth = lpDis->rcItem.right - lpDis->rcItem.left;
	int	iHeight = lpDis->rcItem.bottom - lpDis->rcItem.top;

	//if we have a dib use StretcDIBBits to blit it in the control
	if (m_DibAttached) {
		pDC->SetStretchBltMode(COLORONCOLOR);
		::StretchDIBits(pDC->GetSafeHdc(), 0, 0, iWidth, iHeight,
			0, 0, m_iWidth, m_iHeight,
			m_lpBits, (LPBITMAPINFO) m_lpBMIH, DIB_RGB_COLORS, SRCCOPY);
	}
	else {	//or draw a solid color
		CRect Rect(0, 0, iWidth, iHeight);
		CBrush Brush(m_Color);
		pDC->FillRect(&Rect, &Brush);
	}
}

//set the color 
void CPreviewPic::SetColor(COLORREF color)
{
	//solid color mode
	m_Color = color;
	m_DibAttached = FALSE;
	
	DestroyDib();

	Invalidate(TRUE);
}

void CPreviewPic::SetEmptyColor()
{
	m_Color = ::GetSysColor(COLOR_3DFACE);
	m_DibAttached = FALSE;
	
	DestroyDib();

	Invalidate(TRUE);
}

void CPreviewPic::DestroyDib()
{
	//unlock the previous dib, if there was one
	if (m_DibAttached) {
		::GlobalUnlock(m_hDib);
		::DeleteObject(m_hDib);
		m_DibAttached = FALSE;
	}
}


//assign a DIB
void CPreviewPic::SetImage(HDIB hDIB)
{
	DestroyDib();

	//assign the new dib and associated pointer variables
	m_hDib = hDIB;
	m_lpBMIH = (LPBITMAPINFOHEADER) ::GlobalLock(hDIB);
	m_lpBits = (LPBYTE)m_lpBMIH + *(LPDWORD)m_lpBMIH;

	m_iWidth = m_lpBMIH->biWidth;	//get the dimensions
	m_iHeight = m_lpBMIH->biHeight;

	m_DibAttached = TRUE;
	
	Invalidate(TRUE);
}

//AddScanlineToDib()////////////////////////////////////////////////////////////
//	This function adds a scanline to the specified row in the DIB,
//  The buffer's order must be B-G-R, not R-G-B.
//////////////////////////////////////////////////////////////////////
void CPreviewPic::AddScanlineToDib(int row, unsigned char* pBuffer)
{
    // the Dib wasn't created yet 
    if (!m_DibAttached) 
        return;

//	int image_width = m_lpBMIH->biWidth;
//	int image_height = m_lpBMIH->biHeight;

    DWORD dwBytesPerLine = WIDTHBYTES(m_lpBMIH->biBitCount * m_iWidth);//image_width);

 	//Align = Number of bytes to add at the end of each line of the DIB
    int Align = 4 - (dwBytesPerLine % 4);
    if (Align == 4)	
    	Align = 0;

	LPBYTE lpBits = m_lpBits;
	lpBits += ((m_iWidth * RGBSample) + Align) * ((m_iHeight-1) - row);
	//now lpBits points to the right row

	//copy the memory to the Dib
	memcpy(lpBits, pBuffer, m_iWidth * RGBSample); 
}

