// RTWindow.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "RTWindow.h"
#include "RayTrace3D.h"
#include "RayTrace3DDoc.h"
#include "RayTrace3DView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRTWindow

//IMPLEMENT_DYNCREATE(CRTWindow, CFrameWnd)

CRTWindow::CRTWindow(int newWidth, int newHeight, CDocument* pDoc, CView* pView)
{
	m_pDoc = (CRayTrace3DDoc *) pDoc;
	m_pView = (CRayTrace3DView *) pView;
	m_WindWidth = newWidth;
	m_WindHeight = newHeight;

	Create(NULL, "Ray Trace Output", 0,
		rectDefault, NULL, MAKEINTRESOURCE(IDR_RTWINMENU));

	m_pImageFile = new ImageFile(FALSE);
	m_pImageFile->Reset(newWidth, newHeight, RGBSample);
		
	m_hDib = NULL;
	UpdateImage();
	alreadySaved = FALSE;
}

CRTWindow::~CRTWindow()
{
	//delete the image file
	if (m_pImageFile)
		delete m_pImageFile;
}

BEGIN_MESSAGE_MAP(CRTWindow, CFrameWnd)
	//{{AFX_MSG_MAP(CRTWindow)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRTWindow message handlers

BOOL CRTWindow::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
//	return	CView::OnEraseBkgnd(pDC);

	return TRUE; // tell Windows not to erase the background
}

void CRTWindow::OnFileSave() 
{
	SaveWindowImage();	
}

void CRTWindow::OnFileSaveAs() 
{
	if (m_pImageFile->SaveFileAs())		
		alreadySaved = TRUE;

	//this is for re activating the title bar after a SaveAs is called
	//there is a bug that dehilites the title bar even after the 
	//window is selected
	OnNcActivate(TRUE);
}

void CRTWindow::OnFileClose() 
{
	//send yourself the close message
	SendMessage(WM_CLOSE);

//	OnClose();
}

int CRTWindow::OnCreate(LPCREATESTRUCT lpCreateS) 

{
	if (CFrameWnd::OnCreate(lpCreateS) == -1)
		return -1;
	
	ShowWindow(SW_SHOWDEFAULT);
//	UpdateWindow();
	
	return 0;
}

void CRTWindow::OnClose() 
{
	//send no as external Invocation so we can send
	// the message and close the window if we need to
	if (ClosingTime(FALSE)){
		m_pView->SendMessage(WM_USER_RTWINDOW_DESTROYED, (WPARAM) this, 0);
		CFrameWnd::OnClose();
	}
}

BOOL CRTWindow::ClosingTime(BOOL externInvoke)
{
	int reply;
	if (m_pImageFile->RenderActive()) {//CurrentRow() < m_pImageFile->Height() || 
		//m_pDoc->GetRenderingInProgress()) {
		reply = ::MessageBox(m_hWnd, "Destroy the rendering process?", 
					"Ray Tracer", MB_YESNO | MB_ICONEXCLAMATION);
		if (reply == IDNO)
			return FALSE;

		m_pDoc->SetRenderingInProgress(FALSE);
		//ends the rendering session, so no more scanlines could be added
		m_pImageFile->EndCurrentRender();
	}

	if (!alreadySaved) {
		reply = ::MessageBox(m_hWnd, "Save this image in a file?", 
					"Ray Tracer", MB_YESNOCANCEL | MB_ICONEXCLAMATION);

		if (reply == IDCANCEL)
			return FALSE;
		else if (reply == IDYES) {
			if (!SaveWindowImage())
				return FALSE;
		}
	}

	//close the window
	if (externInvoke)
		CFrameWnd::OnClose();

	//if all of the temptations were avoided
	return TRUE;

}


BOOL CRTWindow::SaveWindowImage()
{
	BOOL saved;
	if (alreadySaved) {
		if (m_pImageFile->SaveFile()) {
			alreadySaved = TRUE;
			saved = TRUE;
		}
		else 
			saved = FALSE;
	}
	else {
		if (m_pImageFile->SaveFileAs()) {
			alreadySaved = TRUE;
			saved = TRUE;
		}
		else 
			saved = FALSE;
	}

	//the title bar doesn't rehilite if this is not called.
	// This is a new bug to my knowledge, that makes 65,001 Microsoft bugs
	OnNcActivate(TRUE);
	return saved;
}

//assign a DIB
void CRTWindow::UpdateImage()
{
	//unlock the previous dib
	if (m_hDib) {
		::GlobalUnlock(m_hDib);
		::DeleteObject(m_hDib);
	}

	//assign the new dib and associated pointer variables
	m_hDib = m_pImageFile->GetImageDib();
	m_lpBMIH = (LPBITMAPINFOHEADER) ::GlobalLock(m_hDib);
	m_lpBits = (LPBYTE)m_lpBMIH + *(LPDWORD)m_lpBMIH;

	Invalidate(TRUE);
	UpdateWindow();
}

void CRTWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	CRect rect; 
	GetClientRect(&rect);
	int theWidth = rect.Width();
	int theHeight = rect.Height();

	dc.SetStretchBltMode(COLORONCOLOR);
	::StretchDIBits(dc.GetSafeHdc(), 0, 0, theWidth, theHeight,
		0, 0, m_lpBMIH->biWidth, m_lpBMIH->biHeight, 
		m_lpBits, (LPBITMAPINFO) m_lpBMIH, DIB_RGB_COLORS, SRCCOPY);

	// Do not call CFrameWnd::OnPaint() for painting messages
}

BOOL CRTWindow::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	// Create a window without min/max buttons or sizable border 
    cs.style = WS_OVERLAPPED | WS_POPUPWINDOW | WS_MINIMIZEBOX | WS_CAPTION;
	cs.hwndParent = AfxGetMainWnd()->GetSafeHwnd();
    
	// Size the window to 1/2 screen size and center it 
    cs.cy = ::GetSystemMetrics(SM_CYSCREEN) / 2; 
    cs.cx = ::GetSystemMetrics(SM_CXSCREEN) / 2;      
	cs.x = ((cs.cx * 2) - cs.cx) / 4;
	cs.y = ((cs.cy * 2) - cs.cy) / 4;

//	CString title = cs.lpszName;
//	if (image_width > cs.cy)
//		newWidth = 


	return CFrameWnd::PreCreateWindow(cs);
}
