// RayTrace3DDoc.cpp : implementation of the CRayTrace3DDoc class
//

#include "stdafx.h"
#include "RayTrace3D.h"

#include "RayTrace3DDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DDoc

IMPLEMENT_DYNCREATE(CRayTrace3DDoc, CDocument)

BEGIN_MESSAGE_MAP(CRayTrace3DDoc, CDocument)
	//{{AFX_MSG_MAP(CRayTrace3DDoc)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(IDC_AUTO_CENTER, OnAutoCenter)
	ON_COMMAND(ID_BACKCOLOR, OnBackColor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DDoc construction/destruction

//call the original
CRayTrace3DDoc::CRayTrace3DDoc()
{
	m_SceneLoaded = FALSE;
}

CRayTrace3DDoc::~CRayTrace3DDoc()
{
	Objects.DeleteObjects();
	Surfaces.DeleteSurfaces();
	Cameras.DeleteCameras();
	Lights.DeleteLights();
}

BOOL CRayTrace3DDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DDoc serialization

void CRayTrace3DDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DDoc diagnostics

#ifdef _DEBUG
void CRayTrace3DDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CRayTrace3DDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DDoc commands

void CRayTrace3DDoc::OnFileOpen() 
{
	OnOpen3DSFile();
}


BOOL CRayTrace3DDoc::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	//if the commands are interface update commands, and there's
	// no scene loaded or the rendering is in progress, 
	// then disable the Ray Trace and Material Dialog menu items. 
	if ((nID == ID_ON_RENDER || nID == ID_MAT_DLG || nID == ID_BACKCOLOR) &&
		 nCode == CN_UPDATE_COMMAND_UI &&
		(!m_SceneLoaded || m_bRenderingInProgress)) {

		((CCmdUI*)pExtra)->Enable(FALSE);
		return TRUE;
	}

	//disable or enable Auto Centering menu item
	if (nID == IDC_AUTO_CENTER && nCode == CN_UPDATE_COMMAND_UI && !m_SceneLoaded) {
		((CCmdUI*)pExtra)->Enable(FALSE);
		return TRUE;
	}
		
	//based on the scene status enable or disable the Camera menu
	CMenu* pSceneMenu = AfxGetMainWnd()->GetMenu()->GetSubMenu(3);
	if (!m_SceneLoaded)
		pSceneMenu->EnableMenuItem(1, MF_BYPOSITION | MF_GRAYED);
	else
		pSceneMenu->EnableMenuItem(1, MF_BYPOSITION | MF_ENABLED);

	// If pHandlerInfo is NULL, then handle the message
	if (pHandlerInfo == NULL)
	{		
		int i = 0;
		// Filter the commands sent to a text color menu option
		for (SimpleList<UINT>::Iterator sc(m_CameraIDList); sc.Valid(); sc.GotoNext())
		{
			UINT currID = sc.Contents();

			if (nID == currID)
			{
				if (nCode == CN_COMMAND)
				{
					// Handle WM_COMMAND message
					DoSelectCamera(i);
				}
				else if (nCode == CN_UPDATE_COMMAND_UI)
				{
					// Update UI element state
					DoUpdateSelectCamera(i, (CCmdUI*)pExtra);
				}
				return TRUE;
			}
			i++;
		}
	}
	
	return CDocument::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//changes the index of the cameras
void CRayTrace3DDoc::DoSelectCamera(int iCamera)
{
	m_iCamera = iCamera;
	m_CameraChanged = TRUE;
	m_pCamera = Cameras[iCamera];   //select the current camera

	UpdateAllViews(NULL);
}


void CRayTrace3DDoc::DoUpdateSelectCamera(int iCamera, CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_iCamera == iCamera);
	pCmdUI->Enable(TRUE);
}

void 
CRayTrace3DDoc::OnOpen3DSFile()
{
	bool error = false;
	CString filters = "3D Studio (*.3DS)|*.3DS|All Files (*.*)|*.*||";
	
	//get the main view so we can send messages to it
	POSITION pos = GetFirstViewPosition();
	CView* pView = GetNextView(pos);

	//if there is a ray tracing session in progress, Ask to end it.
	if (m_bRenderingInProgress) {
		int reply = AfxMessageBox("Before opening a new 3DS file,\nthe Ray Tracing process must be stopped.\nStop the rendering process?", 
			MB_YESNO | MB_ICONEXCLAMATION);
	
		//if the user said no.. then cancel file loading
		if (reply == IDNO)
			return;
	
		//end the current rendering progress
		m_bRenderingInProgress = FALSE;

		// this message disables the
		// RTWindow associated with the render from any further input
		pView->SendMessage(WM_USER_STOP_RENDER, 0, 0);
	}

	//create an Open file dialog
	CFileDialog openFileDlg(TRUE, NULL, NULL, 
		OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
		OFN_FILEMUSTEXIST, filters);

	//initial filter index is "All Files"
	openFileDlg.m_ofn.nFilterIndex = 0;

	//show the dialog
	if (openFileDlg.DoModal() != IDOK)
		return;

	//get the extention and file name
	CString fileExt = openFileDlg.GetFileExt();
	CString file_name = openFileDlg.GetPathName();

	//make all characters upper case so we can compare them
	//to other extentions
	fileExt.MakeUpper();  
	
	//compare the file extention to known types and load 
	//the image.
	if (fileExt == "3DS") {
		if (!m_Obj3DS.Read3DSFile(file_name))
			error = true;
	}
	else {
		AfxMessageBox("No provision for this extention");
		return;
	}

	if (error) {
		CString pcsMsg = "An error occured while reading file: ";
		pcsMsg += file_name;
		pcsMsg += ". File not loaded.";
		AfxMessageBox(pcsMsg);
	}
	else { //Bring out the champaigne
		//file read, now assign OpenGL and RayTracing parts
		m_SceneLoaded = FALSE;
		//reset the last mouse pos member variables
		//these are reset to negative because the system calls
		//OnMouseMove initially and that messes up the scene matrix
		//we use these initial values for the test in OnMouseMove in the Doc
		m_LastX = -1;
		m_LastY = -1;

		//set the window title
		CString title = m_Obj3DS.GetFilename();
		title += " - Ray Tracer";
		AfxGetMainWnd()->SetWindowText(title);

		Lights.DeleteLights();
		m_Obj3DS.GetLightList(Lights);
		
		Surfaces.DeleteSurfaces();
		m_Obj3DS.GetSurfaceList(Surfaces);
	
		Cameras.DeleteCameras();
		m_Obj3DS.GetCameraList(Cameras);

		//delete the meshes in the OpenGL mesh list
		m_MeshListGL.DeleteMeshList();

		//delete the ray tracer's Objects
		Objects.DeleteObjects();

		//get the OpenGL and Ray Tracer's objects
		m_Obj3DS.ConvertMeshData(m_MeshListGL, Objects, Surfaces, m_SceneCenter);

		//draw light sources in next draw
		m_LightsChanged = TRUE;

		//compile the display lists
		m_MeshListGL.BuildDisplayListGL();

		//set the background colors
		m_BackColor = m_Obj3DS.GetBackColor();
		m_Ambient = m_Obj3DS.GetAmbientLight();

		//the scene is loaded, this flag is used in drawing
		m_SceneLoaded = TRUE;

		//Update the Camera Menu
		CMenu* pCameraMenu = AfxGetMainWnd()->GetMenu()->GetSubMenu(3)->GetSubMenu(1);

		//Clear the menu of previous items
		UINT iCount = pCameraMenu->GetMenuItemCount();
		for (UINT i = 0; i < iCount; i++)
			pCameraMenu->RemoveMenu(0, MF_BYPOSITION);
		
		//clear the previous ID's out of the Camera ID list
		m_CameraIDList.Clear();

		int j = 0;
		for (CameraList::Iterator cam(Cameras); cam.Valid(); cam.GotoNext()) {
			//assign the command ID, the base is in Resource.h
			UINT currID = ID_CAMERA0 + j;
			
			//add the ID to the ID list
			m_CameraIDList.AppendToList(currID);
			
			//append the name of the camera to the menu
			pCameraMenu->AppendMenu(MF_STRING, currID, cam.Contents()->GetName());
			j++;
		}
		//select the default camera at 0 and update the view
		DoSelectCamera(0);

		//now tell the view to reload the surfaces in the 
		//Material Dialog if it is open
		pView->SendMessage(WM_USER_UPDATE_DATABASE, 0, 0);

	}

}


//----------------------------------------------------------------------------------------------------------------------
// This is the main paint routine. If there's no data assigned yet then an empty window is drawn else the data stored
// in the various lists. These lists must already be set up.
//currCam has to be assigned to a valid camera
void 
CRayTrace3DDoc::DrawOpenGL()
{
	// Note: We don't need to activate the rendering context here, as we did this in the OnCreate event
	//       handler and we have setup the window to use a private DC (which is therefore never reset).

	//reset the scene's viewing attributes
	if (m_CameraChanged) {

		m_CameraChanged = FALSE;
		// set up projection 
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		//get the main window's client area
		CRect rect;
		AfxGetMainWnd()->GetClientRect(&rect);

		m_pCamera->ApplyPerspectiveOpenGL(rect.Width(), rect.Height());
	}

	m_pCamera->ApplyTransformToSceneGL(m_SceneCenter, m_bAutoCenter);
	
	//if autocentering is turned on
	if (m_bAutoCenter)
		glTranslatef(-m_SceneCenter.x, -m_SceneCenter.y, -m_SceneCenter.z);

	//if materials were changed from the materials dialog then
	// rebuild display lists and reset the m_MaterialsChanged flag
	if (m_MaterialsChanged) {
		m_MeshListGL.BuildDisplayListGL();
		m_MaterialsChanged = FALSE;
	}

	if (m_LightsChanged) {
		Lights.SetupLightsGL();
		m_LightsChanged = FALSE;
	}

	// calls the main display list
	m_MeshListGL.DrawObjectsGL();

}


//when user wants to rotate the camera
void
CRayTrace3DDoc::MouseDownAt(int x, int y)
{
	//get the point the mouse down occured at
	m_LastX = x;
	m_LastY = y;
}

//when the user drags the mouse in the pressed state
// the current camera is rotated according to pitch
// and turn angles derived from the mouse's velocity vector
void
CRayTrace3DDoc::MouseMoveTo(int x, int y)
{
//	Matrix rot_matrix = Pitch(FSceneMatrix, XVector, DegToRad(m_LastY - y)) * 
//		Turn(FSceneMatrix, YVector, DegToRad(m_LastX - x));

	//if the last values are negative then this is the first call
	//so do nothing but assign the last members to current values
	if (m_LastX >= 0 && m_LastY >= 0) { 
		//generate the rotation in the three axes, first convert to radians
		Matrix rot_matrix = RotationXMatrix(float(-(m_LastY - y) * M_PI / 180)) *
							//RotationXMatrix(float(-(x - m_LastX) * M_PI / 180)) * ;
							//RotationYMatrix(float((m_LastX - x) * M_PI / 180)) * ;
							RotationZMatrix(float((x - m_LastX) * M_PI / 180));

		//apply the rotation to the camera
		if (m_SceneLoaded)
			m_pCamera->RotatePos(rot_matrix);

	}

	//new point is now previous point
	m_LastX = x;
	m_LastY = y;
}


void CRayTrace3DDoc::OnAutoCenter() 
{
	UINT newState;
	CMenu* pSceneMenu = AfxGetMainWnd()->GetMenu()->GetSubMenu(3);

	//if auto centering is turned on then uncheck the menu item
	if (m_bAutoCenter) {
		newState = MF_UNCHECKED;
		m_bAutoCenter = FALSE;
	}
	else {	//otherwise enable it
		newState = MF_CHECKED;
		m_bAutoCenter = TRUE;
	}

	//set the new state of the menu item
	pSceneMenu->CheckMenuItem(IDC_AUTO_CENTER, MF_BYCOMMAND | newState);
	//redraw the scene
	UpdateAllViews(NULL);
}

//sets the background color of the scene
void CRayTrace3DDoc::OnBackColor() 
{
	CColorDialog dlg(GetCOLORREF(m_BackColor), CC_FULLOPEN, NULL);
	
	if (dlg.DoModal() == IDOK)
	{
		COLORREF col = dlg.GetColor();
		m_BackColor = RGBColor(col);
		m_MeshListGL.SetBackColor(m_BackColor);
		glClearColor(m_BackColor.x, m_BackColor.y, m_BackColor.z, 1);
		UpdateAllViews(NULL);
	}
}
