// RayTrace3DView.cpp : implementation of the CRayTrace3DView class
//

#include "stdafx.h"
#include "RayTrace3D.h"

#include "RayTrace3DDoc.h"
#include "RayTrace3DView.h"

#include "MaterialDialog.h"
#include "RenderDialog.h"
#include "RTWindow.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DView

IMPLEMENT_DYNCREATE(CRayTrace3DView, CView)

BEGIN_MESSAGE_MAP(CRayTrace3DView, CView)
	//{{AFX_MSG_MAP(CRayTrace3DView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_COMMAND(ID_MAT_DLG, OnMatDialog)
	ON_COMMAND(ID_ON_RENDER, OnRender)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
	ON_MESSAGE (WM_USER_MAT_DIALOG_DESTROYED, OnMatDialogDestroyed)
	ON_MESSAGE (WM_USER_APPLY, OnMaterialsApply)
	ON_MESSAGE (WM_USER_RTWINDOW_DESTROYED, OnRTWindowDestroyed)
	ON_MESSAGE (WM_USER_UPDATE_DATABASE, OnChangeDatabase)
	ON_MESSAGE (WM_USER_STOP_RENDER, OnStopRender)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DView construction/destruction

CRayTrace3DView::CRayTrace3DView()
{
	// TODO: add construction code here
	m_pMatDlg = NULL;
	m_hRC = NULL;
	m_pDC = NULL;
	m_pCurrentRenderImage = NULL;

}

CRayTrace3DView::~CRayTrace3DView()
{
}

BOOL CRayTrace3DView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	
	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DView drawing
void CRayTrace3DView::OnDraw(CDC* pDC)
{
	CRayTrace3DDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// just clear background and depth buffer, color has already been set
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//if the scene database exists in the World
	if (pDoc->SceneLoaded())
		pDoc->DrawOpenGL();

	// copy back buffer to front
	::SwapBuffers(m_pDC->m_hDC);

}

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DView message handlers

int CRayTrace3DView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
//	CRayTrace3DDoc* pDoc = GetDocument();
	CDC* pDC = new CClientDC(this);
	
//	pDoc->InitOpenGL(pDC);

	//FROM INITOPENGL
	int colorDepth;
    
	m_pDC = pDC; //assign the member DC to the one obtained

	colorDepth = ::GetDeviceCaps(m_pDC->m_hDC, BITSPIXEL) * ::GetDeviceCaps(m_pDC->m_hDC, PLANES);
	CreateRenderingContextGL(colorDepth);
	::wglMakeCurrent(m_pDC->m_hDC, m_hRC);
  
	// set some default options and values in the context
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);

	if (colorDepth < 24)
		glEnable(GL_DITHER);
    else 
		glDisable(GL_DITHER);
  
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  
	glClearColor(0, 0, 0, 1);

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	return 0;
}

// Set the OpenGL properties required to draw to the given dc and
// create a rendering context for it.
BOOL
CRayTrace3DView::CreateRenderingContextGL(int colorDepth)
{
	PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),   // size of this pfd
		1,                     // version number
		PFD_DRAW_TO_WINDOW |   // Property Flags
			PFD_SUPPORT_OPENGL |
			PFD_DOUBLEBUFFER, 
		PFD_TYPE_RGBA,         // RGBA type
		colorDepth,                    // 24-bit color depth
		0, 0, 0, 0, 0, 0,      // color bits ignored
		0,                     // no alpha buffer
		0,                     // shift bit ignored

		0,                     // no accumulation buffer
		0, 0, 0, 0,            // accum bits ignored
		32,                    // 32-bit z-buffer
		0,                     // no stencil buffer
		0,                     // no auxiliary buffer
		PFD_MAIN_PLANE,        // main layer
		0,                     // reserved
		0, 0, 0                // layer masks ignored
	};

    int pixelFormat =
        ::ChoosePixelFormat(m_pDC->m_hDC, &pfd);

	if (pixelFormat == 0)
		return FALSE;

    BOOL success =
        ::SetPixelFormat(m_pDC->m_hDC, pixelFormat, &pfd);

	if (!success)
		return FALSE;

	int retval = DescribePixelFormat(m_pDC->m_hDC, pixelFormat, sizeof(pfd), &pfd);
	
	if (retval == 0)
		return FALSE;

    m_hRC = ::wglCreateContext(m_pDC->m_hDC);

	if (m_hRC == NULL)
		return FALSE;

	return TRUE;
}

void CRayTrace3DView::OnDestroy() 
{
	CView::OnDestroy();
	
	// deactivate the rendering context;
	::wglMakeCurrent(NULL, NULL);

	// free the rendering context
	if (m_hRC != NULL)
		::wglDeleteContext(m_hRC);
	
}

BOOL CRayTrace3DView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
//	return	CView::OnEraseBkgnd(pDC);

	return TRUE; // tell Windows not to erase the background
}

void CRayTrace3DView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
//	GetDocument()->ResizeViewportGL(cx, cy);
	//FROM WORLD
	glViewport(0, 0, cx, cy);
	
	GetDocument()->SetCameraChanged(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// CRayTrace3DView diagnostics

#ifdef _DEBUG
void CRayTrace3DView::AssertValid() const
{
	CView::AssertValid();
}

void CRayTrace3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CRayTrace3DDoc* CRayTrace3DView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRayTrace3DDoc)));
	return (CRayTrace3DDoc*)m_pDocument;
}
#endif //_DEBUG

void CRayTrace3DView::OnMatDialog() 
{
	//if the Material Dialog object has not been created yet 
	if (m_pMatDlg == NULL) 
	{
		m_pMatDlg = new CMaterialDialog;		//create the object on heap
		m_pMatDlg->SetMaterialList(GetDocument()->GetSurfaces());  //direct reference from Doc//World class

		m_pMatDlg->Create(this);				//create it and pass the main view
												//so we can send messages to the view
		m_pMatDlg->ShowWindow(SW_SHOW);			//show the dialog box
	}
	
	//if the dialog was chosen from the main menu then it is selected
	m_pMatDlg->SetActiveWindow();
}

//User defined message notifying of the destroyed Material dialog object
LRESULT CRayTrace3DView::OnMatDialogDestroyed(WPARAM wParam, LPARAM lParam)
{
	//set the pointer of the Material Dialog object to NULL so it can be
	// allocated again in the future
	m_pMatDlg = NULL;
	return 0;
}

LRESULT CRayTrace3DView::OnMaterialsApply(WPARAM wParam, LPARAM lParam)
{
	GetDocument()->SetMaterialsChanged(TRUE);  //the Lists should be rebuilt
	Invalidate();	//redraw the content
	return 0;
}

//called when the document sends an update message
LRESULT CRayTrace3DView::OnChangeDatabase(WPARAM wParam, LPARAM lParam)
{
	//update the Material Dialog if it is open
	if (m_pMatDlg != NULL) {
		m_pMatDlg->SetMaterialList(GetDocument()->GetSurfaces()); 
		m_pMatDlg->UpdateMaterials();
	}

	return 0;
}

LRESULT CRayTrace3DView::OnStopRender(WPARAM wParam, LPARAM lParam)
{
	if (m_pCurrentRenderImage != NULL)
		//close the rendering process of the RTWindow
		m_pCurrentRenderImage->EndCurrentRender();

	return 0;
}

void CRayTrace3DView::OnRender() 
{
	CRenderDialog rendDlg;

	if (m_pMatDlg != NULL) {
		CString pcsMsg = "To start rendering the Material Dialog \n";
		pcsMsg += "must be shut down. Shut down Material Dialog?";
		if (AfxMessageBox(pcsMsg, MB_YESNO) == IDNO)
			return;
	
		//destroy the material dialog
		m_pMatDlg->DestroyWindow();
		m_pMatDlg = NULL;
	}

	//if the user chooses to render the image then 
	// make a new child window and add to windows list
	if (rendDlg.DoModal() == IDOK) {
		
		CRayTrace3DDoc* pDoc = GetDocument();

		int height = rendDlg.m_Height;
		int width = rendDlg.m_Width;
		if (rendDlg.m_Mode == 1)
			pDoc->SetAntiAliasMode(World::AntiAliasModes::Adaptive);
		else
			pDoc->SetAntiAliasMode(World::AntiAliasModes::None);

		pDoc->SetDepthLimit(rendDlg.m_Depth);

		
		CRTWindow* newWindow = new CRTWindow(width, height, pDoc, this);
		m_CRTWindowList.AddToList(newWindow);
		//assign the current render image, so we can cancel rendering from the view later
		m_pCurrentRenderImage = newWindow->GetImageFile();

		THREADPARAMS* ptrParams = new THREADPARAMS;
		ptrParams->window = newWindow;
		ptrParams->pDoc = GetDocument();

		//Create the RayTracing thread here
		AfxBeginThread(CRayTrace3DView::RayTraceThreadFunc, (LPVOID) ptrParams);
	}

}

//The thread function that handles main Ray Tracing
//Ray trace the scene, pass in the window the output will go to
UINT CRayTrace3DView::RayTraceThreadFunc(LPVOID pParam)
{
	THREADPARAMS* ptrParms = (THREADPARAMS*) pParam;
	CRTWindow* windowRT = (CRTWindow*) ptrParms->window;
	CRayTrace3DDoc* pDoc = ptrParms->pDoc;
	delete ptrParms;

	//RayTrace the scene
	pDoc->RayTrace(windowRT);
	
	return 0;
}

void CRayTrace3DView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRayTrace3DDoc* pDoc = GetDocument();
	if (pDoc->SceneLoaded())
		pDoc->MouseDownAt(point.x, point.y);

	CView::OnLButtonDown(nFlags, point);
}

void CRayTrace3DView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CView::OnMouseMove(nFlags, point);
	
	if (nFlags & MK_LBUTTON) {
		CRayTrace3DDoc* pDoc = GetDocument();
		if (pDoc->SceneLoaded()) {
			pDoc->MouseMoveTo(point.x, point.y);

			//refresh the screen
			Invalidate();
		}
	}
}


//this is used to reset the scene rotation parameters
void CRayTrace3DView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CRayTrace3DDoc* pDoc = GetDocument();
	if (pDoc->SceneLoaded())
		pDoc->MouseDownAt(-1, -1); //reset to negative
	
	CView::OnLButtonUp(nFlags, point);
}

BOOL CRayTrace3DView::ClosingTime()
{
	//if everything is exited correctly
	//return TRUE
	//clear the child window list and send them close messages
	SimpleList<CRTWindow*>::Manipulator cs(m_CRTWindowList);

	while (cs.Valid()) {
		CRTWindow* child_win = cs.Contents();
		child_win->SetActiveWindow();
		//say yes to external invoke since we don't want
		// the window to send us a close message back
		if (!child_win->ClosingTime(TRUE))
			return FALSE;
		else
			//since we called the OnClose handler for the 
			//window, delete it from the list
			cs.DeleteNode();
	}
	return TRUE;
}

LRESULT CRayTrace3DView::OnRTWindowDestroyed(WPARAM wParam, LPARAM lParam)
{
	CRTWindow* winRT = (CRTWindow*) wParam;

	SimpleList<CRTWindow*>::Manipulator cs(m_CRTWindowList);
	
	while (cs.Valid()) {
		//if the window that sent the message still exists in the list,
		// remove it
		if (winRT == cs.Contents()) {
			cs.DeleteNode();
			return 0;
		}
		else	// DeleteNode makes it point to the next anyway.
			cs.GotoNext();
    }
	return 0;
}

//Save the OpenGL output from the window
void
CRayTrace3DView::SaveScreenShotGL()
{	
	CRect rect;
	GetClientRect(&rect);
	int screenWidth = rect.Width();
	int screenHeight = rect.Height();

	//allocate space for the image
	unsigned char* imgData = new unsigned char[screenWidth * screenHeight * 3];

	if (imgData == NULL) {
		AfxMessageBox("Cannot save screenshot: not enough memory.");
		return;
	}

	// Read the pixels back.
	glReadPixels(0, 0, screenWidth, screenHeight, GL_RGB, GL_UNSIGNED_BYTE, imgData);

	//say no to automatic file open dialog
	ImageFile newImage(FALSE);

	//create a clean canvas
	newImage.Reset(screenWidth, screenHeight, RGBSample);

	//copy the rows to the image
	int bytesPerRow = screenWidth * RGBSample;
	
	//the buffer is bottom-top so feed the scanlines in
	// reversed order
	int i = screenHeight - 1;
	while (i >= 0) {
		newImage.PutScanline(&imgData[bytesPerRow * i]);
		i--;
	}

	//save the image letting the user choose the name
	newImage.SaveFileAs();

	delete [] imgData;
}

void CRayTrace3DView::OnFileSave() 
{
	SaveScreenShotGL();
}

void CRayTrace3DView::OnFileSaveAs() 
{
	SaveScreenShotGL();
}
