/****************************************************************************
 *	
 * 	3DSFile.cpp 				3/9/2000
 *	By Konstantin Khanyants 	kon@unix.temple.edu
 *	
 *	Description: 3D Studio implementation file
 *
 ***************************************************************************/

#include "stdafx.h"
#include "include/Oort.h"
#include "include/3DSPriv.h"
#include <memory.h>

inline short ReadByte(FILE* fp, char& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadUByte(FILE* fp, unsigned char& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadShort(FILE* fp, short& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadUShort(FILE* fp, unsigned short& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadLong(FILE* fp, long& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadULong(FILE* fp, unsigned long& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadFloat(FILE* fp, float& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}

inline short ReadDouble(FILE* fp, double& value) 
{return (fread(&value, sizeof(value), 1, fp) == 1);}


static int Read3DSChunk(FILE* fp, chunk3ds& chunk);
static int Read3DSString(FILE* fp, char* name, int len = 256);
static int ReadAndAllocString3ds(FILE* fp, char3ds*& str, ulong3ds length);
static int ReadPercentageChunk(FILE* fp, float& value);
static int ReadPercentage(FILE* fp, float& value, int tag = 0);
static int ReadColorChunk(FILE* fp, fcolor3ds& color);
static int ReadColor(FILE* fp, fcolor3ds& color, int type);
static int ReadTexVert3ds(FILE* fp, textvert3ds& t);
static int ReadPoint3ds(FILE* fp, point3ds& p);
static int ReadFace3ds(FILE* fp, face3ds& f);
static int Is3DSFile(FILE* fp);


// Read Chunk into Chunk structure
static int
Read3DSChunk(FILE* fp, chunk3ds& chunk)
{
	if (!ReadUShort(fp, chunk.tag)) return FALSE;
	if (!ReadULong(fp, chunk.len)) return FALSE;
	return TRUE;
}

// Read string and truncate if buffer is shorter than
// actual string.
static int 
Read3DSString(FILE* fp, char* name, int len)
{
	int c;

	for (int i=0; (c = fgetc(fp)) != EOF && c != '\0'; i++) {
		if (i < len) {
			name[i] = c;
		}
	}
	if (i < len) {
		name[i] = '\0';
	} else {
		name[len-1] = '\0';
	}

	return (c != EOF);
}


static int
ReadAndAllocString3ds(FILE* fp, char3ds*& str, ulong3ds length)
{
	char3ds *temp = NULL;
  //  ulong3ds len;
   
    temp = new char3ds[length+1];
   	if(temp == NULL) return FALSE;

   	if (!Read3DSString(fp, temp, length+1)) return FALSE;

   	str = new char3ds[strlen(temp)+1];
	if (str == NULL) return FALSE;

	strcpy(str, temp);
   	delete [] temp;
	
   	return TRUE;	
}	


// Read a Percent Chunk
static int
ReadPercentageChunk(FILE* fp, float& value)
{
	chunk3ds	chunk;
	long		chunkStart = ftell(fp);

	if (!Read3DSChunk(fp, chunk)) return FALSE;
	
	if (chunk.tag == INT_PERCENTAGE) {
		short	svalue;
		if (!ReadShort(fp, svalue)) return FALSE;
		value = (float) svalue/ (float) 100.0;
		return TRUE;
	} else if (chunk.tag == FLOAT_PERCENTAGE) {
		if (!ReadFloat(fp, value)) return FALSE;
		return TRUE;
	} 

	fseek(fp, chunkStart + chunk.len, SEEK_SET);
	return FALSE;
}


// Read a Percent Chunk
static int
ReadPercentage(FILE* fp, float& value, int tag)
{	
	if (tag == 0) {
		if (!ReadPercentageChunk(fp, value)) return FALSE;
	}	
	else if (tag == INT_PERCENTAGE) {
		short	svalue;
		if (!ReadShort(fp, svalue)) return FALSE;
		value = (float) svalue/ (float) 100.0;
		return TRUE;
	} 
	else if (tag == FLOAT_PERCENTAGE) {
		if (!ReadFloat(fp, value)) return FALSE;
		return TRUE;
	} 

	return FALSE;
}


// Read a color definition
static int
ReadColorChunk(FILE* fp, fcolor3ds& color)
{
	chunk3ds		chunk;
	long			chunkStart = ftell(fp);
	unsigned char	tmp;

	if (!Read3DSChunk(fp, chunk)) return FALSE;
	switch (chunk.tag) {
	
	case COLOR_F:
		if (!ReadFloat(fp, color.r)) return FALSE;
		if (!ReadFloat(fp, color.g)) return FALSE;
		if (!ReadFloat(fp, color.b)) return FALSE;
		break;
	
	case COLOR_24:
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.r = (float) tmp / (float) 255.0;
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.g = (float) tmp / (float) 255.0;
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.b = (float) tmp / (float) 255.0;
		break;
	
	default:
		fseek(fp, chunkStart + chunk.len, SEEK_SET);
		return FALSE;
	}

	return TRUE;
}

// Read a color definition
static int
ReadColor(FILE* fp, fcolor3ds& color, int type)
{
	unsigned char	tmp;

	switch (type) {
	
	case COLOR_F:
		if (!ReadFloat(fp, color.r)) return FALSE;
		if (!ReadFloat(fp, color.g)) return FALSE;
		if (!ReadFloat(fp, color.b)) return FALSE;
		break;
	
	case COLOR_24:
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.r = (float) tmp / (float) 255.0;
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.g = (float) tmp / (float) 255.0;
		if (!ReadUByte(fp, tmp)) return FALSE;
		color.b = (float) tmp / (float) 255.0;
		break;
	
	default:
		return FALSE;
	}

	return TRUE;
}


static int
ReadTexVert3ds(FILE* fp, textvert3ds& t)
{
   	memcpy(&t, &DefTextVert3ds, sizeof(textvert3ds));

   	if (!ReadFloat(fp, t.u) ||
   		!ReadFloat(fp, t.v))
   		return FALSE;
   		
   	return TRUE;	
}

static int
ReadPoint3ds(FILE* fp, point3ds& p)
{
    memcpy(&p, &DefPoint3ds, sizeof(point3ds));

    if (!ReadFloat(fp, p.x) || 
        !ReadFloat(fp, p.y) ||
   	    !ReadFloat(fp, p.z))
		return FALSE;
	
	return TRUE;		   	   	   
}


static int
ReadFace3ds(FILE* fp, face3ds& f)
{
 	memcpy(&f, &DefFace3ds, sizeof(face3ds));

   	if (!ReadUShort(fp, f.vert_index[0]) ||
   		!ReadUShort(fp, f.vert_index[1]) ||
   		!ReadUShort(fp, f.vert_index[2]) ||
		!ReadUShort(fp, f.flag))
   		return FALSE;
	
	return TRUE;
}

////////////////////////////////////////////////////////////////


int
C3DStudio::ReadTexVerts(TexVerts& textVerts, file3ds& theFile)
{
    if (!ReadUShort(theFile.fp, textVerts.numcoords)) return FALSE;
  
    textVerts.textvertlist = new textvert3ds[textVerts.numcoords]; //calloc(theTexVerts.numcoords, sizeof(textvert3ds));
    
    if (textVerts.textvertlist == NULL) return FALSE;
    memset(textVerts.textvertlist, 0, textVerts.numcoords * sizeof(textvert3ds));
    
    //Read the texture coordinates
    for (ushort3ds i = 0; i < textVerts.numcoords; i++){
       	if (!ReadTexVert3ds(theFile.fp, textVerts.textvertlist[i]))
       		return FALSE;
    }

	return TRUE;
}


// Read vertices
int
C3DStudio::ReadPointArray(PointArray& vertexList, file3ds& theFile) 
{	
	if (!ReadUShort(theFile.fp, vertexList.vertices)) return FALSE;

    vertexList.pointlist = new point3ds[vertexList.vertices];
    
    if (vertexList.pointlist == NULL) return FALSE;
    memset(vertexList.pointlist, 0, vertexList.vertices * sizeof(point3ds));

    // Read the vertices
    for (ushort3ds i = 0; i < vertexList.vertices; i++) {
    	if (!ReadPoint3ds(theFile.fp, vertexList.pointlist[i])) return FALSE;
    }
    
	return TRUE;
}

// Read polygons
int
C3DStudio::ReadFaceArray(FaceArray& faceList, file3ds& theFile) 
{
	// Read the count
	if (!ReadUShort(theFile.fp, faceList.faces)) return FALSE;

	faceList.facelist = new face3ds[faceList.faces];
	
	if (faceList.facelist == NULL) return FALSE;
    memset(faceList.facelist, 0, faceList.faces * sizeof(face3ds));
    
    // Read the faces
    for (ushort3ds i = 0; i < faceList.faces; i++) {
    	if (!ReadFace3ds(theFile.fp, faceList.facelist[i])) return FALSE;
    }

	return TRUE;
}


// Read the materials used by the group
int
C3DStudio::ReadMeshMatGroup(objmat3ds*& meshMat, file3ds& theFile) 
{
	meshMat = new objmat3ds;
	memset(meshMat, 0, sizeof(objmat3ds));

	// Read the material name
	if (!ReadAndAllocString3ds(theFile.fp, meshMat->name, 256)) return FALSE;

	// Read the number of faces to map
	if (!ReadUShort(theFile.fp, meshMat->nfaces)) return FALSE;

	meshMat->faceindex = new ushort3ds[meshMat->nfaces];
	
	if (meshMat->faceindex == NULL) return FALSE;
    memset(meshMat->faceindex, 0, meshMat->nfaces * sizeof(ushort3ds));
    
    for (ushort3ds i = 0; i < meshMat->nfaces; i++) {
       	if (!ReadUShort(theFile.fp, meshMat->faceindex[i])) return FALSE;
    }
	return TRUE;
}


int
C3DStudio::ReadSmoothGroup(SmoothGroup& theSmGroup, long chunkSize, file3ds& theFile)
{         
    theSmGroup.groups = (ushort3ds) ((chunkSize - 6)/4);
    theSmGroup.grouplist = new ulong3ds[theSmGroup.groups];
    
    if (theSmGroup.grouplist == NULL) return FALSE;
    memset(theSmGroup.grouplist, 0, theSmGroup.groups * sizeof(ulong3ds));
    
    for (ushort3ds i = 0; i < theSmGroup.groups; i++) {
       	if (!ReadULong(theFile.fp, theSmGroup.grouplist[i])) return FALSE;
    }

	return TRUE;
}

int
C3DStudio::ReadMeshMatrix(file3ds& theFile, float3ds* matrix)
{   
    for (int i = 0; i < 12; i++)
		if (!ReadFloat(theFile.fp, matrix[i])) return FALSE;

	return TRUE;
}

// Read the Mesh
int
C3DStudio::ReadTriObject(mesh3ds*& newMesh, long triStart, long triSize, file3ds& theFile)
{
	chunk3ds			chunk;
	long				chunkStart = ftell(theFile.fp);

	TexVerts			textVerts;
	PointArray			vertexList;
	FaceArray			faceList;
//	MshMatGroup 		meshMat;
	objmat3ds			*meshMat = NULL;
	SmoothGroup 		smoothGroup;
//	SimpleList<objmat3ds> matList;
	SimpleList<objmat3ds *> matList;

	newMesh = new mesh3ds;	//get new mesh
	//clear the contents
	memset(newMesh, 0, sizeof(mesh3ds));

	while (chunkStart < triStart + triSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case TEX_VERTS:
      		if (!ReadTexVerts(textVerts, theFile)) return FALSE;
      		newMesh->ntextverts = textVerts.numcoords;
      		newMesh->textarray = textVerts.textvertlist;
      		break;
		
		case POINT_ARRAY:
			if (!ReadPointArray(vertexList, theFile)) return FALSE;
			newMesh->nvertices = vertexList.vertices;
			newMesh->vertexarray = vertexList.pointlist;
			break;
		
		case FACE_ARRAY:
			if (!ReadFaceArray(faceList, theFile)) return FALSE;
			newMesh->nfaces = faceList.faces;
			newMesh->facearray = faceList.facelist;
			break;
		
		case MESH_COLOR:	//redundant really
		    if (!ReadUByte(theFile.fp, newMesh->meshcolor)) return FALSE;
     		break;	

		case MESH_MATRIX:  //gets the orientation matrix
			if (!ReadMeshMatrix(theFile, newMesh->locmatrix)) return FALSE;
			break;

		case MSH_MAT_GROUP:
			if (!ReadMeshMatGroup(meshMat, theFile)) return FALSE;
			matList.AppendToList(meshMat);//(objmat3ds) meshMat);
			break;
			
		case SMOOTH_GROUP:
			if (!ReadSmoothGroup(smoothGroup, chunk.len, theFile)) return FALSE;
			newMesh->smootharray = smoothGroup.grouplist;
			break;	
		
		case OBJ_HIDDEN:
			newMesh->ishidden = TRUE;
			break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}

	/* Put the material in the List */
	
	//get the number of materials per object
	newMesh->nmats = (ushort3ds) matList.ListCount();
	
	//get room for the material array
	newMesh->matarray = new objmat3ds[newMesh->nmats];
	if (newMesh->matarray == NULL) return FALSE;
	
	//get the material names from the list and put 
	// them into the mesh's matarray
	int count = 0;
//	for (SimpleList<objmat3ds>::Iterator matObj(matList); matObj.Valid(); matObj.GotoNext())
//		newMesh->matarray[count++] = matObj.Contents();

	while (matList.ListCount()) {
		meshMat = matList.ExtractHead();
		newMesh->matarray[count++] = *meshMat;
		delete meshMat;
	}	
		//newMesh->matarray[count++] = matList.ExtractHead();

	return TRUE;
	
}


int
C3DStudio::ReadCamera(camera3ds*& camera, long camStart, long camSize, file3ds& theFile)
{
	camera = new camera3ds;
	memset(camera, 0, sizeof(camera3ds));	//clear memory
	
    // read all of the fields and search for the ranges
    if (!ReadPoint3ds(theFile.fp, camera->position) ||
     	!ReadPoint3ds(theFile.fp, camera->target) ||	
     	!ReadFloat(theFile.fp, camera->roll) ||		
     	!ReadFloat(theFile.fp, camera->focallength)) 
  		return FALSE;
  
  	//fov set like in the toolkit because the value read is camera focal length
  	camera->fov = 2400.0f / camera->focallength;
  
  	long		chunkStart = ftell(theFile.fp);
	chunk3ds	chunk;
	
  	while (chunkStart < camStart + camSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
  
	  	case CAM_RANGES:
     		if (!ReadFloat(theFile.fp, camera->ranges.cam_near) ||
     			!ReadFloat(theFile.fp, camera->ranges.cam_far))
     			return FALSE;
     		break;
  		
  		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}
	
  	return TRUE;
}


int
C3DStudio::ReadSpotLight(spotlight3ds*& spotLight, long spotStart, long spotSize, file3ds& theFile)
{
	chunk3ds	chunk;
	long		chunkStart;

	spotLight = new spotlight3ds;	//make a new spotlight
	if (spotLight == NULL) return FALSE;
	memset(spotLight, 0, sizeof(spotlight3ds));
			
	if (!ReadPoint3ds(theFile.fp, spotLight->target) || 
    	!ReadFloat(theFile.fp, spotLight->hotspot) || //hotspotangle
     	!ReadFloat(theFile.fp, spotLight->falloff))   //falloffangle  
		return FALSE;

	chunkStart = ftell(theFile.fp);

	while (chunkStart < spotStart + spotSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case DL_SPOT_ROLL:
      		if (!ReadFloat(theFile.fp, spotLight->roll)) return FALSE;
     		break;
      
    	case DL_SPOT_ASPECT:
     	 	if (!ReadFloat(theFile.fp, spotLight->aspect)) return FALSE;
     		break;
     
      	case DL_SPOT_PROJECTOR:
     		if (!ReadAndAllocString3ds(theFile.fp, spotLight->projector.bitmap, 256)) return FALSE;
     		break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}

	return TRUE;
}


int
C3DStudio::ReadLight(light3ds*& light, long lightStart, long lightSize, file3ds& theFile)
{
	chunk3ds	chunk;
	long		chunkStart;
	spotlight3ds *spotLight = NULL;
	
	light = new light3ds;
	memset(light, 0, sizeof(light3ds)); //clear the memory
	
	//get the position of the light source
	if (!ReadPoint3ds(theFile.fp, light->pos)) return FALSE;

	chunkStart = ftell(theFile.fp);
	
	while (chunkStart < lightStart+lightSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case DL_SPOTLIGHT:
			if (!ReadSpotLight(spotLight, chunkStart, chunk.len, theFile)) return FALSE;
			light->spot = spotLight;
			break;
		
		case COLOR_F:
			if (!ReadColor(theFile.fp, light->color, COLOR_F)) return FALSE;
			break;
			
		case COLOR_24:
			if (!ReadColor(theFile.fp, light->color, COLOR_24)) return FALSE;
			break;	
		
		case DL_OFF:
			light->dloff = TRUE;
			break;
		
		case DL_ATTENUATE:
			light->attenuation.on = TRUE;
			break;	
			
	    case DL_INNER_RANGE:
  			if (!ReadFloat(theFile.fp, light->attenuation.inner)) return FALSE;
     		break;
      
      	case DL_OUTER_RANGE:
      		if (!ReadFloat(theFile.fp, light->attenuation.outer)) return FALSE;
     		break;
     
      	case DL_MULTIPLIER:	/* The light intensity */
     		if (!ReadFloat(theFile.fp, light->multiplier)) return FALSE;
     		break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}

	return TRUE;
}


// Read the named object: mesh, light, camera
int
C3DStudio::ReadNamedObject(long namedStart, long namedSize, file3ds& theFile)
{
	char3ds*	groupName = NULL;
	chunk3ds	chunk;
	long		chunkStart;
	camera3ds	*camera = NULL;
	light3ds	*light = NULL;
	mesh3ds		*triMesh = NULL;

	if (!ReadAndAllocString3ds(theFile.fp, groupName, 256)) return FALSE;

	// Sanitize the string
//	for (unsigned int i = 0; i < strlen(groupName); i++) {
//		if (!isalnum(groupName[i])) {
//			groupName[i] = '_';
//		}
//	}

//	root->StatusMessage("Reading object '%s'", groupName);

	chunkStart = ftell(theFile.fp);

	while (chunkStart < namedStart+namedSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case N_TRI_OBJECT:
			if (!ReadTriObject(triMesh, chunkStart, chunk.len, theFile)) return FALSE;
			triMesh->name = groupName;
			m_MeshList.AppendToList(triMesh);
			break;
		
		case N_DIRECT_LIGHT:
			if (!ReadLight(light, chunkStart, chunk.len, theFile)) return FALSE;
			light->name = groupName;
			m_LightList.AppendToList(light);
			break;	
		
		case N_CAMERA:
			if (!ReadCamera(camera, chunkStart, chunk.len, theFile)) return FALSE;
			camera->name = groupName;
			m_CameraList.AppendToList(camera);
			break;
			
		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}

	return TRUE;
}


int
C3DStudio::ReadTextureMap(bitmap3ds*& bitmap, long texStart, long texSize, file3ds& theFile)
{
	long			chunkStart = ftell(theFile.fp);
	chunk3ds		chunk;
	short3ds		actualBumpPercent = 0;	
	ushort3ds		tiling;

	bitmap = new bitmap3ds;
	if (bitmap == NULL) return FALSE;
	memset(bitmap, 0, sizeof(bitmap3ds));

	while (chunkStart < texStart + texSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {

		case INT_PERCENTAGE:
	 		if (!ReadPercentage(theFile.fp, bitmap->percent, INT_PERCENTAGE)) return FALSE;
	 		break;

	 	case FLOAT_PERCENTAGE:
			if (!ReadPercentage(theFile.fp, bitmap->percent, FLOAT_PERCENTAGE)) return FALSE;
			break;
		
		case MAT_MAPNAME:
			if (!ReadAndAllocString3ds(theFile.fp, bitmap->name, 256)) return FALSE;
			break;
		
		case MAT_MAP_TILING:
			if (!ReadUShort(theFile.fp, tiling)) return FALSE;
			if (tiling & TEX_DECAL) {
				if (tiling & TEX_NOWRAP) 
					bitmap->tiling = Decal;
				else 
					bitmap->tiling = Both;
			}
			else 
				bitmap->tiling = Tile;
			
			bitmap->ignorealpha = (char3ds)((tiling & TEX_DONT_USE_ALPHA) == TEX_DONT_USE_ALPHA);

			if (tiling & TEX_SAT) bitmap->filter = SummedArea;
			else bitmap->filter = Pyramidal;

			bitmap->mirror = (char3ds)((tiling & TEX_MIRROR) == TEX_MIRROR);
			bitmap->negative = (char3ds)((tiling & TEX_INVERT) == TEX_INVERT);

			if (tiling & TEX_TINT) {
				if (tiling & TEX_ALPHA_SOURCE) bitmap->source = AlphaTint;
				else bitmap->source = RGBLumaTint;
			}
			else {
				if (tiling & TEX_RGB_TINT) bitmap->source = RGBTint;
  				else if (tiling & TEX_ALPHA_SOURCE) bitmap->source = Alpha;
				else bitmap->source = RGB;
			}
			break;
		
		case MAT_MAP_USCALE:
			if (!ReadFloat(theFile.fp, bitmap->uscale)) return FALSE;
			break;
		
		case MAT_MAP_VSCALE:
			if (!ReadFloat(theFile.fp, bitmap->vscale)) return FALSE;
			break;
		
		case MAT_MAP_ANG:	
			if (!ReadFloat(theFile.fp, bitmap->rotation)) return FALSE;
			break;
		
		case MAT_MAP_UOFFSET:
			if (!ReadFloat(theFile.fp, bitmap->uoffset)) return FALSE;
			break;
		
		case MAT_MAP_VOFFSET:
			if (!ReadFloat(theFile.fp, bitmap->voffset)) return FALSE;
			break;
		
		case MAT_BUMP_PERCENT:
			if (!ReadShort(theFile.fp, actualBumpPercent)) return FALSE;
			bitmap->percent = (float) (actualBumpPercent) / 100.0f;
			break;
				
		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}
	return TRUE;
}



// Read a material definition, add it to the material dictionary.
int
C3DStudio::ReadMatEntry(long matStart, long matSize, file3ds& theFile)
{
	long			chunkStart = ftell(theFile.fp);
	chunk3ds		chunk;
	
	material3ds		*newMat = new material3ds;
	bitmap3ds		*bitmap = NULL;

	if (newMat == NULL) return FALSE;
	memset(newMat, 0, sizeof(material3ds));

	while (chunkStart < matStart + matSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case MAT_NAME:
			if (!ReadAndAllocString3ds(theFile.fp, newMat->name, 256)) return FALSE;
//			root->StatusMessage("Reading material '%s'", name);
			break;
		
		case MAT_SHADING:
			short3ds shade_type;
			if (!ReadShort(theFile.fp, shade_type)) return FALSE;
			newMat->shading = (shadetype3ds) shade_type; 
			break;

		case MAT_AMBIENT:
			if (!ReadColorChunk(theFile.fp, newMat->ambient)) return FALSE;
			break;
		
		case MAT_DIFFUSE:
			if (!ReadColorChunk(theFile.fp, newMat->diffuse)) return FALSE;
			break;
		
		case MAT_SPECULAR:
			if (!ReadColorChunk(theFile.fp, newMat->specular)) return FALSE;
			break;
		
		case MAT_SHININESS:
			if (!ReadPercentageChunk(theFile.fp, newMat->shininess)) return FALSE;
			break;

		case MAT_SHIN2PCT:
			if (!ReadPercentageChunk(theFile.fp, newMat->shinstrength)) return FALSE;
			break;

		case MAT_TRANSPARENCY:
			if (!ReadPercentageChunk(theFile.fp, newMat->transparency)) return FALSE;
			break;
		
		case MAT_TEXMAP:
			bitmap = new bitmap3ds;
			if (!ReadTextureMap(bitmap, chunkStart, chunk.len, theFile)) return FALSE;
			newMat->texture.map = bitmap;
			break;
		
		case MAT_BUMPMAP:
			bitmap = new bitmap3ds;
			if (!ReadTextureMap(bitmap, chunkStart, chunk.len, theFile)) return FALSE;
			newMat->bump.map = bitmap;
			break;
					
		case MAT_TWO_SIDE:
			newMat->twosided = TRUE;
			break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}
	
	//add the material to the list
	m_MaterialList.AppendToList(newMat);

	return TRUE;
}

/* Searches an MDATA chunk looking for a DEFAULT_VIEW chunk. *vp should allready be
   initialized before passing it to this function */
/*
int
C3DStudio::ReadDefaultView(long viewStart, long viewSize, file3ds& theFile)
{
	long			chunkStart = ftell(theFile.fp);
	chunk3ds		chunk;
	
	viewport3ds&  viewPort = m_ViewPort;
	
	while (chunkStart < viewStart + viewSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case VIEW_TOP:
			viewPort.type = TopView3ds;
			goto GetPort;
    	case VIEW_BOTTOM:
    		viewPort.type = BottomView3ds;
			goto GetPort;
    	case VIEW_LEFT:
    		viewPort.type = LeftView3ds;
			goto GetPort;
    	case VIEW_RIGHT:
    		viewPort.type = RightView3ds;
			goto GetPort;
    	case VIEW_FRONT:
    		viewPort.type = FrontView3ds;
			goto GetPort;
    	case VIEW_BACK:
    		viewPort.type = BackView3ds;
    		
    	GetPort:
    		ViewStandard sdata;
    	 	if (!ReadFloat(theFile.fp, sdata.viewwidth) ||
     			!ReadPoint3ds(theFile.fp, sdata.viewtargetcoord))
     			return FALSE;
     		
	        viewPort.ortho.center.x = sdata.viewtargetcoord.x;
	        viewPort.ortho.center.y = sdata.viewtargetcoord.y;
	        viewPort.ortho.center.z = sdata.viewtargetcoord.z;
	        viewPort.ortho.zoom = sdata.viewwidth / (float3ds)viewPort.size.width;
	        break;
     		
    	case VIEW_USER:
    	 	
    	 	ViewUser udata;
     		if (!ReadFloat(theFile.fp, sdata.viewwidth) ||
     		  	!ReadFloat(theFile.fp, sdata.xyviewangle) ||
     		  	!ReadFloat(theFile.fp, sdata.yzviewangle) ||
     			!ReadFloat3ds(theFile.fp, sdata.bankangle) ||
     			!ReadPoint3ds(theFile.fp, sdata.viewtargetcoord))
    	 		return FALSE;
    	 	
    	 	viewPort.type = UserView3ds;
	       	viewPort.user.center.x = udata.viewtargetcoord.x;
	       	viewPort.user.center.y = udata.viewtargetcoord.y;
	       	viewPort.user.center.z = udata.viewtargetcoord.z;
	       	viewPort.user.zoom = udata.viewwidth / (float3ds)viewPort.size.width;
	       	viewPort.user.horang = udata.xyviewangle;
	       	viewPort.user.verang = udata.yzviewangle;
	       	break;
    	 	
    	case VIEW_CAMERA:
     		
     		ViewCamera cdata;
     		ReadAndAllocString3ds(theFile.fp, cdata.name, 256);
     		viewPort.camera.name = cdata.name;
     		break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}
	return TRUE;
} 
*/


// Read toplevel objects
int 
C3DStudio::ReadMDATA(long mdataStart, long mdataSize, file3ds& theFile)
{
	long			chunkStart = ftell(theFile.fp);
	chunk3ds		chunk;

	while (chunkStart < mdataStart + mdataSize &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case MESH_VERSION:
			if (!ReadULong(theFile.fp, m_Version)) return FALSE;
			break;
		
		case MAT_ENTRY:
			if (!ReadMatEntry(chunkStart, chunk.len, theFile)) return FALSE;
			break;
		
		case MASTER_SCALE:
			if (!ReadFloat(theFile.fp, m_Scale)) return FALSE;
			break;
		
		case NAMED_OBJECT:
			if (!ReadNamedObject(chunkStart, chunk.len, theFile)) return FALSE;
			break;
				
//		case DEFAULT_VIEW:
//			if (!ReadDefaultView(chunkStart, chunk.len, theFile)) return FALSE;
//			break;
		
		case SOLID_BGND:
			if (!ReadColorChunk(theFile.fp, m_BackColor)) return FALSE;
			break;

		case AMBIENT_LIGHT:
			if (!ReadColorChunk(theFile.fp, m_AmbientLight)) return FALSE;
			break;

		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}
	return TRUE;
}

// Read 3ds file
int
C3DStudio::Read3DSData(long fileStart, long fileLen, file3ds& theFile)
{
	long			chunkStart = ftell(theFile.fp);
	chunk3ds		chunk;
	unsigned long	version;

	while (chunkStart < fileStart + fileLen &&
		Read3DSChunk(theFile.fp, chunk)) {
		switch (chunk.tag) {
		
		case M3D_VERSION:
			if (!ReadULong(theFile.fp, version)) return FALSE;
			break;
		
		case MDATA:
//			InitViewPort();	//initialize the viewport 
			if (!ReadMDATA(chunkStart, chunk.len, theFile)) return FALSE;
			break;
		
		default:
			// Skip past unexpected chunks
			fseek(theFile.fp, chunkStart + chunk.len, SEEK_SET);
		}
		chunkStart = ftell(theFile.fp);
	}

	return TRUE;
}

// Verifies that the current file is a 3ds file.
static int 
Is3DSFile(FILE* fp)
{
	chunk3ds	chunk;
	long		pos = ftell(fp);

	if (!Read3DSChunk(fp, chunk)) {
		fseek(fp, pos, SEEK_SET);
		return FALSE;
	}

	fseek(fp, pos, SEEK_SET);
	return (chunk.tag == M3DMAGIC);
}
/*
static void
InitViewPort()
{
	//initialize the viewport
	m_ViewPort.type = TopView3ds;
   	m_ViewPort.ortho.center.x = 
   	m_ViewPort.ortho.center.y = viewPort.ortho.center.z = 0.0f;
   	m_ViewPort.ortho.zoom = 0.7395f;
   	m_ViewPort.user.center.x = viewPort.user.center.y = viewPort.user.center.z = 0.0f;
   	m_ViewPort.user.zoom = 0.7395f;
   	m_ViewPort.user.horang = 20.0f;
    m_ViewPort.user.verang = 30.0f;
   	m_ViewPort.camera.name = NULL;
   	m_ViewPort.size.xpos = 0 = viewPort.size.ypos = 0;
   	m_ViewPort.size.width = viewPort.size.height = 1000;
}   	
*/   	

// External entry for reading 3ds file.
BOOL
C3DStudio::Read3DSFile(CString file_name)
{
	CString		pcsMsg;
	file3ds		theFile;
	chunk3ds	chunk;
	FILE* fp;
	
	// Open 3DS file (remember it s binary!)
	if ((fp = fopen(file_name, "rb")) != NULL) {
		ulong3ds	chunkStart = ftell(fp);
		theFile.fp = fp;		//assign the file pointer to our file
		
		// Get File Size
		fseek(fp, 0, SEEK_END);
		theFile.size = ftell(fp);
		fseek(fp, 0, SEEK_SET);

		// Verify file type
		if (!Is3DSFile(fp)) {
			pcsMsg = "File ";
			pcsMsg += file_name;
			pcsMsg += " is not a 3ds binary file.";
			AfxMessageBox(pcsMsg);
			return FALSE;
		}

		//Clear the database
		DestroyDatabase();

		//get the filename
//		int	nameLen = strlen(file_name);		
//		m_Filename = new char[nameLen+1];	//copy to the class member
//		strcpy(m_Filename, file_name);	
		m_Filename = file_name;
		
		// Loop through the chunks
		while (chunkStart < theFile.size &&
			Read3DSChunk(fp, chunk)) {
			switch (chunk.tag) {
			case M3DMAGIC:
				if (!Read3DSData(chunkStart, chunk.len, theFile)) {
					pcsMsg = "Read failed for 3ds file ";
					pcsMsg += file_name;
					AfxMessageBox(pcsMsg);
					fclose(fp);
					return FALSE;
				}
				break;
			default:
				// Skip past unexpected chunks
				fseek(fp, chunkStart + chunk.len, SEEK_SET);
			}
			chunkStart = ftell(fp);
		}
		fclose(fp);
		return TRUE; //successful return 
	}
	return FALSE;
}
