// ===================================================================
//  File: aggreg.cpp			12/25/1999
//	By Konstantin Khanyants
//	Description:
//		Implementation of Aggregate objects--lists of objects that
//		are associated with each other, e.g. the polygons that a
//		model imported from a 3D Studio file is made of.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include "include/Oort.h"

Aggregate::Aggregate(const Aggregate& x): Object3D(AGGREGATE), list(x.list) 
{ 
	name = x.name;
}


Aggregate::Aggregate(const ObjectList& List, CString xname): Object3D(AGGREGATE)
{
    list = ObjectList(List);
	name = xname;
}

int
Aggregate::NearestInt(Ray& ray, float& t, float maxt)
{
//REM    cerr << "Aggregate::NearestInt called\n";
//    exit(1);
    return 0;
}

SpanList *
Aggregate::FindAllIntersections(Ray& ray)
{
//REM    cerr << "Aggregate::FindAllIntersections called\n";
//    exit(1);
    return 0;
}

void
Aggregate::Describe()
{
	AfxMessageBox(CString("Aggregate Object = ") + name);
	for (ObjectList::Iterator sc(list); sc.Valid(); sc.GotoNext())
		sc.Contents()->Describe();
}

Vector3D
Aggregate::FindNormal(const Vector3D& x)
{
//REM    cerr << "Error: Aggregate::FindNormal called\n";
//    exit(1);
    return 0;
}

void
Aggregate::ApplyTransform(const Matrix& tform)
{
    list.TransformList(tform);
}

Object3D *
Aggregate::Dup() const
{
	ObjectList newList;
	for (ObjectList::Iterator cs((SimpleList<Object3D *> &)list); cs.Valid(); cs.GotoNext())
		newList.AppendToList(cs.Contents()->Dup());

    return new Aggregate(newList, name);
}

float
Aggregate::PtDistance(Vector3D& x)
{
//REM    cerr << "Error: Aggregate::PtDistance called\n";
//    exit(1);
    return 0;
}

AxisAlignedBox
Aggregate::BBox() const
{
//REM PUT IN ERROR STRINGS    cerr << "Error: Aggregate::BBox() called\n";
//REM    exit(0);
	return AxisAlignedBox(Vector3D(0), Vector3D(0));
}

void 
Aggregate::SetName(CString newName)
{
//	name = new char[strlen(newName)+1];
//	strcpy(name, newName);
	name = newName;
}


void
Aggregate::AddObject(Object3D *newobj)
{
    list.AddToList(newobj);
}

void
Aggregate::AddObject(const Object3D& newobj)
{
    list.AddToList(newobj.Dup());
}

void
Aggregate::AppendObject(Object3D *newobj)
{
    list.AppendToList(newobj);
}

void
Aggregate::AppendObject(const Object3D& newobj)
{
    list.AppendToList(newobj.Dup());
}

Object3D*
Aggregate::ObjectAt(int index)
{
	return list[index];
}


void
Aggregate::CountMe() const
{
//REM PUT IN ERROR STRINGS   cerr << "Error: Aggregate::CountMe called\n";
//REM    exit(0);
}

void
Aggregate::NewSurface(HallSurface *newsurf)
{
    for (ObjectList::Iterator sc(list); sc.Valid(); sc.GotoNext())
	sc.Contents()->OverrideColor(newsurf);
}