
#include "stdafx.h"
#include "include/Oort.h"
#include "include/World.h"

class VectorListGL : public SimpleList<point3ds *> {
public:
	VectorListGL() : SimpleList<point3ds *>() { }
	virtual void AppendToList(point3ds& vert);
	virtual void AppendNULLEntry();
	virtual point3ds* operator[] (int inx);
};

void
VectorListGL::AppendToList(point3ds& vert)
{
	point3ds* newPoint = new point3ds;
	memcpy(newPoint, &vert, sizeof(point3ds));
	SimpleList<point3ds *>::AppendToList(newPoint);
}

void
VectorListGL::AppendNULLEntry()
{
	point3ds* nullPt = new point3ds;
	memset(nullPt, 0, sizeof(point3ds));
	SimpleList<point3ds *>::AppendToList(nullVect);
}

point3ds*
VectorListGL::operator[] (int inx) 
{
    if (inx >= n)
		inx = n - 1;
	if (inx < 0)
        inx = 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return pNode->contents;
	}
	return 0;
}
//////////////////////////////////////////////////////////////////

class TexListGL : public SimpleList<TexPointGL *> {
public:
	virtual void AppendToList(textvert3ds& texPt3ds);
	virtual void AppendToList(TexPointGL* ptGL);
	virtual TexPointGL* operator[] (int inx);
};

void
TexListGL::AppendToList(TexPointGL* ptGL)
{ 
	TexPointGL* newPt = new TexPointGL;
	*newPt = *ptGL;
	SimpleList<TexPointGL *>::AppendToList(newPt); 
}

void
TexListGL::AppendToList(textvert3ds& texPt3ds)
{
	TexPointGL* texPtGL = new TexPointGL;
	texPtGL->s = texPt3ds.u;
	texPtGL->t = texPt3ds.v;

	SimpleList<TexPointGL *>::AppendToList(texPtGL);
}

TexPointGL*
TexListGL::operator[] (int inx) 
{
    if (inx >= n)
		inx = n - 1;
	if (inx < 0)
        inx = 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return pNode->contents;
	}
	return 0;
}

//////////////////////////////////////////////////////////////////

class MarkedList : public SimpleList<BOOL *> {
public:
	MarkedList() : SimpleList<BOOL *>() { }
	virtual void AppendToList(BOOL state);
	virtual BOOL* operator[] (int inx);
};

void
MarkedList::AppendToList(BOOL state)
{
	BOOL* bPtr = new BOOL;
	*bPtr = state;
	SimpleList<BOOL *>::AppendToList(bPtr);
}

BOOL*
MarkedList::operator[] (int inx) 
{
	ModifyCheck();
    if (inx >= n)
		inx = n - 1;
	if (inx < 0)
        inx = 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return pNode->contents;
	}
	return 0;  //if you get to this you've really messed up
}

class SmoothList : public SimpleList<long *> {
public:
	SmoothList() : SimpleList<long *>() { }
	void AppendNULLEntry();
	long GetSmoothIndex(int thisIndex, unsigned long smoothingGroup);
	void StoreSmoothIndex(int thisIndex, unsigned long smoothingGroup, int newIndex);

};

void
SmoothList::AppendNULLEntry()
{
//	SmoothEntry entry;
	long *entry = new long[32];
	memset(entry, 0xFF, sizeof(long) * 32);
	AppendToList(entry);
}

// Retrieves the vertex index for the given index and smoothing group.
// This redirection is necessary because a vertex might have been duplicated.
long
SmoothList::GetSmoothIndex(int thisIndex, unsigned long smoothingGroup)
{
	ModifyCheck();
    if (thisIndex >= n)
		thisIndex = n - 1;
	if (thisIndex < 0)
        thisIndex = 0;
    
	int smoothIndex = -1;

	//convert smoothing group into index
	while (smoothingGroup > 0) {
		smoothingGroup = smoothingGroup >> 1;
		smoothIndex++;
	}

	SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != thisIndex) {
    	 	pNode = pNode->next;
			i++;
		}
		return (pNode->contents[smoothIndex]);
		
	}
	return -1;
}

// Stores new vertex index (NewIndex) into the smooth index array of vertex ThisIndex
// using field SmoothingGroup, which must not be 0.
// For each vertex in the vertex array (also for duplicated vertices) an array of 32 cardinals
// is maintained (each for one possible smoothing group. If a vertex must be duplicated because
// it has no smoothing group or a different one then the index of the newly created vertex is
// stored in the SmoothIndices to avoid loosing the conjunction between not yet processed vertices
// and duplicated vertices.
// Note: Only one smoothing must be assigned per vertex. Some available models break this rule and
//       have more than one group assigned to a face.
void
SmoothList::StoreSmoothIndex(int thisIndex, unsigned long smoothingGroup, int newIndex)
{
//	ModifyCheck();
    if (thisIndex >= n)
		thisIndex = n - 1;
	if (thisIndex < 0)
        thisIndex = 0;
    
	int smoothIndex = -1;

	//convert smoothing group into index
	while (smoothingGroup > 0) {
		smoothingGroup = smoothingGroup >> 1;
		smoothIndex++;
	}

	SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != thisIndex) {
			pNode = pNode->next;
			i++;
		}
		pNode->contents[smoothIndex] = newIndex;	
	}
}


static point3ds VectorSubtract(point3ds& p1, point3ds& p2)
{
	point3ds result;

	result.x = p1.x - p2.x;
	result.y = p1.y - p2.y;
	result.z = p1.z - p2.z;
	
	return result;
}

static point3ds VectorAdd(point3ds& p1, point3ds& p2)
{
	point3ds result;

	result.x = p1.x + p2.x;
	result.y = p1.y + p2.y;
	result.z = p1.z + p2.z;
	
	return result;
}

static point3ds VectorCrossProduct(point3ds& p1, point3ds& p2)
{
	point3ds result;

	result.x = p1.y * p2.z - p1.z * p2.y;
	result.y = p1.z * p2.x - p1.x * p2.z;
	result.z = p1.x * p2.y - p1.y * p2.x;

	return result;
}

static point3ds VectorNormalize(point3ds& vec)
{
	point3ds result;
	float magnitude = (float) sqrt(vec.x * vec.x + vec.y * vec.y + vec.z * vec.z);
    
	result.x = vec.x / magnitude;
	result.y = vec.y / magnitude;
	result.z = vec.z / magnitude;

	return result;
}

//---------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////
//ObjectList must be clean before this. Surfaces must be assigned
//meshListGL must also be empty
void
C3DStudio::ConvertMeshData(MeshListGL& meshListGL, ObjectList& objects, SurfaceList& surfaces)
{
	VectorListGL				vertListGL;
	TexListGL					texListGL;
	VectorListGL				normalListGL;
	SmoothList					smoothList;
	MarkedList					markedList;

	unsigned long smoothGroup;
	point3ds vector1, vector2, normal;
	int currentIndex; //currVertex, 
	int currentVertexCount;
	face3ds* FaceArray = NULL;
	ushort3ds currFace;
	mesh3ds* currMesh;
	const BOOL markerFALSE = FALSE;

	//set the version of the 3DS file database
	meshListGL.SetVersion(m_Version);

	//set the ambient and background colors of the imported scene
	meshListGL.SetBackColor(RGBColor(m_BackColor));
	meshListGL.SetAmbientLight(RGBColor(m_AmbientLight));

	int i = 0;	//LCV

	//now fill in the meshGL structure
	for (SimpleList<mesh3ds *>::Iterator mesh(m_MeshList); mesh.Valid(); mesh.GotoNext()) {
		//get the current 3DStudio mesh
        currMesh = mesh.Contents();

		if (currMesh->nvertices < 3) {
			AfxMessageBox("Unrecoverable error during conversion");
			// unbelievable, what terrible objects sometimes can be found
			exit(1);
		}

		//get space for one mesh object and clear it
		MeshGL* meshGL = new MeshGL;
//		memset(meshGL, 0, sizeof(MeshGL));
		meshGL->ListName = 0;
		meshGL->Vertices = NULL;
		meshGL->Normals = NULL;
		meshGL->TexCoords = NULL;
		
		//get the number of vertices in the mesh, some may be added
		currentVertexCount = currMesh->nvertices;

		//get the name of the new mesh from 3DStudio's mesh
		meshGL->MeshName = currMesh->name; //now in CString implementation

		//visibility flag
		meshGL->Visible = !currMesh->ishidden;

		//copy the face array
		FaceArray = new face3ds[currMesh->nfaces];
		memcpy(FaceArray, currMesh->facearray, currMesh->nfaces * sizeof(face3ds));

		//fill in the vertices, normals and markers
		for (i = 0; i < currMesh->nvertices; i++) {
			vertListGL.AppendToList(currMesh->vertexarray[i]);
			normalListGL.AppendNULLEntry();
			markedList.AppendToList(markerFALSE);
		}			

		// texturing data available (optional)?
		if (currMesh->ntextverts > 0)
			for (i = 0; i < currMesh->ntextverts; i++)
				texListGL.AppendToList(currMesh->textarray[i]);

		//fill in the smooth List with -1's
		if (currMesh->smootharray != NULL)
			for (i = 0; i < currMesh->nvertices; i++)
				smoothList.AppendNULLEntry();

		//if no smoothing is defined in the mesh then just compute and store
		// the face normal for each vertex in that face	
		if (currMesh->smootharray == NULL) {
			// traverse the faces
			for (currFace = 0; currFace < currMesh->nfaces; currFace++) {
				//alias the FaceArray for faster access
				face3ds& faceRec = FaceArray[currFace];

				// normal vector for the face
				vector1 = VectorSubtract(*vertListGL[faceRec.vert_index[0]], *vertListGL[faceRec.vert_index[1]]);
				vector2 = VectorSubtract(*vertListGL[faceRec.vert_index[2]], *vertListGL[faceRec.vert_index[1]]);
				
				//the vertices are defined in counter-clockwise order in 3DStudio file 
				//versions prior to 3rd
				if (m_Version < 3)
					normal = VectorCrossProduct(vector1, vector2);
                else 
					normal = VectorCrossProduct(vector2, vector1);
				
				// go for each vertex in the current face
				for (int currVertex = 0; currVertex < 3; currVertex++) {
					// copy current index into vertex array for faster access
					currentIndex = faceRec.vert_index[currVertex];
					
					// already been touched?
					if (*markedList[currentIndex]) {
						// already touched vertex must be duplicated
						{
							// enhance vertex array..FIXED
							vertListGL.AppendToList(currMesh->vertexarray[currentIndex]);
							// enhance texture coordinates array
							if (texListGL.ListCount() > 0)
								texListGL.AppendToList(texListGL[currentIndex]);
							// enhance the normal array
							normalListGL.AppendToList(normal);
							// enhance the marker array
							markedList.AppendToList(markerFALSE);

							currentVertexCount++;

						}
						//tell which index to use from now on
						faceRec.vert_index[currVertex] = currentVertexCount - 1;
					}
					else {
						// not yet touched, so just store the normal
						*normalListGL[currentIndex] = normal;
						*markedList[currentIndex] = TRUE;
					}
				}
			}
		} //end if no smoothing groups
		else {
			// smoothing groups are to be considered
		for (currFace = 0; currFace < currMesh->nfaces; currFace++) {
			//alias the current face for faster access
			face3ds& faceRec = FaceArray[currFace];
				
			// normal vector for the face
			vector1 = VectorSubtract(*vertListGL[faceRec.vert_index[0]], *vertListGL[faceRec.vert_index[1]]);
			vector2 = VectorSubtract(*vertListGL[faceRec.vert_index[2]], *vertListGL[faceRec.vert_index[1]]);
			
			//get the cross product 
			if (m_Version < 3)
				normal = VectorCrossProduct(vector1, vector2);
		    else 
				normal = VectorCrossProduct(vector2, vector1);
        
			//get the current smoothing group from the original smooth array
			smoothGroup = currMesh->smootharray[currFace];
        
			// go for each vertex in the current face
			for (int currVertex = 0; currVertex < 3; currVertex++) {
			
			// copy current index for faster access
			currentIndex = faceRec.vert_index[currVertex];

			// Has vertex already been touched?
			if (*markedList[currentIndex]) {
				// check smoothing group
				if (smoothGroup == 0) {
					// no smoothing then just duplicate vertex
					{
						// enhance vertex array
						vertListGL.AppendToList(currMesh->vertexarray[currentIndex]);
						// enhance texture coordinates array
						if (texListGL.ListCount() > 0)
							texListGL.AppendToList(texListGL[currentIndex]);
						// enhance the normal array
						normalListGL.AppendToList(normal);
						// enhance the marker array
						markedList.AppendToList(markerFALSE);
						// enhance the smooth array
						smoothList.AppendNULLEntry();
			
						currentVertexCount++;

					}
					//tell which index to use from now on
					faceRec.vert_index[currVertex] = currentVertexCount - 1;
					// mark new vertex also as touched
					*markedList[currentVertexCount - 1] = TRUE;
				
				} //end if no smoothing group assigned 
				else {
					// this vertex must be smoothed, check if there's already a (duplicated) vertex
					// for this smoothing group
					long targetVertex = smoothList.GetSmoothIndex(currentIndex, smoothGroup);
								  
					if (targetVertex < 0) {
						// vertex has not yet been duplicated for this smoothing group, so do it now
						{
							// enhance vertex array
							vertListGL.AppendToList(currMesh->vertexarray[currentIndex]);
							// enhance texture coordinates array
							if (texListGL.ListCount() > 0)
								texListGL.AppendToList(texListGL[currentIndex]);
							// enhance the normal array
							normalListGL.AppendToList(normal);
							// enhance the marker array
							markedList.AppendToList(markerFALSE);
							// enhance the smooth array
							smoothList.AppendNULLEntry();
							
							currentVertexCount++;

						}
						//tell which index to use from now on
						faceRec.vert_index[currVertex] = currentVertexCount - 1;

						//store the smooth indices of the duplicated vertex once for itself and once for
						// the vertex it was duplicated from
						smoothList.StoreSmoothIndex(currentIndex, smoothGroup, currentVertexCount - 1);
						smoothList.StoreSmoothIndex(currentVertexCount - 1, smoothGroup, currentVertexCount - 1);
						// mark new vertex also as touched
						*markedList[currentVertexCount - 1] = TRUE;
					}	//end if vertex was not duplicated yet 
					else {
					// vertex has already been duplicated, so just add normal vector to other vertex...
						*normalListGL[targetVertex] = VectorAdd(*normalListGL[targetVertex], normal);
						// ...and tell which new vertex has to be used from now on
						faceRec.vert_index[currVertex] = (ushort3ds) targetVertex;
					}
				}	//end else if smoothing group assigned
			} // end if is marked
			else { //if it is not marked
				// vertex not yet touched, so just store the normal
				*normalListGL[currentIndex] = normal;

				// initialize smooth indices for this vertex
				if (smoothGroup != 0) 
					smoothList.StoreSmoothIndex(currentIndex, smoothGroup, currentIndex);
				
				//and mark the vertex as touched
				*markedList[currentIndex] = TRUE;
			}
			} //end for every vertex
		} //for every face
		} //end else

		//the current count of vertices
		meshGL->Count = currentVertexCount;

		//put the vertices into the OpenGL's mesh structure
		meshGL->Vertices = new float[vertListGL.ListCount() * 3];
		i = 0;
		while (!vertListGL.Empty()) {
			point3ds* vert = vertListGL.ExtractHead();
			meshGL->Vertices[i++] = vert->x;
			meshGL->Vertices[i++] = vert->y;
			meshGL->Vertices[i++] = vert->z;
			delete vert;
		}

		//copy the texture vertices if they exist
		if (currMesh->ntextverts > 0) {
			meshGL->TexCoords = new TexPointGL[texListGL.ListCount()];
			i = 0;
			while (!texListGL.Empty()) {
				TexPointGL* texPtGL = texListGL.ExtractHead();
				meshGL->TexCoords[i++] = *texPtGL;
				delete texPtGL;
			}
		}

		//copy and normalize the normals 
		meshGL->Normals = new float[normalListGL.ListCount() * 3];
		i = 0;
		while (!normalListGL.Empty()) {
			point3ds* oldNorm = normalListGL.ExtractHead();
			point3ds normal = VectorNormalize(*oldNorm);

			meshGL->Normals[i++] = normal.x;
			meshGL->Normals[i++] = normal.y;
			meshGL->Normals[i++] = normal.z;
			delete oldNorm;
		}

		//alias the faces of the OpenGL mesh structure
		FaceListGL *facesListGL = &meshGL->Faces;
		
		//if there are no materials assigned to the mesh
		//then just copy the face information and set TextureIndex to
		// default material(0) in the OpenGL's material list
		if (currMesh->nmats == 0) {
			facesListGL->nMatGroups = 1;//currMesh->nmats;
			facesListGL->MatGroups = new FaceGroup;
			facesListGL->MatGroups->SurfPtr = surfaces[0];  //assign the first material in the list
			facesListGL->MatGroups->IndexCount = 3 * currMesh->nfaces;
			facesListGL->MatGroups->Indices = new GLint[facesListGL->MatGroups->IndexCount];
			for (currFace = 0; currFace < currMesh->nfaces; currFace++) {
				facesListGL->MatGroups->Indices[3 * currFace + 0] = FaceArray[currFace].vert_index[0];//currMesh->facearray[currFace].vert_index[0];
				facesListGL->MatGroups->Indices[3 * currFace + 1] = FaceArray[currFace].vert_index[1];//currMesh->facearray[currFace].vert_index[1];
				facesListGL->MatGroups->Indices[3 * currFace + 2] = FaceArray[currFace].vert_index[2];//currMesh->facearray[currFace].vert_index[2];
			}
		}
		else {
			//if a material or several materials are assigned to the Mesh

			//get the space for the material definitions
			facesListGL->nMatGroups = currMesh->nmats;
			facesListGL->MatGroups = new FaceGroup[currMesh->nmats];
	
			//alias the matgroups of the OpenGL's FaceGroup
			FaceGroup* faceGroups = facesListGL->MatGroups;
			//for all of the assigned materials in the mesh
			for (ushort3ds currMat = 0; currMat < currMesh->nmats; currMat++) {

				//find the material in the material array and init TextureIndex which is later used as
				// an index into the material array
				faceGroups[currMat].SurfPtr = surfaces.FindSurfaceByName(currMesh->matarray[currMat].name);

				if (faceGroups[currMat].SurfPtr == NULL) {
					CString pcsMsg = "Material ";
					pcsMsg += currMesh->matarray[currMat].name;
					pcsMsg += " not found. Using default material.";
					AfxMessageBox(pcsMsg);
					//get the default surface.. at least one must exist
					faceGroups[currMat].SurfPtr = surfaces[0]; 
				}

				//get the space for the individual face group indices in OpenGL's mesh
				faceGroups[currMat].IndexCount = 3 * currMesh->matarray[currMat].nfaces;
				faceGroups[currMat].Indices = new GLint[faceGroups[currMat].IndexCount];
			
				// copy all vertices belonging to the current face into our index array,
				// there won't be redundant vertices since this would mean a face has more than one
				// material
				for (i = 0; i < currMesh->matarray[currMat].nfaces; i++) {
					faceGroups[currMat].Indices[3 * i + 0] = FaceArray[currMesh->matarray[currMat].faceindex[i]].vert_index[0];
					faceGroups[currMat].Indices[3 * i + 1] = FaceArray[currMesh->matarray[currMat].faceindex[i]].vert_index[1];
					faceGroups[currMat].Indices[3 * i + 2] = FaceArray[currMesh->matarray[currMat].faceindex[i]].vert_index[2];
				}
			} //end for
		} //end else

		//add the mesh GL object to the main list
		meshListGL.AppendToList(meshGL);

		//create a new aggregate object from the OpenGL mesh
		Aggregate* aggrObj = CreateAggregateObj(meshGL, currMesh);
		objects.AppendToList(aggrObj);

		//clear all the local lists and arrays
		//vertListGL, texListGL.Clear(), and normalListGL already cleared
		while (smoothList.ListCount())
			delete [] smoothList.ExtractHead();
			
		while (markedList.ListCount())
			delete markedList.ExtractHead();
		//smoothList.Clear();
		//markedList.Clear();
		delete [] FaceArray;

    } //end for every mesh 
}


//Creates a new Aggregate object containing triangles
Aggregate*
C3DStudio::CreateAggregateObj(MeshGL* meshGL, mesh3ds* currMesh)
{
	Aggregate* aggrObj = new Aggregate;

	aggrObj->SetName(meshGL->MeshName);

	//alias the material groups for faster access
	FaceGroup* matGroup = meshGL->Faces.MatGroups;

	//for every material group in the mesh
	for (int i = 0; i < meshGL->Faces.nMatGroups; i++) {
	
		//for every face in the material group
		for (int j = 0; j < matGroup[i].IndexCount; j += 3) {
			
			Vertex* Verts = new Vertex[3];	//make new triangle vertices
			//assign the vertices to poly fields in the mesh

			//copy indices for faster access
			int face[3];
			face[0] = matGroup[i].Indices[j + 0];
			face[1] = matGroup[i].Indices[j + 1];
			face[2] = matGroup[i].Indices[j + 2];

			//assign Vertices, texture coordinates and normals
			for (int v_index = 0; v_index < 3; v_index++) {
				Verts[v_index].Pos = meshGL->Vertices[face[v_index] * 3];

				if (currMesh->ntextverts > 0)
					Verts[v_index].UV = meshGL->TexCoords[face[v_index]];

				Verts[v_index].Normal = meshGL->Normals[face[v_index] * 3];
			}

			Mapping* newMapping;
			//set the mapping type for the object
			if (currMesh->ntextverts > 0)
				newMapping = new UVMapping;	//default UV mapping
			else {
				//get the mesh matrix position
				Vector3D center = Vector3D(currMesh->locmatrix[9],
										   currMesh->locmatrix[10],
										   currMesh->locmatrix[11]);
			
				newMapping = new SphericalMapping(center);  //or spherical
			}

			//now supports the face surface mapping
			Triangle* newTriangle = new Triangle(Verts, matGroup[i].SurfPtr, newMapping);
			aggrObj->AppendObject(newTriangle);
		
		} //end for (j...meshGL->Vertices
	}	//end for (i..

	return aggrObj;
}
