// ===================================================================
//  File: distrib.cpp							12/27/1999
//	By Konstantin Khanyants						kon@unix.temple.edu
//	Description:
//		Implements functions used for adaptive
//		supersampling scheme of ray distribution.
//		It's hard to unify the code for different types of
//		ray (primary, secondary and shadow), but common pieces 
//		of code have been collected here.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include "include/Oort.h"

float
MinDist(const Vector3D& v, const Vector3D *samples, int n)
{
    float ret = MAXFLOAT;
    for (int i = 0; i < n; i++) {
	float dist = Magnitude(v - samples[i]);
	if (dist < ret)
	    ret = dist;
    }
    return ret;
}

// Return nonzero if the contrast given by the min and max doesn't meet
// the tolerance given (i.e., returns nonzero if more rays are needed).
int
MoreRays(const RGBColor& minC, const RGBColor& maxC, const RGBColor& tol)
{
    RGBColor num = maxC - minC;
    RGBColor denom = maxC + minC;
    if (denom.x != 0.0 && num.x / denom.x > tol.x)
	return 1;
    if (denom.y != 0.0 && num.y / denom.y > tol.y)
	return 1;
    if (denom.z != 0.0 && num.z / denom.z > tol.z)
	return 1;
    return 0;
}