// ===================================================================
//  File: inv.cpp								12/27/1999
//	By Konstantin Khanyants						kon@unix.temple.edu
//	Description:
//		Implementation file for inverse-ray objects (objects with a
//		canonical form that are transformed by transforming the ray
//		into the same space as the canonical form).
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include "include/Oort.h"

// Transform the ray so that it is in canonical space.
Ray
InverseRayObject::InvertRay(Ray& ray)
{
    Ray ret = ray;
    ret.ApplyTransform(inv);
    return ret;
}

// Transform a point so that it is in canonical space.
Vector3D
InverseRayObject::InvertPt(const Vector3D& v)
{
    return v * inv;
}

// Rotate a vector so that it is in canonical space.
Vector3D
InverseRayObject::InvRotatePt(const Vector3D& v)
{
    return RotateOnly(v, inv);
}

// Transform a ray from canonical space back to regular space.
Ray
InverseRayObject::TransformRay(Ray& ray)
{
    Ray ret = ray;
    ret.ApplyTransform(mat);
    return ret;
}

// Transform a point from canonical space back to regular space.
Vector3D
InverseRayObject::TransformPt(const Vector3D& v)
{
    return v * mat;
}

// Rotate a point from canonical space back to regular space.
Vector3D
InverseRayObject::RotatePt(const Vector3D& v)
{
    return RotateOnly(v, mat);
}

// Compute the nearest intersection along the ray, if any.
// This function transforms the ray to canonical space before
// passing it along to the Intersect routine for the object.
int
InverseRayObject::NearestInt(Ray& cpyme, float& t, float maxt)
{
    float tempt;
    Ray ray = InvertRay(cpyme);

    int ret = NearestInt(ray, tempt, maxt);
    if (ret != 0) {
	t = cpyme.Interp(mat * ray.Extrap(tempt));
	ret = (t > Limits::Threshold && t < maxt);
    }
    return ret;
}

// Apply a transform to the object.  This function affects only
// the transformations to and from the canonical space of the
// object.
void
InverseRayObject::ApplyTransform(const Matrix& tform)
{
    mat = tform * mat;
    inv *= Invert(tform);
}