// ===================================================================
//  File: object3d.cpp							12/27/1999
//	By Konstantin Khanyants						kon@unix.temple.edu
//	Description:
//		Support routines for Object3D class.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include "include/Oort.h"
#include "include/World.h"

// =========================================================
// Object3D::ShadeObject
//	Called by World when it is determined that the 
//	object was the closest along the given ray.
// =========================================================
RGBColor
Object3D::ShadeObject(World& world, Ray& ray, Vector3D& Intersection, int depth)
{
    ShadingInfo shade;

    shade.p = Intersection;
    shade.normal = FindNormal(Intersection);
	shade.incident = ray.dir;

	//used only for back facing polygons
	if (DotProd(shade.incident, shade.normal) > 0)
	shade.normal = -shade.normal;
    
	shade.reflected = ray.ReflectRay(shade.normal);
    shade.dudt = Vector3D(0);			//added for bump mapping
    shade.dvdt = Vector3D(0);
    shade.obj = this;
    shade.ray = &ray;
    shade.world = &world;
    return surf->ComputeColor(shade, depth);
}

int
Object3D::HitAtAll(Ray& ray, float maxt)
{
    float tempt;
    return NearestInt(ray, tempt, maxt) != 0;
}

int
Object3D::IsInside(Vector3D& v)
{
    float dist = PtDistance(v);
    return dist < 0 || fabs(dist) < Limits::Small;
}

//delete all of the surfaces
void
SurfaceList::DeleteSurfaces()
{
	while (n > 0)
		delete (ExtractHead());
}

int
SurfaceList::SurfaceThere(HallSurface *probe)
{
    for (Iterator sc(*this); sc.Valid(); sc.GotoNext()) {
	if (sc.Contents() == probe)
	    return 1;
    }
    return 0;
}

HallSurface*
SurfaceList::FindSurfaceByName(CString surfName)
{
	for (Iterator sc(*this); sc.Valid(); sc.GotoNext()) {
	if (surfName == sc.Contents()->name)
	    return sc.Contents();
    }
    return NULL;
}

//NEW TRANSFORM SURFACES ONCE ONLY
void
SurfaceList::TransformSurfaces(const Matrix& Transform)
{
    for (Iterator sc(*this); sc.Valid(); sc.GotoNext())
	sc.Contents()->PreMulTransform(Transform);
}


//added 6/12/2000 for accessing objects by pointer and not by
//pointer to pointer
HallSurface*
SurfaceList::operator[] (int inx) 
{
//	ModifyCheck();
    if (inx >= n || inx < 0)
        return 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return pNode->contents;
	}
	return 0;
}


void
ObjectList::TransformList(const Matrix& Transform)
{
    for (Iterator sc(*this); sc.Valid(); sc.GotoNext()) {
		sc.Contents()->ApplyTransform(Transform);
	}
}

//added 6/12/2000 for accessing objects by pointer and not by
//pointer to pointer
Object3D *
ObjectList::operator[] (int inx) 
{
//	ModifyCheck();
    if (inx >= n || inx < 0)
        return 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return pNode->contents;
	}
	return 0;
}

//delete all of the Ray tracer geometric objects
void
ObjectList::DeleteObjects()
{
	while (n > 0)
		delete (ExtractHead());

}
