// ===================================================================
// 	File: polygon.cpp						12/26/1999
//	By Konstantin Khanyants					kon@unix.temple.edu
//	Description:
//		Polygon support routines for OORT.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include "include/Oort.h"
/*
// Polygon constructor
Polygon::Polygon(int NewNumVerts,	// number of vertices
		 Vector3D *NewVertices,	// vertices
		 HallSurface *NewSurface, Mapping *NewMapping):  // polygon surface
    Planar(NewSurface, NewMapping)
{
    NumVerts = NewNumVerts;		  // copy given info
    Vertices = new Vector3D[NumVerts];
    for (int i = 0; i < NumVerts; i++)
	Vertices[i] = NewVertices[i];
    Precompute();			  // precompute
}

Polygon::Polygon(const Polygon& x): Planar(x.surf, x.mapping)
{
    Vertices = new Vector3D[NumVerts = x.NumVerts];
    for (int i = 0; i < NumVerts; i++)
	Vertices[i] = x.Vertices[i];
    Precompute();
}

// Polygon destructor
Polygon::~Polygon()
{
	if (mapping) 
		delete mapping;
    delete[] Vertices;
}

// Precompute various parameters about the polygon:
//	The plane equation (including normal)
//	which component of the normal is dominant (needed
//		for intersection routine)
//	Bounding rectangle of projection (used by intersection
//		routine to eliminate intersections)
//	Overall bounding box of polygon (used by intersection
//		routine to intersect with ray bounding boxes)
void 
Polygon::Precompute()
{
    int i;			  // Index variable
    float max;			  // Maximum component of normal
    Vector3D minvert, maxvert;	  // Bounds of polygon

    // Compute the plane equation of the polygon.
    Vector3D xprod = CrossProd(Vertices[2] - Vertices[1],
			       Vertices[0] - Vertices[1]);
    if (fabs(Magnitude(xprod)) > Limits::Small) {
	bad = 0;
	Normal = Normalize(xprod);
    }
    else {
	bad = 1;
	return;
    }
    d = -DotProd(Normal, Vertices[0]);

    // Precompute which.  which is 0 if the normal is dominant
    // in the X direction, 1 if the Y direction, 2 if the Z direction.

    which = 0;			  // Assume we'll ignore X for now
    max = (float) fabs(Normal.x);
    if (fabs(Normal.y) > max) {   // Check to see if we ignore Y
	max = (float) fabs(Normal.y);	  // Yes, ignore Y instead
	which = 1;
    }
    if (fabs(Normal.z) > max)	  // Check to see if we ignore Z
	which = 2;		  // Yep

    // Compute bounding rectangle of projection.  This is fairly
    // successful at rejecting nonintersections.
    Vertices[0].ExtractVerts(&umin, &vmin, which);
    umax = umin;
    vmax = vmin;
    for (i = 1; i < NumVerts; i++) {
	float tempx, tempy;

	Vertices[i].ExtractVerts(&tempx, &tempy, which);
	if (tempx < umin)
	    umin = tempx;
	else if (tempx > umax)
	    umax = tempx;
	if (tempy < vmin)
	    vmin = tempy;
	else if (tempy > vmax)
	    vmax = tempy;
    }
}


// int_lineseg returns 1 if the given line segment intersects a 2D
// ray travelling in the positive X direction.	This is used in the
// Jordan curve computation for polygon intersection.
int
int_lineseg(float px, float py, float u1, float v1, float u2, float v2)
{
    float t;
    float ydiff;

    u1 -= px; u2 -= px; 	  // translate line
    v1 -= py; v2 -= py;

    if ((v1 > 0 && v2 > 0) ||
	(v1 < 0 && v2 < 0) ||
	(u1 < 0 && u2 < 0))
	 return 0;

    if (u1 > 0 && u2 > 0)
	return 1;

    ydiff = v2 - v1;
    if (fabs(ydiff) < Limits::Small) {	  // denominator near 0
	if (((fabs(v1) > Limits::Small) ||
	    (u1 > 0) || (u2 > 0)))
	    return 0;
	return 1;
    }

    t = -v1 / ydiff;		  // Compute parameter

    return (u1 + t * (u2 - u1)) > 0;
}

// Polygon intersection routine.
int
Polygon::NearestInt(Ray& ray, float& t, float maxt)
{
    int count;
    float u, v, u1, v1, u2, v2;
    int i;

    Statistics::Intersections::Polygon++;
    if (Planar::NearestInt(ray, t, maxt)) {
	// Project the intersection point onto the coordinate plane
	// specified by which.
	ray.Extrap(t).ExtractVerts(&u, &v, which);

	// Reject intersection if outside projected bounding rectangle
	if (u < umin || u > umax || v < vmin || v > vmax)
	    return 0;

	// We're stuck with the Jordan curve computation.  Count number
	// of intersections between the line segments the polygon comprises
	// with a ray originating at the point of intersection and
	// travelling in the positive X direction.

	count = 0;
	Vertices[NumVerts - 1].ExtractVerts(&u1, &v1, which);

	for (i = 0; i < NumVerts; i++) {
	    Vertices[i].ExtractVerts(&u2, &v2, which);
	    count += int_lineseg(u, v, u1, v1, u2, v2) != 0;

	    u1 = u2;
	    v1 = v2;
	}

	// We hit polygon if number of intersections is odd.
	return count & 1;
    }
    return 0;
}

// Apply transform to a polygon: transform each vertex, then
// re-precompute all the precomputed information.
void
Polygon::ApplyTransform(const Matrix& tform)
{
    for (int i = 0; i < NumVerts; i++)
	Vertices[i] = tform * Vertices[i];
    Precompute();
}

// Duplicate a polygon.
Object3D *
Polygon::Dup() const
{
    return new Polygon(NumVerts, Vertices, surf, mapping);
}

AxisAlignedBox
Polygon::BBox() const
{
    int i;
    Vector3D minV = Vertices[0];
    Vector3D maxV = Vertices[0];

    for (i = 1; i < NumVerts; i++) {
	Minimize(minV, Vertices[i]);
	Maximize(maxV, Vertices[i]);
    }
    // Make sure the bounding box isn't flat.
    Vector3D diff = minV - maxV;
    for (i = 0; i < 3; i++)
	if (fabs(diff[i]) < Limits::Threshold) {
	    minV[i] -= Limits::Threshold;
	    maxV[i] += Limits::Threshold;
	}
    return AxisAlignedBox(minV, maxV);
}
*/