// ===================================================================
//  File: vector.cpp							12/27/1999
//	By Konstantin Khanyants						kon@unix.temple.edu
//	Description:
//		Vector3D support routines for OORT.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================

#include "stdafx.h"
#include <math.h>
#include <stdio.h>
#include "include/Oort.h"

// Given min a vector to minimize and a candidate vector, replace
// elements of min whose corresponding elements in Candidate are
// smaller.  This function is used for finding objects' bounds,
// among other things.
void
Minimize(Vector3D& minV, const Vector3D& Candidate)
{
    if (Candidate.x < minV.x)
	minV.x = Candidate.x;
    if (Candidate.y < minV.y)
	minV.y = Candidate.y;
    if (Candidate.z < minV.z)
	minV.z = Candidate.z;
}

// Given min a vector to minimize and a candidate vector, replace
// elements of min whose corresponding elements in Candidate are
// larger.  This function is used for finding objects' bounds,
// among other things.
void
Maximize(Vector3D& max, const Vector3D& Candidate)
{
    if (Candidate.x > max.x)
	max.x = Candidate.x;
    if (Candidate.y > max.y)
	max.y = Candidate.y;
    if (Candidate.z > max.z)
	max.z = Candidate.z;
}

// Project the vector onto the YZ, XZ, or XY plane depending on which.
// which	Coordinate plane to project onto
// 0		YZ
// 1		XZ
// 2		XY
// This function is used by the polygon intersection code.

void
Vector3D::ExtractVerts(float *px, float *py, int which) const
{
    switch (which) {
	case 0:
	    *px = y;
	    *py = z;
	    break;
	case 1:
	    *px = x;
	    *py = z;
	    break;
	case 2:
	    *px = x;
	    *py = y;
	    break;
    }
}

Vector3D::Vector3D(const Vector2D& v) 
{ 
	x = v.x; 
	y = v.y; 
	z = 0; 
}

Vector3D::Vector3D(point3ds& p) 
{
	x = p.x; 
	y = p.y; 
	z = p.z; 
}

///Added because data types can't be declared 4/6/2000
//added 6/11/2000 for 3dstudio conversion of vertices
Vector3D& Vector3D::operator= (point3ds& point)
{
	x = point.x; y = point.y; z = point.z;
	return *this;
}

//added 4/4/2000 for OpenGL conversion of vertices
Vector3D& Vector3D::operator= (Vector3f vec)
{
	x = vec[0]; y = vec[1]; z = vec[2];
	return *this;
}

//VECTOR 2D CLASS CONSTRUCTOR
Vector2D::Vector2D(const Vector3D& v) 
{ 
	x = v.x; 
	y = v.y; 
}

//added 6/11/2000 for uv coordinate assignment in 3DStudio
Vector2D& Vector2D::operator= (TexPointGL& texVert)
{
	x = texVert.s;
	y = texVert.t;
	return *this;
}

//added for interaction with 3D Studio ojects 6/12/2000
RGBColor::RGBColor(fcolor3ds& col3ds)
{
	x = col3ds.r; y = col3ds.g; z = col3ds.b;
}

RGBColor::RGBColor(COLORREF& col) 
{ 
	x = (col & 0xff) / 255.0f;
	y = ((col >> 8) & 0xff) / 255.0f;
	z = ((col >> 16) & 0xff) / 255.0f;
}


COLORREF GetCOLORREF(const RGBColor& x)
{
	return RGB(x.x * 255.0, x.y * 255.0, x.z * 255.0);
}

// Given min a vector to minimize and a candidate vector, replace
// elements of min whose corresponding elements in Candidate are
// smaller.  This function is used for finding objects' bounds,
// among other things.
void
Minimize(Vector2D& _min, const Vector2D& Candidate)
{
    if (Candidate.x < _min.x)
	_min.x = Candidate.x;
    if (Candidate.y < _min.y)
	_min.y = Candidate.y;
}

// Given min a vector to minimize and a candidate vector, replace
// elements of min whose corresponding elements in Candidate are
// larger.  This function is used for finding objects' bounds,
// among other things.
void
Maximize(Vector2D& _max, const Vector2D& Candidate)
{
    if (Candidate.x > _max.x)
	_max.x = Candidate.x;
    if (Candidate.y > _max.y)
	_max.y = Candidate.y;
}

