/****************************************************************************
 *	
 * 	3DSFile.h 					3/9/2000
 *	By Konstantin Khanyants 	kon@unix.temple.edu
 *	
 *	Description: 3D Studio include file that contains the constants
 *				 and defines.
 *
 *  Original Copyright 1997 by Autodesk, Inc.
 *
 ***************************************************************************/


#ifndef _3DSFILE_H_
#define _3DSFILE_H_

//#include "Oort.h"
#include <stdio.h>
					
/* Note: Item's prefixed with a "Priv" comment are internally used, are not
   part of the toolkit's published interface, and should not be used
   by non-ftk library functions */


/* Data Type Synonyms For Portability */

#define char3ds char
#define uchar3ds unsigned char
#define byte3ds char
#define ubyte3ds unsigned char
#define short3ds short int
#define ushort3ds unsigned short int
#define long3ds long int
#define ulong3ds unsigned long int
#define float3ds float
#define double3ds double
#define int3ds   int
#define void3ds void

/* 3DS File Toolkit revision level.  Future revisions will increment this
   integer number */
#define FTKRevisionLevel3ds 2

/* point3ds - 3D point structure */
typedef struct {
   float3ds x, y, z;
} point3ds;

//typedef float3ds point3ds[3];  //changed for OpenGl 3/21/2000

/* fcolor3ds - Floating point color definition */
typedef struct {
   float3ds r;   /* RGB Color components */
   float3ds g;
   float3ds b;
} fcolor3ds;       

//Original implementation
/* face3ds - Face list element */
//typedef struct
//{
//   ushort3ds v1, v2, v3, flag;
//} face3ds;

/* face3ds - Face list element */ 
typedef struct
{
	ushort3ds vert_index[3];
	ushort3ds flag;
} face3ds;


/* textvert3ds - Texture assignment coordinate */
typedef struct
{
   float3ds u, v;
} textvert3ds;

/* Priv */ /* chunk3ds - Internal database representation of file information */
typedef struct chunk3ds {
	ushort3ds	tag;		 /* Type of chunk */
    ulong3ds    len;         /* Number of bytes used by chunk */
//    ulong3ds     position;     /* Offset in source file */
//    void        *data;         /* Memory copy of file data */
} chunk3ds;


/* file3ds - Handle to 3D Studio files */
typedef struct
{
	FILE *fp;         /* stdio file pointer */
//  char3ds *filename;  /* name of file */
	ulong3ds size;		/* added size of file */
} file3ds;

typedef enum
{
   NoView3ds,
   TopView3ds,
   BottomView3ds,
   LeftView3ds,
   RightView3ds,
   FrontView3ds,
   BackView3ds,
   UserView3ds,
   CameraView3ds,
   SpotlightView3ds
} viewtype3ds;

typedef struct
{
   ushort3ds xpos;
   ushort3ds ypos;
   ushort3ds width;
   ushort3ds height;
} viewsize3ds;

/* Used to describe top, bottom, left, right, front and back views */
typedef struct
{
   point3ds center; /* Center of orthagonal view */
   float3ds zoom; /* View zoom factor */
} orthoview3ds;

/* Used to describe user views */
typedef struct
{
   point3ds center; /* Center of user view */
   float3ds zoom; /* View zoom factor */
   float3ds horang; /* Horizontal angle of view */
   float3ds verang; /* Veritical angle of view */
} userview3ds;

/* Used to describe camera views */
typedef struct
{
   char3ds *name; /* Name of the camera used in the view */
} cameraview3ds;

/* Viewport structure details the kind of view in a viewport */
typedef struct
{
   viewtype3ds type; /* top, bottom, left, right, front, back, user and camera */
   viewsize3ds size; /* Size of the viewport */
   orthoview3ds ortho; /* used for top, bottom, left, right, front, and back views */
   userview3ds user; /* Used for user views */
   cameraview3ds camera; /* Used for camera views */
} viewport3ds;



typedef struct {
   short3ds form; /* 0 = single window, 1 = 2 split verticle, 2 = 2 split horizontal, 3 = 4 equal squares,
		     4 = 2 squares left & 1 rect right, 5 = 1 rect at top & 2 sqr on bot, 6 = 1 rect left & 2 sqr right,
		     7 = 2 sqr top & 1 rect bot, 8 = 3 split vertical, 9 = 2 split horiz, 10 = 3 sqr left and 1 rect right,
		     11 = 1 rect left & 3 sqr. right.

		     form becomes 0 during swap and its preswapped value is stored in the swapport field */
   short3ds top; /* Active window index of 0 to 5 */
   short3ds ready; 
   short3ds wstate; /* 0 if no swap window, 1 if in swaped "w" state.  During a swap, the old 0 window gets stored as the 4 window */
   short3ds swapws;  
   short3ds swapport; /* The preswapped value from the form field */
   short3ds swapcur; /* The window index that was swapped */
} ViewportLayout;

typedef struct { /* Values given for 1024x768 resolution */
   ushort3ds xpos; /* 0 */
   ushort3ds ypos; /* 14 */
   ushort3ds width; /* 895 */
   ushort3ds height; /* 725 */
} ViewportSize;

typedef struct {
   ushort3ds flags;
   ushort3ds axislockout;
   ushort3ds winxpos;
   ushort3ds winypos;
   ushort3ds winwidth;
   ushort3ds winheight;
   ushort3ds view; /* 0 = No view, 1 = Top View, 2 = Bottom View, 3 = Left View, 4 = Right View, 5 = Front View, 6 = Back View, 7 = User View, 18 = Spotlight View, 65535 = Camera View */
   float3ds zoomfactor;
   point3ds center;
   float3ds horizang;
   float3ds vertang;
   char3ds *camname;
} ViewportData, ViewportData3;

typedef struct {
   point3ds   viewtargetcoord;
   float3ds viewwidth;
} ViewStandard;

typedef struct {
   point3ds viewtargetcoord;
   float3ds viewwidth;
   float3ds xyviewangle;
   float3ds yzviewangle;
   float3ds bankangle;
} ViewUser;

typedef struct {
   char3ds *name;
} ViewCamera;


//typedef enum 
//{
//   UseShadowMap,
//   UseRayTraceShadow
//} shadowstyle3ds;

//typedef struct { /* global shadow settings */
//   shadowstyle3ds type; /* Either UseShadowMaps or UseRayTraceShadows */
//   float3ds bias; /* shadow bias factor. */
//   float3ds raybias; /* shadow ray bais factor. Used in R3 */
 //  short3ds mapsize; /* shadow map size */
//   float3ds filter; /* shadow filter */
//} shadowsets3ds;
   
typedef struct {
   float3ds masterscale; /* Master mesh scale factor */
//   shadowsets3ds shadow; /* Global shadow settings */
   fcolor3ds ambientlight;  /* ambient light color */
   point3ds oconsts; /* default object constructing axis */
} meshset3ds;

/* enumerate list of possible backgrounds used in file */
typedef enum
{
   NoBgnd,
   UseSolidBgnd,
   UseVGradientBgnd, 
   UseBitmapBgnd
} backgroundtype3ds;

typedef struct 
{
   char3ds *name; /* name of background bitmap */
} bitmapbgnd3ds;

typedef struct 
{
   fcolor3ds color; /* color of solid background */
} solidbgnd3ds;

typedef struct 
{
   float3ds gradpercent; /* Placement of mid color band, ranges from 0-1 */ 
   fcolor3ds  top; /* color of top band */
   fcolor3ds  mid; /* color of mid background band */
   fcolor3ds  bottom; /* color of bottom band */
} vgradientbgnd3ds;

typedef struct {
   bitmapbgnd3ds bitmap;
   solidbgnd3ds solid;
   vgradientbgnd3ds vgradient;
   backgroundtype3ds bgndused; /* background in effect */
} background3ds;

/* Used for shading field in material3ds structure */
typedef enum
{
   Wire = 0,
   Flat = 1,
   Gouraud = 2,
   Phong = 3,
   Metal = 4
} shadetype3ds;

/* Used for tiling field in bitmap3ds structure */
typedef enum 
{
   Tile = 1,
   Decal = 2,
   Both = 3
} tiletype3ds;

typedef enum
{
   Pyramidal,
   SummedArea
} filtertype3ds;

typedef enum
{
   RGB,
   Alpha,
   RGBLumaTint,
   AlphaTint,
   RGBTint
} tinttype3ds;


/* Used By AddMaterial3ds */

typedef struct {
   ubyte3ds firstframe; /* True for First Frame Only */
   ubyte3ds flat; /* True for flat mirror reflection */
   ulong3ds size;     /* Map resolution */
   ulong3ds nthframe; /* Map update period */
} acubic3ds;          /* Cubic reflection map defintion */

typedef struct {
   char3ds  *name;  /* Bitmap file name |was 18*/
   float3ds percent;  /* Strength percentage */
   tiletype3ds tiling;  /* Tile/Decal/Both */
   byte3ds ignorealpha;
   filtertype3ds filter; /* Pyramidal/Summed Area */
   float3ds blur;
   byte3ds mirror;
   byte3ds negative;
   float3ds uscale, vscale;
   float3ds uoffset, voffset;
   float3ds rotation;
   tinttype3ds source; /* RGB/RGB Luma Tint/Alpha Tint/RGB Tint */
   fcolor3ds tint1;
   fcolor3ds tint2;
   fcolor3ds redtint;
   fcolor3ds greentint;
   fcolor3ds bluetint;
//   ulong3ds datasize; /* Size of procedural data */
//   void3ds *data;     /* Procedural data */
} bitmap3ds;          /* Bit map definition */

//NOW POINTERS
 
/* Structure to all map settings */
typedef struct {
   bitmap3ds *map;  /* The map settings */
   bitmap3ds *mask; /* The mask settings */
} mapset3ds;

typedef struct {
   bitmap3ds *map; /* The map settings */
   byte3ds 	 useauto; /* True3ds if automatic reflections are being used */
   acubic3ds automap; /* Automatic reflection definitions */
   bitmap3ds *mask; /* The mask settings */
} rmapset3ds;
   
typedef struct {
   char3ds *name;      /* Name */
   fcolor3ds ambient;     /* Ambient light color */
   fcolor3ds diffuse;     /* Diffuse light color */
   fcolor3ds specular;    /* Specular light color */
   float3ds shininess;    /* Shininess factor */
   float3ds shinstrength; /* Shininess strength */
   float3ds blur;         /* Blur factor */
   float3ds transparency; /* Trasparency factor */
   float3ds transfalloff; /* Fallof factor */
   float3ds selfillumpct;     /* Self illumination percentage */
   float3ds wiresize;     /* Width of wireframe */
   shadetype3ds shading;      /* Shading type */
   byte3ds useblur;       /* Blurring flag */
   byte3ds usefall;       /* Transparency falloff flag */
   byte3ds twosided;      /* Two sided material flag */
   byte3ds selfillum;     /* Self illumination flag */
   byte3ds additive;      /* Additive transparency flag */
   byte3ds usewire;       /* Use wireframe rendering */
   byte3ds usewireabs;    /* Wire size is in units, not pixels. */
   byte3ds facemap;       /* Face mapping switch */
   byte3ds soften;        /* Soften switch */
   mapset3ds texture;     /* Texture map settings */
   mapset3ds texture2;    /* Second texture map settings */
   mapset3ds opacity;     /* Opacity map settings */
   mapset3ds bump;        /* Bump map settings */
   mapset3ds specmap;     /* Specularity map settings */
   mapset3ds shinmap;     /* Shininess map settings */
   mapset3ds illummap;    /* Self illumination map settings */
   rmapset3ds reflect;    /* Reflection map settings */
} material3ds;            /* Material definition */

/* mapinfo3ds - Texture map icon placement */
typedef struct {
   ushort3ds maptype;    /* Icon type */
   float3ds tilex;       /* Tiling */
   float3ds tiley;
   float3ds cenx;        /* Position of center */
   float3ds ceny;
   float3ds cenz;
   float3ds scale;       /* Icon scaling factor */
   float3ds matrix[12];  /* Orientation matrix */
   float3ds pw;          /* Planar icon width */
   float3ds ph;          /* Planar icon height */
   float3ds ch;          /* Cylinder icon height */
} mapinfo3ds;            

/* objmat3ds - Material assignments by face */
typedef struct {
   char3ds 	 *name;      /* Material name */
   ushort3ds nfaces;     /* Number of faces using material */
   ushort3ds *faceindex; /* List of faces using material */
} objmat3ds;             

/////////////////////////////////////////////
typedef struct {
   ushort3ds groups;
   ulong3ds *grouplist;
} SmoothGroup;

//list of verts
typedef struct {
   ushort3ds 	numcoords;
   textvert3ds  *textvertlist;
} TexVerts;

typedef struct {
   ushort3ds faces;
   face3ds   *facelist;
} FaceArray;

typedef struct {
   ushort3ds vertices;
   point3ds *pointlist;
} PointArray;

typedef struct {
   char3ds *matname;
   ushort3ds faces;
   ushort3ds *facelist;
} MshMatGroup;

typedef struct {
   float3ds xmatrix[12];
} MeshMatrix;
/////////////////////////////////////////////

/* mesh3ds - Mesh object definition */
typedef struct {
   char3ds *name;   	   /* Object name */
   byte3ds ishidden;       /* Hidden object flag */
//   byte3ds isvislofter;    /* Lofter visibility flag */
//   byte3ds ismatte;        /* Matte object flag */
//   byte3ds isnocast;       /* Doesn't cast shadow's flag */
//   byte3ds isfast;         /* Fast display flag */
//   byte3ds isnorcvshad;    /* Doesn't recieve shadows */
//   byte3ds isfrozen;       /* Frozen object flag */
   ushort3ds nvertices;    /* Vertice count */
   point3ds *vertexarray;  /* List of vertices */
   ushort3ds nvflags;      /* Number of vertex flags */
   ushort3ds *vflagarray;  /* List of vertex flags */
   ushort3ds ntextverts;   /* Number of texture vertices */   
   textvert3ds *textarray; /* List of texture coordinates */
//   byte3ds usemapinfo;     /* Boolean for use of mapping icon information */
//   mapinfo3ds map;         /* Mapping icon info */
   float3ds locmatrix[12]; /* Object orientation matrix */
   ushort3ds nfaces;       /* Face count */
   face3ds *facearray;     /* List of faces */
   ulong3ds *smootharray;  /* Smoothing group assignment list */
//   byte3ds useboxmap;     /* Boolean used to indicate the use of box mapping */
//   char3ds boxmap[6][17];  /* Material names used in boxmapping */
   ubyte3ds meshcolor;     /* UI color assigned to the mesh */
   ushort3ds nmats;        /* Assigned materials count */
   objmat3ds *matarray;    /* Material assignment list */
//   byte3ds useproc;        /* Use animated stand-in flag */
//   ulong3ds procsize;      /* Size of animated stand-in data */
//   char3ds procname[13];   /* Name of animated stand-in procedure */
//   void3ds *procdata;      /* Animated stand-in data */
} mesh3ds;                  

/* conestyle3ds - Spolight projection cone shape. */
typedef enum
{
   Circular,
   Rectangular
} conestyle3ds;

/* spotshadow - Spotlight shadow settings */
typedef struct {
   byte3ds cast; /* True if spotlight casts shadows */
//   shadowstyle3ds type; /* UseShadow or UseRayTrace */
   byte3ds local; /* True if local shadow settings are being used */
   float3ds bias; /* Shadow bias */
   float3ds filter; /* Shadow filter */
   ushort3ds mapsize; /* Shadow map size */
   float3ds raybias; /* Ray tracing shadow bias */
} spotshadow3ds; 

/* spotcond3ds - cone visability settings */
typedef struct {
   conestyle3ds type; /* Circular or rectangular light cone */
   byte3ds show; /* True if cone is visable */
   byte3ds overshoot;  /* True if cone overshoot is on */
} spotcone3ds;

/* spotproject3ds - spotlight projectio bitmap */
typedef struct {
   byte3ds use; /* True if using projector */
   char3ds *bitmap; /* Name of projector bitmap */
} spotprojector3ds;

/* spotlight3ds - spotlight settings */
typedef struct
{
   point3ds target;  /* Spotlight target */
   float3ds hotspot; /* Hotspot angle */
   float3ds falloff; /* Hotspot falloff */
   float3ds roll;    /* Roll angle */
   float3ds aspect;  /* Aspect ratio */
   spotshadow3ds shadows; 
   spotcone3ds cone;
   spotprojector3ds projector;
} spotlight3ds;

/* liteattenuate3ds - Light attenuation settings */
typedef struct {
   byte3ds on; /* True if light attenuation is on */
   float3ds inner; /* Inner range of attenuation */
   float3ds outer; /* Outer range of attenuation */ 
} liteattenuate3ds;

/* light3ds - omni and spotlight settings */
typedef struct {
   char3ds *name;   /* Light name */
   point3ds pos; /* Light position */
   fcolor3ds color; /* Light color */
   float3ds multiplier;/* Light intensity multiplier */
   byte3ds dloff; /* True if light is off */
   liteattenuate3ds attenuation;
//   namelist3ds *exclude;
   spotlight3ds *spot; /* If not null, then struct is a spotlight, else omni. */
} light3ds;

/* camrange3ds - Camera atomosphere ranges */
typedef struct {
   float3ds cam_near; /* Nearest effect radiu */
   float3ds cam_far; /* Farthest effect radius */
} camranges3ds;

typedef struct 
{
   char3ds *name;
   point3ds position;
   point3ds target;
   float3ds roll;
   float3ds focallength;
   float3ds fov;
   byte3ds showcone;
   camranges3ds ranges;
} camera3ds;

typedef enum
{
  LightTarget,
  CameraTarget
} targettype3ds;

/*----------------------------------------
  Constants for commonly used structures
  --------------------------------------*/

/* Flags used by the flag field of the mesh3ds structure */
static const ushort3ds FaceCAVisable3ds = 0x0001;  /* Flags the CA edge as visable */
static const ushort3ds FaceBCVisable3ds = 0x0002;  /* Flags the BC edge as visable */
static const ushort3ds FaceABVisable3ds = 0x0004;  /* Flags the AB edge as visable */
static const ushort3ds FaceUWrap3ds     = 0x0008;  /* Flags the face as being at
						      a texture coord u wrap
						      seam */
static const ushort3ds FaceVWrap3ds     = 0x0010;  /* Flags the face as being at
						      a texture coord v wrap
						      seam */

/* Basic structure default contents for ease of initialization */
static const point3ds DefPoint3ds = {0.0F, 0.0F, 0.0F};
static const textvert3ds DefTextVert3ds = {0.0F, 0.0F};
static const face3ds DefFace3ds = {0, 1, 2, 0};

/* Flags for initflags parameter */
static const ushort3ds InitNoExtras3ds    = 0x0000;
static const ushort3ds InitVertexArray3ds = 0x0001;
static const ushort3ds InitTextArray3ds   = 0x0002;
static const ushort3ds InitFaceArray3ds   = 0x0004;
static const ushort3ds InitMatArray3ds    = 0x0008;
static const ushort3ds InitSmoothArray3ds = 0x0010;
static const ushort3ds InitProcData3ds    = 0x0020;
static const ushort3ds InitVFlagArray3ds  = 0x0040;


/* Smoothing group flags used in the smootharray field of the mesh3ds structure */
static const ulong3ds Smooth01Group3ds = 0x00000001;
static const ulong3ds Smooth02Group3ds = 0x00000002;
static const ulong3ds Smooth03Group3ds = 0x00000004;
static const ulong3ds Smooth04Group3ds = 0x00000008;
static const ulong3ds Smooth05Group3ds = 0x00000010;
static const ulong3ds Smooth06Group3ds = 0x00000020;
static const ulong3ds Smooth07Group3ds = 0x00000030;
static const ulong3ds Smooth08Group3ds = 0x00000080;
static const ulong3ds Smooth09Group3ds = 0x00000100;
static const ulong3ds Smooth10Group3ds = 0x00000200;
static const ulong3ds Smooth11Group3ds = 0x00000400;
static const ulong3ds Smooth12Group3ds = 0x00000800;
static const ulong3ds Smooth13Group3ds = 0x00001000;
static const ulong3ds Smooth14Group3ds = 0x00002000;
static const ulong3ds Smooth15Group3ds = 0x00004000;
static const ulong3ds Smooth16Group3ds = 0x00008000;
static const ulong3ds Smooth17Group3ds = 0x00010000;
static const ulong3ds Smooth18Group3ds = 0x00020000;
static const ulong3ds Smooth19Group3ds = 0x00040000;
static const ulong3ds Smooth20Group3ds = 0x00080000;
static const ulong3ds Smooth21Group3ds = 0x00100000;
static const ulong3ds Smooth22Group3ds = 0x00200000;
static const ulong3ds Smooth23Group3ds = 0x00400000;
static const ulong3ds Smooth24Group3ds = 0x00800000;
static const ulong3ds Smooth25Group3ds = 0x01000000;
static const ulong3ds Smooth26Group3ds = 0x02000000;
static const ulong3ds Smooth27Group3ds = 0x04000000;
static const ulong3ds Smooth28Group3ds = 0x08000000;
static const ulong3ds Smooth29Group3ds = 0x10000000;
static const ulong3ds Smooth30Group3ds = 0x20000000;
static const ulong3ds Smooth31Group3ds = 0x40000000;
static const ulong3ds Smooth32Group3ds = 0x80000000;

/* For the MAT_TILING chunk */
static const ushort3ds TEX_DECAL  = 1;
static const ushort3ds TEX_MIRROR  = 1<<1;
static const ushort3ds TEX_UNUSED1  = 1<<2;
static const ushort3ds TEX_INVERT  = 1<<3;
static const ushort3ds TEX_NOWRAP  = 1<<4;
static const ushort3ds TEX_SAT  = 1<<5;	 /* summed area table */
static const ushort3ds TEX_ALPHA_SOURCE  = 1<<6; /* use ALPHA instead of RGB of map */
static const ushort3ds TEX_TINT  = 1<<7;         /* tint for color */
static const ushort3ds TEX_DONT_USE_ALPHA  = 1<<8;    /* don't use map alpha */
static const ushort3ds TEX_RGB_TINT  = 1<<9;    /* Do RGB color transform */

#endif
