/****************************************************************************
 *	
 * 	C3DStudio.h 				3/9/2000
 *	By Konstantin Khanyants 	kon@unix.temple.edu
 *	
 *	Description: 3D Studio class include file.
 *
 ***************************************************************************/

#ifndef _C3DSTUDIO_

#define _C3DSTUDIO_

class Vector3D;
class RGBColor;
class LightList;
class SurfaceList;
class CameraList;
class MeshListGL;
class ObjectList;
class Aggregate;

class C3DStudio {

public:
	C3DStudio();
	C3DStudio(CString filename);
	~C3DStudio();
	
	int			GetVersion() { return m_Version; }
	RGBColor	GetAmbientLight() { return RGBColor(m_AmbientLight); }
	RGBColor	GetBackColor() { return RGBColor(m_BackColor); }
	CString		GetFilename();
	void		GetLightList(LightList& lights);
	void		GetSurfaceList(SurfaceList& surfaces);
	void		GetCameraList(CameraList& cameras);
	
	void		ConvertMeshData(MeshListGL& meshListGL, ObjectList& objects, SurfaceList& surfaces, Vector3D& autoCenter);

	BOOL		Read3DSFile(CString filename);
	void		DestroyDatabase();
private:
	int Read3DSData(long fileStart, long fileLen, file3ds& theFile);
	int ReadMDATA(long mdataStart, long mdataSize, file3ds& theFile);
//	int ReadDefaultView(long viewStart, long viewSize, file3ds& theFile);
	int ReadMatEntry(long matStart, long matSize, file3ds& theFile);
	int	ReadTextureMap(bitmap3ds*& bitmap, long texStart, long texSize, file3ds& theFile);
	int ReadNamedObject(long namedStart, long namedSize, file3ds& theFile);
	int ReadLight(light3ds*& light, long lightStart, long lightSize, file3ds& theFile);
	int ReadSpotLight(spotlight3ds*& spotLight, long spotStart, long spotSize, file3ds& theFile);
	int ReadCamera(camera3ds*& camera, long camStart, long camSize, file3ds& theFile);
	int ReadTriObject(mesh3ds*& newMesh, long triStart, long triSize, file3ds& theFile);
	int ReadMeshMatrix(file3ds& theFile, float3ds* matrix);
	int ReadSmoothGroup(SmoothGroup& theSmGroup, long chunkSize, file3ds& theFile);
	int	ReadMeshMatGroup(objmat3ds*& meshMat, file3ds& theFile);
	int ReadFaceArray(FaceArray& faceList, file3ds& theFile);
	int ReadPointArray(PointArray& vertexList, file3ds& theFile);
	int ReadTexVerts(TexVerts& theTexVerts, file3ds& theFile);
	
	BOOL FindImageFile(CString imageName, CString& completeName);
	//added 4/4/2000 for getting Ray Tracer polygons from the 3DS database
	Aggregate* CreateAggregateObj(MeshGL* meshGL, mesh3ds* currMesh);
protected:

	CString 					m_Filename;
	SimpleList<material3ds *>	m_MaterialList;
	SimpleList<mesh3ds *>		m_MeshList;
	SimpleList<light3ds *>		m_LightList;
	SimpleList<camera3ds *>		m_CameraList;
	//contains all about current active views
//	viewport3ds				m_ViewPort;
	float					m_Scale;
	unsigned long			m_Version;
	fcolor3ds				m_BackColor;
	fcolor3ds				m_AmbientLight;

};

#endif