/****************************************************************************
 *	
 * 	Camera.h 					3/10/2000
 *	By Konstantin Khanyants 	kon@unix.temple.edu
 *	
 *	Description: A camera class based on the camera model in
 *				 3D Studio.
 * 
 *	Camera structures definition as in 3DSFile.h:
 *	typedef struct {
 *		float3ds cam_near; 
 *		float3ds cam_far;
 *	} camranges3ds;
 *
 *	typedef struct {
 *		char3ds *name;
 *		point3ds position;
 *		point3ds target;
 *		float3ds roll;
 *		float3ds focallength
 *		float3ds fov;
 *		camranges3ds ranges;
 *	} camera3ds;
 *
 *
 ***************************************************************************/

#ifndef _CAMERA_

#define _CAMERA_

class Camera {

public:
	Camera();
	Camera(camera3ds& cam3ds);
	
	void	 ApplyPerspectiveOpenGL(int width, int height);
	void	 ApplyTransformToSceneGL(Vector3D& sceneCenter, BOOL onAuto);
	void	 RotatePos(Matrix& rot_matrix);

	void	 SetCamera3DS(camera3ds& cam3ds);
	CString	 GetName();
	Vector3D GetPos() { return m_Pos; }
	Vector3D GetTarget() { return m_Target; }
	float	 GetRoll() { return m_Roll; }
	float	 GetFocalLength() { return m_Focal_length; }
	float	 GetRangeNear() { return m_Range_near; }
	float	 GetRangeFar() { return m_Range_far; }

	void	 Describe();
protected:
	CString  m_Name;
	Vector3D m_Pos;
	Vector3D m_Target;
	Vector3D m_Up;
	Matrix   m_SceneMatrix;

	float m_Roll;	//in degrees
	float m_Focal_length;	//distance from the viewer to the screen
	float m_Fov;	//in degrees
	float m_Range_near;
	float m_Range_far;

	friend class World;
	friend class C3DStudio;
	friend class CameraList;
	friend class CMaterialDialog;
};


class CameraList : public SimpleList<Camera *> {
public:
	CameraList(): SimpleList<Camera *>() { }
	virtual ~CameraList() { }//DeleteCameras(); }
	Camera* FindCameraByName(CString camName);
	virtual Camera* operator[] (int inx);
	virtual void DeleteCameras();
};

#endif