// ===================================================================
//  File: ImageFile.h					5/1/2000
//	By Konstantin Khanyants
//	Description:
//		Header file for a bitmap image based class. This class 
//		contains functions to open and save in formats like
//		TIF, JPEG, BMP, and Targa. It also has the ability 
//		to generate windows Dibs from the internal image format.
//
// ===================================================================

#ifndef _IMAGEFILE_

#define _IMAGEFILE_

#define HDIB HANDLE

class RGBColor;

const int RGBSample = 3;
const int GRAYSample = 1;

//------------------------------------------------------------
//ImageFile class
//------------------------------------------------------------
class ImageFile {

public:
	ImageFile(BOOL openFile = TRUE);
	~ImageFile();
	virtual BOOL OpenFile();
	virtual BOOL SaveFile();
	virtual BOOL SaveFileAs();
	virtual BOOL OpenFile(CString file_name);
	virtual void SetFileName(char* file_name);
	virtual char* GetFileName();
	virtual HDIB GetImageDib();
	virtual BOOL PutScanline(unsigned char* pBuffer);
	virtual void Reset(int width = 0, int height = 0, int nSamples = 0);
	virtual int Width() { return image_width; }
	virtual int Height() { return image_height; }
	virtual int CurrentRow() { return currnt_row; }
	virtual BOOL ImageLoaded() { return imageLoaded; }
	virtual int SamplesPerPixel() { return SamplePerPixel; }
	virtual RGBColor GetRGBColorPixel(int x, int y);
	virtual void PutRGBColorPixel(RGBColor& col, int x, int y);
	virtual ImageFile* Dup(); 
	virtual unsigned char** GetPixels() { return pImage; }
	virtual void EndCurrentRender() { currnt_row = image_height; }
	virtual BOOL RenderActive() { return currnt_row < image_height; }

private:
	virtual BOOL AllocateImage();
	virtual void DeallocateImage();
	virtual BOOL LoadJpeg();
	virtual BOOL LoadTiff();
	virtual BOOL LoadDib();
	virtual BOOL LoadTga();
	virtual BOOL SaveJpeg(int quality, int mode24);
	virtual BOOL SaveTiff(int mode24);
	virtual BOOL SaveDib();
	virtual BOOL SaveTga(int mode24);
	
private:
	char* filename;
	int image_height;
	int image_width;
	int currnt_row;		//used for adding scanlines from the outside
	int SamplePerPixel;
	BOOL imageLoaded;
	unsigned char** pImage;

};


#endif