// ===================================================================
//  File: list.h						12/26/1999
//	By Konstantin Khanyants				kon@unix.temple.edu
//	Description:
//		Template-based simple linked-list class.
//
//	Original Copyright:     The Object-Oriented Ray Tracer (OORT)
//            				Copyright (C) 1993 by Nicholas Wilt.
//
// ===================================================================


#ifndef _LISTH_

#define _LISTH_

template<class T> class SimpleList {
protected:
    int icount, mcount;

    void ModifyCheck() {
//	if (icount || mcount) 
//	    exit(1);
	}

protected:
    class SimpleListNode {
    public:
	T contents;		//now a reference MODIFIED
	SimpleListNode *next;

	SimpleListNode(const T& Contents, SimpleListNode *Next):
	    contents(Contents), next(Next) { }
    };

    int n;
    SimpleListNode *head;

public:
    SimpleList();
    SimpleList(const T& x);
    SimpleList(const SimpleList& x);
    SimpleList& operator=(const SimpleList& x);
    virtual ~SimpleList();

    virtual void AddToList(const SimpleList<T>& x);
    virtual void AddToList(const T& x);
    virtual int ListCount() const { return n; }
    virtual T ExtractHead();
    virtual int Query(T *passbk) const;
    virtual int Empty() const;
    virtual T *Head() const;
    virtual T *Extract(const T& x);
//    virtual T *operator[] (int inx);
	virtual void AppendToList(const SimpleList<T>& x);
    virtual void AppendToList(const T& x); 
	virtual void Clear();
	
    virtual SimpleListNode *ListHead() const { return head; }
    virtual void ReplaceHead(SimpleListNode *newhead) { head = newhead; }
    virtual int& N() { return n; }

    virtual void IncICount() { icount++; }
    virtual void DecICount() { icount--; }
    virtual void IncMCount() { mcount++; }
    virtual void DecMCount() { mcount--; }

    class Iterator {
    protected:
	SimpleList<T> *list;
	SimpleListNode *ptr;
	int attached;
    public:
	Iterator(SimpleList& x) {
	    attached = 1;
	    list = &x;
	    x.IncICount();
        ptr = x.ListHead();
	}
	
	//test this first
	Iterator(Iterator& iter) {
		ptr = iter.list->ListHead();
		list = iter.list;
		iter.list->IncICount();
		attached = 1;
	}
		
	virtual ~Iterator() { if (attached) list->DecICount(); }
	virtual void Detach() { ptr = 0; list->DecICount(); attached = 0; }
	virtual T Contents() { return (ptr) ? ptr->contents : 0; }
 	virtual int Valid() { return ptr != 0; }
	virtual int GotoNext() { 
	    if (! ptr)
		return -1;
	    ptr = ptr->next;
	    return 0;
	}
    };

    class Manipulator : public Iterator {
    protected:
	SimpleListNode *prev;
    public:
	Manipulator(SimpleList& x): SimpleList<T>::Iterator(x) {
	    x.IncMCount();
	    prev = 0;
	}
	virtual ~Manipulator() { list->DecMCount(); }
	virtual void InsertNode(const T& x) {
	    SimpleListNode *addme = new SimpleListNode(x, ptr);
	    prev->next = addme;	    	
	    list->N() += 1;
	}
	virtual void DeleteNode() {
	    if (prev) {
		if (ptr) {
		    prev->next = ptr->next;
	    	    delete ptr;
		}
	    	ptr = prev->next;
	    }
	    else {
		list->ReplaceHead((ptr) ? ptr->next : 0);
		if (ptr)
		    delete ptr;
		ptr = list->ListHead();
	    }
	    list->N() -= 1;
	}
	virtual void ReplaceContents(const T& x) {
	    if (ptr)
		ptr->contents = x;
	}
	virtual int GotoNext() {
	    if (ptr)
		prev = ptr;
	    return Iterator::GotoNext();
	}
    };

};

template<class T>
SimpleList<T>::SimpleList()
{
    head = 0;
    n = 0;
    icount = mcount = 0;
}

template<class T>
SimpleList<T>::SimpleList(const T& x)
{
    head = new SimpleListNode(x, 0);
    n = 1;
    icount = mcount = 0;
}

template<class T>
SimpleList<T>::SimpleList(const SimpleList<T>& x)
{
    head = 0;
    n = 0;
    icount = mcount = 0;
    for (SimpleListNode *sc = x.head; sc; sc = sc->next)
	AddToList(sc->contents);
}

template<class T>
SimpleList<T>&
SimpleList<T>::operator=(const SimpleList<T>& x)
{
    SimpleListNode *sc;
    ModifyCheck();
    for (sc = head; sc; ) {
	SimpleListNode *temp = sc->next;
	delete sc;
	sc = temp;
    }
    head = 0;
    n = 0;
    for (sc = x.head; sc; sc = sc->next)
	AddToList(sc->contents);

    return *this;
}

template<class T>
SimpleList<T>::~SimpleList() 
{
    ModifyCheck();
    SimpleListNode *sc = head;
    while (sc) {
	SimpleListNode *temp = sc->next;
	delete sc;
	sc = temp;
    }
}

template<class T> 
void
SimpleList<T>::AddToList(const SimpleList<T>& x) 
{
    ModifyCheck();
    for (SimpleListNode *sc = x.head; sc; sc = sc->next)
	AddToList(sc->contents);
}

template<class T>
void
SimpleList<T>::AddToList(const T& x)
{
    ModifyCheck();
    head = new SimpleListNode(x, head);
    n++;
}

////ADDED///////
template<class T>	//6/12/2000
void
SimpleList<T>::AppendToList(const SimpleList<T>& x) 
{
    ModifyCheck();
    for (SimpleListNode *sc = x.head; sc; sc = sc->next)
	AppendToList(sc->contents);
}

template<class T>
void
SimpleList<T>::AppendToList(const T& x) 
{
	//when appending the next field is zero
	SimpleListNode *addme = new SimpleListNode(x, 0);
	n++;
	//if there are elements in the list
	if (head) {
		SimpleListNode *temp = head;
		while (temp->next)	//while the next pointer exists go to next
			temp = temp->next; 
		temp->next = addme;
	}
	//if there are no elements in the list
	else 
		head = addme;
}

template<class T>
T
SimpleList<T>::ExtractHead()
{
    T ret;
    ModifyCheck();
    if (head) {
	SimpleListNode *temp = head->next;
	ret = head->contents;
	delete head;
	head = temp;
	n--;
    }
    return ret;
}

template<class T> 
int
SimpleList<T>::Query(T *passbk) const
{
    for (SimpleListNode *sc = head; sc; sc = sc->next) {
	if (sc->contents == *passbk) {
	    *passbk = sc->contents;
	    return 1;
	}
    }
    return 0;
}

template<class T>
int
SimpleList<T>::Empty() const
{
    return head == 0;
}

template<class T>
T *
SimpleList<T>::Head() const
{
    return (head) ? &head->contents : 0;
}

template<class T> 
T *
SimpleList<T>::Extract(const T& x) 
{
    SimpleListNode *sc;
    ModifyCheck();
    for (sc = head; sc; sc = sc->next) {
		if (sc->contents == x)
            return &sc->contents;
    }
    return 0;
}

////////ADDED///////////////////////////////////////////////////
/*template<class T>
T *
SimpleList<T>::operator[] (int inx) 
{
	ModifyCheck();
    if (inx >= n || inx < 0)
        return 0;
    SimpleListNode *pNode = head;
	if (head) {
		int i = 0;
		while (i != inx) {
    	 	pNode = pNode->next;
			i++;
		}
  	  	return &pNode->contents;
	}
	return 0;
}
*/
template <class T>
void 
SimpleList<T>::Clear()
{
 //   ModifyCheck();
    while (head) {
		SimpleListNode *temp = head->next;
		delete head;
		head = temp;
    }
	n = 0;
	head = 0;
}

#endif