// ===================================================================
//  File: vector2D.h					10/1/2000
//	BY Konstantin Khanyants				kon@unix.temple.edu
//	Description:
//		Header file for Vector2D class in OORT.
//
// ===================================================================

#ifndef _VECTOR2D_

#define _VECTOR2D_


class Vector3D;
class Matrix;

class Vector2D {

public:
    float x, y;

    // constructors
    Vector2D() { }
    Vector2D(float X, float Y) { x = X; y = Y; }
    Vector2D(float X) { x = y = X; }
    Vector2D(const Vector2D& v) { x = v.x; y = v.y; }
	Vector2D(const Vector3D& v);

    // Functions to get at the vector components
    float& operator[] (int inx) {
	if (inx == 0)      return x;
	else	       	   return y;
    }
    const float& operator[] (int inx) const {
	if (inx == 0)      return x;
	else	       	   return y;
    }

    friend float Magnitude(const Vector2D& v);
    friend Vector2D Normalize(const Vector2D& A);

    // Rotate a normal vector.
//    friend Vector2D PlaneRotate(const Matrix&, const Vector2D&);

    // Unary operators
    Vector2D operator+ () const;
    Vector2D operator- () const;

    // Assignment operators
    Vector2D& operator+= (const Vector2D& A);
    Vector2D& operator-= (const Vector2D& A);
    Vector2D& operator*= (const Vector2D& A);
    Vector2D& operator*= (float A);
    Vector2D& operator/= (float A);
	Vector2D& operator=	 (TexPointGL& texVert);

    // Binary operators
    friend Vector2D operator+ (const Vector2D& A, const Vector2D& B);
    friend Vector2D operator- (const Vector2D& A, const Vector2D& B);
    friend Vector2D operator* (const Vector2D& A, const Vector2D& B);
    friend Vector2D operator* (const Vector2D& A, float B);
    friend Vector2D operator* (float A, const Vector2D& B);
    friend Vector2D operator/ (const Vector2D& A, const Vector2D& B);

    friend int operator< (const Vector2D& A, const Vector2D& B);

    friend Vector2D operator/ (const Vector2D& A, float B);
    friend int operator== (const Vector2D& A, const Vector2D& B);
    friend float DotProd(const Vector2D& A, const Vector2D& B);

    friend void Minimize(Vector2D& min, const Vector2D& Candidate);
    friend void Maximize(Vector2D& max, const Vector2D& Candidate);
};


inline float Magnitude(const Vector2D& v) 
{
    return (float) sqrt(v.x*v.x + v.y*v.y);
}

inline Vector2D Normalize(const Vector2D& A)
{
    return A / Magnitude(A);
}

inline float DotProd (const Vector2D& A, const Vector2D& B)
{
    return A.x * B.x + A.y * B.y;
}

inline Vector2D Vector2D::operator+ () const
{
    return *this;
}

inline Vector2D Vector2D::operator- () const
{
    return Vector2D(-x, -y);
}

inline Vector2D& Vector2D::operator+= (const Vector2D& A)
{
    x += A.x;  y += A.y;
    return *this;
}

inline Vector2D& Vector2D::operator-= (const Vector2D& A)
{
    x -= A.x;  y -= A.y;
    return *this;
}

inline Vector2D& Vector2D::operator*= (float A)
{
    x *= A;  y *= A;
    return *this;
}

inline Vector2D& Vector2D::operator/= (float A)
{
    x /= A;  y /= A;
    return *this;
}

inline Vector2D& Vector2D::operator*= (const Vector2D& A)
{
    x *= A.x;  y *= A.y;
    return *this;
}

inline Vector2D operator+ (const Vector2D& A, const Vector2D& B)
{
    return Vector2D(A.x + B.x, A.y + B.y);
}

inline Vector2D operator- (const Vector2D& A, const Vector2D& B)
{
    return Vector2D(A.x - B.x, A.y - B.y);
}

inline Vector2D operator* (const Vector2D& A, const Vector2D& B)
{
    return Vector2D(A.x * B.x, A.y * B.y);
}

inline Vector2D operator* (const Vector2D& A, float B)
{
    return Vector2D(A.x * B, A.y * B);
}

inline Vector2D operator* (float A, const Vector2D& B)
{
    return B * A;
}

inline Vector2D operator/ (const Vector2D& A, const Vector2D& B)
{
    return Vector2D(A.x / B.x, A.y / B.y);
}

inline Vector2D operator/ (const Vector2D& A, float B)
{
    return Vector2D(A.x / B, A.y / B);
}

inline int operator< (const Vector2D& A, const Vector2D& B)
{
    return A.x < B.x && A.y < B.y;
}

// Might replace floating-point == with comparisons of
// magnitudes, if needed.
inline int operator== (const Vector2D& A, const Vector2D& B)
{
    return (A.x == B.x) && (A.y == B.y);
}

#endif
